(function($) {
	var createdElements = [];
	var defaults        = {
		options: {
			prependExistingHelpBlock: false,
			sniffHtml: true,
			preventSubmit: true,
			submitError: false,
			submitSuccess: false,
			semanticallyStrict: false,
			autoAdd: {
				helpBlocks: true
			},
			filter: function() {
				if ( $( this ).parents( '.control-group' ).length < 1) {
					return false;
				}
				if ( $( this ).hasClass('arf_color_picker_input') ){
					return $( this ).parents('.arf_field_type_colorpicker').is(':visible');
				}	
				if ( $( this ).hasClass( 'arf_star_rating_input' ) ) {
					return $( this ).parents( '.arf_field_type_scale' ).is( ':visible' );
				}
				if ( $( this ).attr( 'id' ) == 'arf_google_recaptcha_response' ) {
					return $( this ).parent().is( ":visible" );
				}
				isMaterialForm = $( this ).parents( '.arf_materialize_form' ).length > 0;
				isMaterialOutlineForm = $( this ).parents( '.arf_material_outline_form').length > 0;
				
				if ( ( isMaterialForm || isMaterialOutlineForm ) && $( this ).hasClass( 'arf-selectpicker-input-control' ) ) {
					if ( ! $( this ).is( ':visible' ) && $( this ).parents( '.control-group' ).is( ':visible' ) ) {
						return $( this ).parents( '.control-group' ).is( ':visible' );
					}
				}

				if( $( this ).hasClass('arf_signature_output_input') ){
					return $( this ).parent().is( ":visible" );	
				}
				
				return $( this ).is( ":visible" );
			}
		},
		methods: {
			init: function(options) {
				var settings         = $.extend( true, {}, defaults );
				settings.options     = $.extend( true, settings.options, options );
				var $siblingElements = this;
				var uniqueForms      = $.uniqueSort(
					$siblingElements.map(
						function() {
							return $( this ).parents( "form" )[0];
						}
					).toArray()
				);

				$( uniqueForms ).on(
					"submit",
					function(e) {
						var $form = $( this );

						window.arf_is_submitting_form = true;
						var warningsFound             = 0;
						var $inputs                   = $form.find( "input,textarea,select" ).not( "[type=submit],[type=image],[data-jqvalidate='false'],[disabled='disabled']" ).filter( settings.options.filter );
						$inputs.each(
							function(i, el) {
								var $this         = $( el );
								var $controlGroup = $this.parents( ".control-group" ).first();
								var $helpBlock    = $controlGroup.find( ".help-block" ).first();
								$this.trigger( 'change' );
							}
						);
						$inputs.trigger( "submit.validation" ).trigger( "validationLostFocus.validation" );
						$inputs.each(
							function(i, el) {
								var $this     = $( el ),
								$controlGroup = $this.parents( ".control-group" ).first();
								if ($controlGroup.hasClass( "arf_warning" )) {
									$controlGroup.removeClass( "arf_warning" ).addClass( "arf_error" );
									warningsFound++;
								}
							}
						);
						$inputs.trigger( "validationLostFocus.validation" );
						if (warningsFound) {
							if (settings.options.preventSubmit) {
								e.preventDefault();
							}
							$form.addClass( "arf_error" );
							if ( typeof settings.options.submitError == 'function' ) {
								settings.options.submitError( $form, e, $inputs.jqBootstrapValidation( "collectErrors", true ) );
							}
						} else {
							$form.removeClass( "arf_error" );
							if ( typeof settings.options.submitSuccess == 'function' ) {
								settings.options.submitSuccess( $form, e );
							}
						}
					}
				);
				return this.each(
					function() {
						var $this = $( this );

						if ( typeof $this.attr( 'data-jqvalidate' ) != 'undefined' && $this.attr( 'data-jqvalidate' ) == "false" ) {
							return;
						}
						var $controlGroup  = $this.parents( ".control-group" ).first();
						var $helpBlock     = $controlGroup.find( ".help-block" ).first();
						var $form          = $this.parents( "form" ).first();
						var validatorNames = [];

						if ( ! $helpBlock.length && settings.options.autoAdd && settings.options.autoAdd.helpBlocks) {
							$helpBlock = $( '<div class="help-block" />' );
							$controlGroup.find( '.controls' ).append( $helpBlock );
							createdElements.push( $helpBlock[0] );
						}

						if (settings.options.sniffHtml) {
							var message = "";
							if ($this.attr( "pattern" ) !== undefined) {
								message = "Not in the expected format<!-- data-validation-pattern-message to override -->";
								if ($this.data( "validationPatternMessage" )) {
									message = $this.data( "validationPatternMessage" );
								}
								$this.data( "validationPatternMessage", message );
								$this.data( "validationPatternRegex", $this.attr( "pattern" ) );
							}
							if ($this.attr( "max" ) !== undefined || $this.attr( "aria-valuemax" ) !== undefined) {
								var max = ($this.attr( "max" ) !== undefined ? $this.attr( "max" ) : $this.attr( "aria-valuemax" ));
								message = "Too high: Maximum of '" + max + "'<!-- data-validation-max-message to override -->";
								if ($this.data( "validationMaxMessage" )) {
									message = $this.data( "validationMaxMessage" );
								}
								$this.data( "validationMaxMessage", message );
								$this.data( "validationMaxMax", max );
							}
							if ($this.attr( "min" ) !== undefined || $this.attr( "aria-valuemin" ) !== undefined) {
								var min = ($this.attr( "min" ) !== undefined ? $this.attr( "min" ) : $this.attr( "aria-valuemin" ));
								message = "Too low: Minimum of '" + min + "'<!-- data-validation-min-message to override -->";
								if ($this.data( "validationMinMessage" )) {
									message = $this.data( "validationMinMessage" );
								}
								$this.data( "validationMinMessage", message );
								$this.data( "validationMinMin", min );
							}
							if ($this.attr( "maxlength" ) !== undefined) {
								message = "Too long: Maximum of '" + $this.attr( "maxlength" ) + "' characters<!-- data-validation-maxlength-message to override -->";
								if ($this.data( "validationMaxlengthMessage" )) {
									message = $this.data( "validationMaxlengthMessage" );
								}
								$this.data( "validationMaxlengthMessage", message );
								$this.data( "validationMaxlengthMaxlength", $this.attr( "maxlength" ) );
							}
							if ($this.attr( "minlength" ) !== undefined) {
								message = "Too short: Minimum of '" + $this.attr( "minlength" ) + "' characters<!-- data-validation-minlength-message to override -->";
								if ($this.data( "validationMinlengthMessage" )) {
									message = $this.data( "validationMinlengthMessage" );
								}
								$this.data( "validationMinlengthMessage", message );
								$this.data( "validationMinlengthMinlength", $this.attr( "minlength" ) );
							}
							if ($this.attr( "required" ) !== undefined || $this.attr( "aria-required" ) !== undefined) {
								message = settings.builtInValidators.required.message;
								if ($this.data( "validationRequiredMessage" )) {
									message = $this.data( "validationRequiredMessage" );
								}
								$this.data( "validationRequiredMessage", message );
							}
							if ($this.attr( "type" ) !== undefined && $this.attr( "type" ).toLowerCase() === "number") {
								message = settings.builtInValidators.number.message;
								if ($this.data( "validationNumberMessage" )) {
									message = $this.data( "validationNumberMessage" );
								}
								$this.data( "validationNumberMessage", message );
							}
							if ($this.attr( "type" ) !== undefined && $this.attr( "type" ).toLowerCase() === "email") {
								message = "Not a valid email address<!-- data-validator-validemail-message to override -->";
								if ($this.data( "validationValidemailMessage" )) {
									message = $this.data( "validationValidemailMessage" );
								} else if ($this.data( "validationEmailMessage" )) {
									message = $this.data( "validationEmailMessage" );
								}
								$this.data( "validationValidemailMessage", message );
							}
							if ($this.attr( "minchecked" ) !== undefined) {
								message = "Not enough options checked; Minimum of '" + $this.attr( "minchecked" ) + "' required<!-- data-validation-minchecked-message to override -->";
								if ($this.data( "validationMincheckedMessage" )) {
									message = $this.data( "validationMincheckedMessage" );
								}
								$this.data( "validationMincheckedMessage", message );
								$this.data( "validationMincheckedMinchecked", $this.attr( "minchecked" ) );
							}
							if ($this.attr( "maxchecked" ) !== undefined) {
								message = "Too many options checked; Maximum of '" + $this.attr( "maxchecked" ) + "' required<!-- data-validation-maxchecked-message to override -->";
								if ($this.data( "validationMaxcheckedMessage" )) {
									message = $this.data( "validationMaxcheckedMessage" );
								}
								$this.data( "validationMaxcheckedMessage", message );
								$this.data( "validationMaxcheckedMaxchecked", $this.attr( "maxchecked" ) );
							}
							if ($this.attr( "maxselected" ) !== undefined) {
								message = "Too many options selected; Maximum of '" + $this.attr( "maxselected" ) + "' required<!-- data-validation-maxselected-message to override -->";
								if ($this.data( "validationMaxselectedMessage" )) {
									message = $this.data( "validationMaxselectedMessage" );
								}
								$this.data( "validationMaxselectedMessage", message );
								$this.data( "validationMaxselectedMaxselected", $this.attr( "maxselected" ) );
							}
							if ($this.attr( "minselected" ) !== undefined) {
								message = "Not enough options selected; Minimum of '" + $this.attr( "minselected" ) + "' required<!-- data-validation-minselected-message to override -->";
								if ($this.data( "validationMinselectedMessage" )) {
									message = $this.data( "validationMinselectedMessage" );
								}
								$this.data( "validationMinselectedMessage", message );
								$this.data( "validationMinselectedMinselected", $this.attr( "minselected" ) );
							}
						}

						if ($this.data( "validation" ) !== undefined) {
							validatorNames = $this.data( "validation" ).split( "," );
						}
						$.each(
							$this.data(),
							function(i, el) {
								var parts = i.replace( /([A-Z])/g, ",$1" ).split( "," );
								if (parts[0] === "validation" && parts[1]) {
									validatorNames.push( parts[1] );
								}
							}
						);
						var validatorNamesToInspect    = validatorNames;
						var newValidatorNamesToInspect = [];
						do {
							$.each(
								validatorNames,
								function(i, el) {
									validatorNames[i] = formatValidatorName( el );
								}
							);
							validatorNames             = $.uniqueSort( validatorNames );
							newValidatorNamesToInspect = [];
							$.each(
								validatorNamesToInspect,
								function(i, el) {
									if ($this.data( "validation" + el + "Shortcut" ) !== undefined) {
										$.each(
											$this.data( "validation" + el + "Shortcut" ).split( "," ),
											function(i2, el2) {
												newValidatorNamesToInspect.push( el2 );
											}
										);
									} else if (settings.builtInValidators[el.toLowerCase()]) {
										var validator = settings.builtInValidators[el.toLowerCase()];
										if (validator.type.toLowerCase() === "shortcut") {
											$.each(
												validator.shortcut.split( "," ),
												function(i, el) {
													el = formatValidatorName( el );
													newValidatorNamesToInspect.push( el );
													validatorNames.push( el );
												}
											);
										}
									}
								}
							);
							validatorNamesToInspect = newValidatorNamesToInspect;
						} while (validatorNamesToInspect.length > 0) {
							var validators = {};
						}
						$.each(
							validatorNames,
							function(i, el) {
								var message            = $this.data( "validation" + el + "Message" );
								var hasOverrideMessage = (message !== undefined);
								var foundValidator     = false;
								message = (message ? message : "This field cannot be blank. <!-- Add attribute 'data-validation-" + el.toLowerCase() + "-message' to input to change this message -->");

								$.each(
									settings.validatorTypes,
									function(validatorType, validatorTemplate) {
										if (validators[validatorType] === undefined) {
											validators[validatorType] = [];
										}
										if ( ! foundValidator && $this.data( "validation" + el + formatValidatorName( validatorTemplate.name ) ) !== undefined) {
											validators[validatorType].push(
												$.extend(
													true,
													{
														name: formatValidatorName( validatorTemplate.name ),
														message: message
													},
													validatorTemplate.init( $this, el )
												)
											);
											foundValidator = true;
										}
									}
								);

								if ( ! foundValidator && settings.builtInValidators[el.toLowerCase()]) {
									var validator = $.extend( true, {}, settings.builtInValidators[el.toLowerCase()] );
									if (hasOverrideMessage) {
										validator.message = message;
									}
									var validatorType = validator.type.toLowerCase();
									if (validatorType === "shortcut") {
										foundValidator = true;
									} else {
										$.each(
											settings.validatorTypes,
											function(validatorTemplateType, validatorTemplate) {
												if (validators[validatorTemplateType] === undefined) {
													validators[validatorTemplateType] = [];
												}
												if ( ! foundValidator && validatorType === validatorTemplateType.toLowerCase()) {
													$this.data( "validation" + el + formatValidatorName( validatorTemplate.name ), validator[validatorTemplate.name.toLowerCase()] );
													validators[validatorType].push( $.extend( validator, validatorTemplate.init( $this, el ) ) );
													foundValidator = true;
												}
											}
										);
									}
								}
								if ( ! foundValidator) {
									$.error( "Cannot find validation info for '" + el + "'" );
								}
							}
						);
						$helpBlock.data( "original-contents", ($helpBlock.data( "original-contents" ) ? $helpBlock.data( "original-contents" ) : $helpBlock.html()) );
						$helpBlock.data( "original-role", ($helpBlock.data( "original-role" ) ? $helpBlock.data( "original-role" ) : $helpBlock.attr( "role" )) );
						$controlGroup.data( "original-classes", ($controlGroup.data( "original-clases" ) ? $controlGroup.data( "original-classes" ) : $controlGroup.attr( "class" )) );
						$this.data( "original-aria-invalid", ($this.data( "original-aria-invalid" ) ? $this.data( "original-aria-invalid" ) : $this.attr( "aria-invalid" )) );

						$this.on(
							"validation.validation",
							function(event, params) {
								var value       = getValue( $this );
								var errorsFound = [];
								if (value == null) {
									value = ''; }
								$.each(
									validators,
									function(validatorType, validatorTypeArray) {
										if ( typeof value != 'undefined' && (value || value.length || (params && params.includeEmpty) || ( ! ! settings.validatorTypes[validatorType].blockSubmit && params && ! ! params.submitting))) {
											$.each(
												validatorTypeArray,
												function(i, validator) {
													if (settings.validatorTypes[validatorType].validate( $this, value, validator )) {
														errorsFound.push( validator.message );
													}
												}
											);
										}
									}
								);
								return errorsFound;
							}
						);
						$this.on(
							"getValidators.validation",
							function() {
								return validators;
							}
						);
						$this.on(
							"submit.validation",
							function() {
								return $this.triggerHandler(
									"change.validation",
									{
										submitting: true
									}
								);
							}
						);
						$this.on(
							["click","change","blur"].join( ".validation " ) + ".validation",
							function(e, params) {
								var value       = getValue( $this );
								var errorsFound = [];

								var isARFormsLite = jQuery( $this.closest( 'form' )[0] ).hasClass( 'arfliteshowmainform' ) || false;

								$controlGroup.find( "input,textarea,select" ).each(
									function(i, el) {
										var oldCount = errorsFound.length;
										if ( (e.type == 'focusout' && $this.attr( 'type' ) == 'tel') || (typeof $this.attr( 'data-mask' ) != 'undefined') ) {
											if( typeof $this.attr( 'data-mask' ) != 'undefined' && (e.type == 'blur' || window.arf_is_submitting_form) ){
												$.each(
													$( el ).triggerHandler( "validation.validation", params ),
													function(j, message) {
														errorsFound.push( message );
													}
												);
											} else {
												setTimeout(
													function(){
														$.each(
															$( el ).triggerHandler( "validation.validation", params ),
															function(j, message) {
																errorsFound.push( message );
															}
														);
													}
												);
											}
										} else {
											$.each(
												$( el ).triggerHandler( "validation.validation", params ),
												function(j, message) {
													errorsFound.push( message );
												}
											);
											if ( $( el ).hasClass( 'arf_phone_utils' ) ) {
												var fkey = 'field_' + el.id;
												if ( typeof window.phone_fields[fkey] != 'undefined' && el.value != '' && ! window.phone_fields[fkey].isValidNumber() ) {
													if ( typeof el.getAttribute( 'data-do-validation' ) != 'undefined' && el.getAttribute( 'data-do-validation' ) != null && el.getAttribute( 'data-do-validation' ) == 'true' ) {
														var msg = $( el ).attr( 'data-invalid-format-message' ) ? $( el ).attr( 'data-invalid-format-message' ) : 'Invalid Phone Number';
														errorsFound.push( msg );
													}
												}
											}
										}
										if (errorsFound.length > oldCount) {
											$( el ).attr( "aria-invalid", "true" );
										} else {
											var original = $this.data( "original-aria-invalid" );
											$( el ).attr( "aria-invalid", (original !== undefined ? original : false) );
											var form_id    = $this.closest( 'form' ).find( '[data-id="form_id"]' ).val();
											if( isARFormsLite ){
												var error_type = (jQuery( '[data-id="arflite_form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
											} else {
												var error_type = (jQuery( '[data-id="form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
											}
											if (error_type == 'advance') {
												if ($controlGroup.find( 'input,select,textarea' ).attr( 'aria-invalid' ) == 'true') {
													$controlGroup.find( '.popover' ).hide();
												} else {
													$controlGroup.find( '.popover' ).remove();
												}
											}
										}
									}
								);
								errorsFound = $.uniqueSort( errorsFound.sort() );

								if (errorsFound.length) {
									$controlGroup.removeClass( "arf_success arf_error" ).addClass( "arf_warning" );
									if (settings.options.semanticallyStrict && errorsFound.length === 1) {
										$helpBlock.html( errorsFound[0] + (settings.options.prependExistingHelpBlock ? $helpBlock.data( "original-contents" ) : "") );
									} else {

										var form_id = $this.closest( 'form' ).find( '[data-id="form_id"]' ).val();
										if ( isARFormsLite ) {
											var error_type = (jQuery( '[data-id="arflite_form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
										} else {
											var error_type = (jQuery( '[data-id="form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
										}

										if (error_type == 'advance') {
											if ( isARFormsLite ) {
												arflite_show_tooltip( $controlGroup, $helpBlock, '' + errorsFound + '' );
											} else {
												arf_show_tooltip( $controlGroup, $helpBlock, '' + errorsFound + '' );
											}
										} else {
											$helpBlock.removeClass( 'arfanimated bounceInDownNor' );
											if ( isARFormsLite ) {
												arflite_show_tooltip_destroy( $controlGroup, $helpBlock, '' + errorsFound + '' );
											} else {
												arf_show_tooltip_destroy( $controlGroup, $helpBlock, '' + errorsFound + '' );
											}
											$helpBlock.addClass( 'arfanimated bounceInDownNor' );
											$helpBlock.html( "<ul role=\"alert\"><li>" + errorsFound.join( "</li><li>" ) + "</li></ul>" + (settings.options.prependExistingHelpBlock ? $helpBlock.data( "original-contents" ) : "") ).show();
										}
									}
								} else {
									var form_id = $this.closest( 'form' ).find( '[data-id="form_id"]' ).val();

									if ( isARFormsLite ) {
										var error_type = (jQuery( '[data-id="arflite_form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
									} else {
										var error_type = (jQuery( '[data-id="form_tooltip_error_' + form_id + '"]' ).val() == 'advance') ? 'advance' : 'normal';
									}

									if (error_type == 'advance') {
										if ( ! $controlGroup.hasClass( 'error' ) && ! $controlGroup.hasClass( 'arf_warning' )) {
											if ($controlGroup.find( 'input,select,textarea' ).attr( 'aria-invalid' ) == 'true') {
												$controlGroup.find( '.popover' ).hide();
											} else {
												$controlGroup.find( '.popover' ).remove();
											}
										}
									}
									$controlGroup.removeClass( "arf_warning arf_error arf_success" );
									if (value != null && value.length > 0) {
										$controlGroup.addClass( "arf_success" );
									}
									$helpBlock.html( $helpBlock.data( "original-contents" ) ).removeClass( 'arfanimated bounceInDownNor' );
								}
								if (e.type === "blur") {
									$controlGroup.removeClass( "arf_success" );
								}
							}
						);
						$this.on(
							"validationLostFocus.validation",
							function() {
								$controlGroup.removeClass( "arf_success" );
							}
						);
					}
				);
			},
			destroy: function() {
				return this.each(
					function() {
						var $this     = $( this ),
						$controlGroup = $this.parents( ".control-group" ).first(),
						$helpBlock    = $controlGroup.find( ".help-block" ).first();

						 $( this ).off( "submit" );

						$this.off( '.validation' );
						$helpBlock.html( $helpBlock.data( "original-contents" ) );
						$controlGroup.attr( "class", $controlGroup.data( "original-classes" ) );
						$this.attr( "aria-invalid", $this.data( "original-aria-invalid" ) );
						$helpBlock.attr( "role", $this.data( "original-role" ) );
						if (createdElements.indexOf( $helpBlock[0] ) > -1) {
							$helpBlock.remove();
						}
					}
				);
			},
			collectErrors: function(includeEmpty) {
				var errorMessages = {};
				this.each(
					function(i, el) {
						var $el             = $( el );
						var name            = $el.attr( "name" );
						var errors          = $el.triggerHandler(
							"validation.validation",
							{
								includeEmpty: true
							}
						);
						errorMessages[name] = $.extend( true, errors, errorMessages[name] );
					}
				);

				$.each(
					errorMessages,
					function(i, el) {
						if (el.length === 0) {
							delete errorMessages[i];
						}
					}
				);
				return errorMessages;
			},
			hasErrors: function() {
				var errorMessages = [];
				this.each(
					function(i, el) {
						errorMessages = errorMessages.concat(
							$( el ).triggerHandler( "getValidators.validation" ) ? $( el ).triggerHandler(
								"validation.validation",
								{
									submitting: true
								}
							) : []
						);
					}
				);
				return (errorMessages.length > 0);
			},
			override: function(newDefaults) {
				defaults = $.extend( true, defaults, newDefaults );
			}
		},
		validatorTypes: {
			callback: {
				name: "callback",
				init: function($this, name) {
					return {
						validatorName: name,
						callback: $this.data( "validation" + name + "Callback" ),
						lastValue: $this.val(),
						lastValid: true,
						lastFinished: true
					};
				},
				validate: function($this, value, validator) {
					if (validator.lastValue === value && validator.lastFinished) {
						return ! validator.lastValid;
					}
					if (validator.lastFinished === true) {
						validator.lastValue    = value;
						validator.lastValid    = true;
						validator.lastFinished = false;
						var rrjqbvValidator    = validator;
						var rrjqbvThis         = $this;
						executeFunctionByName(
							validator.callback,
							window,
							$this,
							value,
							function(data) {
								if (rrjqbvValidator.lastValue === data.value) {
									rrjqbvValidator.lastValid = data.valid;
									if (data.message) {
										rrjqbvValidator.message = data.message;
									}
									rrjqbvValidator.lastFinished = true;
									rrjqbvThis.data( "validation" + rrjqbvValidator.validatorName + "Message", rrjqbvValidator.message );
									setTimeout(
										function() {
											rrjqbvThis.trigger( "change.validation" );
										},
										1
									);
								}
							}
						);
					}
					return false;
				}
			},
			ajax: {
				name: "ajax",
				init: function($this, name) {
					return {
						validatorName: name,
						url: $this.data( "validation" + name + "Ajax" ),
						lastValue: $this.val(),
						lastValid: true,
						lastFinished: true
					};
				},
				validate: function($this, value, validator) {
					if ("" + validator.lastValue === "" + value && validator.lastFinished === true) {
						return validator.lastValid === false;
					}
					if (validator.lastFinished === true) {
						validator.lastValue    = value;
						validator.lastValid    = true;
						validator.lastFinished = false;
						$.ajax(
							{
								url: validator.url,
								data: "value=" + value + "&field=" + $this.attr( "name" ),
								dataType: "json",
								success: function(data) {
									if ("" + validator.lastValue === "" + data.value) {
										validator.lastValid = ! ! (data.valid);
										if (data.message) {
											validator.message = data.message;
										}
										validator.lastFinished = true;
										$this.data( "validation" + validator.validatorName + "Message", validator.message );
										setTimeout(
											function() {
												$this.trigger( "change.validation" );
											},
											1
										);
									}
								},
								failure: function() {
									validator.lastValid    = true;
									validator.message      = "ajax call failed";
									validator.lastFinished = true;
									$this.data( "validation" + validator.validatorName + "Message", validator.message );
									setTimeout(
										function() {
											$this.trigger( "change.validation" );
										},
										1
									);
								}
							}
						);
					}
					return false;
				}
			},
			regex: {
				name: "regex",
				init: function($this, name) {
					return {
						regex: regexFromString( $this.data( "validation" + name + "Regex" ) )
					};
				},
				validate: function($this, value, validator) {
					return ( ! validator.regex.test( value ) && ! validator.negative) || (validator.regex.test( value ) && validator.negative);
				}
			},
			required: {
				name: "required",
				init: function($this, name) {
					return {};
				},
				validate: function($this, value, validator) {
					
					if( $this.hasClass('arf_autocomplete_check_option') && (value !== '' || value !== null ) ){
						var val = value;

					    if ($this.attr('data-source') == undefined) {
					        var data_id = $this.attr('data-id');
					        var option = jQuery('input[name="item_meta[' + data_id + ']"]').attr('data-sep-labels');
					    } else {
					        var option = $this.attr('data-source');
					    }    

					    var option = JSON.parse(option);
					    var option = Object.values(option);

					    var find_value = option.indexOf(val);

					    if (find_value == -1) {
					        $this.val('');
						    value = '';
					    }
					}
					return ! ! (value.length === 0 && ! validator.negative) || ! ! (value.length > 0 && validator.negative);
				},
				blockSubmit: true
			},
			match: {
				name: "match",
				init: function($this, name) {
					var element = $this.parents( "form" ).first().find( "[name=\"" + $this.data( "validation" + name + "Match" ) + "\"]" ).first();
					element.on(
						"validation.validation",
						function() {
							if ($this.val() != '') {
								$this.trigger(
									"change.validation",
									{
										submitting: true
									}
								);
							}
						}
					);
					return {
						"element": element
					};
				},
				validate: function($this, value, validator) {
					return (value !== validator.element.val() && ! validator.negative) || (value === validator.element.val() && validator.negative);
				},
				blockSubmit: true
			},
			max: {
				name: "max",
				init: function($this, name) {
					return {
						max: $this.data( "validation" + name + "Max" )
					};
				},
				validate: function($this, value, validator) {
					return (parseFloat( value, 10 ) > parseFloat( validator.max, 10 ) && ! validator.negative) || (parseFloat( value, 10 ) <= parseFloat( validator.max, 10 ) && validator.negative);
				}
			},
			min: {
				name: "min",
				init: function($this, name) {
					return {
						min: $this.data( "validation" + name + "Min" )
					};
				},
				validate: function($this, value, validator) {
					return (parseFloat( value ) < parseFloat( validator.min ) && ! validator.negative) || (parseFloat( value ) >= parseFloat( validator.min ) && validator.negative);
				}
			},
			maxlength: {
				name: "maxlength",
				init: function($this, name) {
					return {
						maxlength: $this.data( "validation" + name + "Maxlength" )
					};
				},
				validate: function($this, value, validator) {
					return ((value.length > validator.maxlength) && ! validator.negative) || ((value.length <= validator.maxlength) && validator.negative);
				}
			},
			minlength: {
				name: "minlength",
				init: function($this, name) {
					return {
						minlength: $this.data( "validation" + name + "Minlength" )
					};
				},
				validate: function($this, value, validator) {
					return ((value.length < validator.minlength) && ! validator.negative) || ((value.length >= validator.minlength) && validator.negative);
				}
			},
			maxchecked: {
				name: "maxchecked",
				init: function($this, name) {
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						maxchecked: $this.data( "validation" + name + "Maxchecked" ),
						elements: elements
					};
				},
				validate: function($this, value, validator) {
					return (validator.elements.filter( ":checked" ).length > validator.maxchecked && ! validator.negative) || (validator.elements.filter( ":checked" ).length <= validator.maxchecked && validator.negative);
				},
				blockSubmit: true
			},
			strongpass:{
				name:'strongpass',
				init:function( $this, name ){
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return{
						strongpass: $this.data('validation'+name),
						elements: elements
					}
				},
				validate: function( $this, value, validator ){

					var regex = new RegExp( '^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[~!@#$%^&*\\\(\\\)\\\-_+=|\\\\\\\{\\\}\\\[\\\]:;\\\"\'<>,.?\\\/]).{8,}$', 'g' );
					var check_regex = new RegExp( '[^\\x00-\\x7F]+', 'u' );
					if( check_regex.test( value ) ){
						var regex = new RegExp( '(?=.*\\\p{L})(?=.*\\\p{N})(?=.*[~!@#$%^&*\\\(\\\)_+=|\\\\\\\{\\\}\\\[\\\]:;\'<>,.?\/]).{8,}', 'gmu' );
					}
					return !( regex.test( value ) );

				},
				blockSubmit: true
			},
			maxselected:{
				name:"maxselected",
				init: function( $this, name ){
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						maxselected: $this.data( "validation" + name + "Maxselected" ),
						elements: elements
					};
				},
				validate: function( $this, value, validator ){
					return ( null != value && value.length > validator.maxselected && ! validator.negative);
				},
				blockSubmit: true
			},
			minselected:{
				name:"minselected",
				init: function( $this, name ){
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						minselected: $this.data( "validation" + name + "Minselected" ),
						elements: elements
					};
				},
				validate: function( $this, value, validator ){
					return ( null != value && value.length < validator.minselected && ! validator.negative);
				},
				blockSubmit: true
			},
			minchecked: {
				name: "minchecked",
				init: function($this, name) {
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						minchecked: $this.data( "validation" + name + "Minchecked" ),
						elements: elements
					};
				},
				validate: function($this, value, validator) {
					return (validator.elements.filter( ":checked" ).length < validator.minchecked && ! validator.negative) || (validator.elements.filter( ":checked" ).length >= validator.minchecked && validator.negative);
				},
				blockSubmit: true
			},
			rating:{
				name:"rating",
				init:function($this,name){
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						rating:1,
						elements:elements
					}
				},
				validate:function($this, value,validator){
					var checkedValue = 0;

					for ( var s = 0, st; st = validator.elements[s]; s++ ) {
						if ( jQuery( st ).is( ':checked' ) ) {
							checkedValue = st.value;
							break;
						}
					}
					return (checkedValue < validator.rating);
				},
				blockSubmit: true
			},
			maxupload: {
				name: "maxupload",
				init: function($this, name) {
					var elements = $this.parents( "form" ).first().find( "[name=\"" + $this.attr( "name" ) + "\"]" );
					elements.on(
						"click.validation",
						function() {
							$this.trigger(
								"change.validation",
								{
									includeEmpty: true
								}
							);
						}
					);
					return {
						maxupload: $this.data( "validation" + name + "Maxupload" ),
						elements: elements
					};
				},
				validate: function($this, value, validator) {
					value = value.split( ',' );
					return ( null != value && value.length > validator.maxupload && ! validator.negative);
				},
				blockSubmit: true
			},
		},
		builtInValidators: {
			email: {
				name: "Email",
				type: "shortcut",
				shortcut: "validemail"
			},
			validemail: {
				name: "Validemail",
				type: "regex",
				regex: "[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\\.[A-Za-z]{2,4}",
				message: "Not a valid email address<!-- data-validator-validemail-message to override -->"
			},
			passwordagain: {
				name: "Passwordagain",
				type: "match",
				match: "password",
				message: "Does not match the given password<!-- data-validator-paswordagain-message to override -->"
			},
			positive: {
				name: "Positive",
				type: "shortcut",
				shortcut: "number,positivenumber"
			},
			negative: {
				name: "Negative",
				type: "shortcut",
				shortcut: "number,negativenumber"
			},
			number: {
				name: "Number",
				type: "regex",
				regex: "([+-]?\\\d+(\\\.\\\d*)?([eE][+-]?[0-9]+)?)?",
				message: "Must be a number<!-- data-validator-number-message to override -->"
			},
			integer: {
				name: "Integer",
				type: "regex",
				regex: "[+-]?\\\d+",
				message: "No decimal places allowed<!-- data-validator-integer-message to override -->"
			},
			positivenumber: {
				name: "Positivenumber",
				type: "min",
				min: 0,
				message: "Must be a positive number<!-- data-validator-positivenumber-message to override -->"
			},
			negativenumber: {
				name: "Negativenumber",
				type: "max",
				max: 0,
				message: "Must be a negative number<!-- data-validator-negativenumber-message to override -->"
			},
			required: {
				name: "Required",
				type: "required",
				message: "This is required<!-- data-validator-required-message to override -->"
			},
			checkone: {
				name: "Checkone",
				type: "minchecked",
				minchecked: 1,
				message: "Check at least one option<!-- data-validation-checkone-message to override -->"
			},
			selectone: {
				name: "Selectone",
				type: "minselected",
				minselected: 1,
				message: "Check at least one option<!-- data-validation-selectone-message to override -->"
			},
			rating: {
				name: "Rating",
				type: "rating",
				minchecked: 1,
				message: "Please give at-least one star"
			},
			strongpass:{
				name: "Strongpass",
				type: 'strongpass',
				message: 'Please enter strong password'
			}

		}
	};
	var formatValidatorName = function(name) {
		return name.toLowerCase().replace(
			/(^|\s)([a-z])/g,
			function(m, p1, p2) {
				return p1 + p2.toUpperCase();
			}
		);
	};
	var getValue            = function($this) {
		var value = $this.val();
		var type  = $this.attr( "type" );
		var id    = $this.attr( 'id' );
		if ( typeof id != 'undefined' && id == 'arf_google_recaptcha_response') {
			var obj   = $this.parents( '.control-group' ).find( "textarea[id^=g-recaptcha-response]" );
			var value = obj.val();
		}

		if ( type == 'file' && value == '' ) {
			if ( typeof $this.attr( 'multiple' ) != 'undefined' && $this.attr( 'multiple' ) == 'multiple' ) {
				var form_id     = $this.attr( 'data-form-id' );
				var data_fid    = $this.attr( 'data-form-data-id' );
				var field_id    = $this.attr( 'id' ).replace( 'field_','' );
				var dataFieldId = $this.attr( 'data-field-id' );
				var image_name  = (jQuery( '#uploaded_file_name_' + form_id + '_' + data_fid + '_' + dataFieldId ) != null) ? jQuery( '#uploaded_file_name_' + form_id + '_' + data_fid + '_' + dataFieldId ).val() : '';
				value = image_name;
				if ( value == '' ) {
					if ( document.getElementById( 'arf_multi_file_uploader_' + field_id ) != null && document.getElementById( 'arf_multi_file_uploader_' + field_id ).querySelector( '.arf_info' ) != null ) {
						value = document.getElementById( 'arf_multi_file_uploader_' + field_id ).querySelector( '.arf_info' ).innerHTML;
					}
				}
			}
		}

		if ( $this.hasClass( 'frm_date' ) && value != "" ) {

			var field_off_day = $this.attr( 'data-off-days' );
			if ( typeof field_off_day != 'undefined' && field_off_day != null) {
				var date         = new Date( value );
				var current_date = date.getDay();
				var off_days     = JSON.parse( field_off_day );
				current_date     = String( current_date );

				if ( off_days.indexOf( current_date ) > -1 ) {
					$this.val( '' );
					value = '';
				}
			}
		}

		if (type == "checkbox") {
			value = ($this.is( ":checked" ) ? value : "");
		}
		if (type == "radio") {
			value = ($( 'input[name="' + $this.attr( "name" ) + '"]:checked' ).length > 0 ? value : "");
		}
		if( $this.hasClass('arf-selectpicker-input-control') && type == 'text' && $('dl[data-name="' + $this.attr('name') ).hasClass('multi-select') ){
			if( /,/.test( value ) ){	
				value = value.split(',');
			}
		}
		return value;
	};

	function regexFromString(inputstring) {
		return new RegExp( "^" + inputstring + "$", 'u' );
	}

	function executeFunctionByName(functionName, context) {
		var args       = Array.prototype.slice.call( arguments ).splice( 2 );
		var namespaces = functionName.split( "." );
		var func       = namespaces.pop();
		for (var i = 0; i < namespaces.length; i++) {
			context = context[namespaces[i]];
		}
		return context[func].apply( this, args );
	}
	$.fn.jqBootstrapValidation = function(method) {
		if (defaults.methods[method]) {
			return defaults.methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ) );
		} else if (typeof method === 'object' || ! method) {
			return defaults.methods.init.apply( this, arguments );
		} else {
			$.error( 'Method ' + method + ' does not exist on jQuery.jqBootstrapValidation' );
			return null;
		}
	};
	$.jqBootstrapValidation    = function(options) {
		$( ":input" ).not( "[type=image],[type=submit]" ).jqBootstrapValidation.apply( this, arguments );
	};
})( jQuery );
