(function( factory ) {
	"use strict";
	if ( typeof define === 'function' && define.amd ) {
		define(
			['jquery'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {
		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}
			if ( ! $ ) {
				$ = typeof window !== 'undefined' ?
					require( 'jquery' ) :
					require( 'jquery' )( root );
			}
			return factory( $, root, root.document );
		};
	} else {
		factory( jQuery, window, document );
	}
}
(function( $, window, document, undefined ) {
	"use strict";
	var DataTable = function ( options )
	{
		this.$ = function ( sSelector, oOpts )
		{
			return this.api( true ).$( sSelector, oOpts );
		};
		this._ = function ( sSelector, oOpts )
		{
			return this.api( true ).rows( sSelector, oOpts ).data();
		};

		this.api = function ( traditional )
		{
			return traditional ?
				new _Api(
					_fnSettingsFromNode( this[ _ext.iApiIndex ] )
				) :
				new _Api( this );
		};

		this.fnAddData = function( data, redraw )
		{
			var api = this.api( true );

			var rows = $.isArray( data ) && ( $.isArray( data[0] ) || $.isPlainObject( data[0] ) ) ?
				api.rows.add( data ) :
				api.row.add( data );

			if ( redraw === undefined || redraw ) {
				api.draw();
			}

			return rows.flatten().toArray();
		};

		this.fnAdjustColumnSizing = function ( bRedraw )
		{
			var api      = this.api( true ).columns.adjust();
			var settings = api.settings()[0];
			var scroll   = settings.oScroll;

			if ( bRedraw === undefined || bRedraw ) {
				api.draw( false );
			} else if ( scroll.sX !== "" || scroll.sY !== "" ) {

				_fnScrollDraw( settings );
			}
		};

		this.fnClearTable = function( bRedraw )
		{
			var api = this.api( true ).clear();

			if ( bRedraw === undefined || bRedraw ) {
				api.draw();
			}
		};

		this.fnClose = function( nTr )
		{
			this.api( true ).row( nTr ).child.hide();
		};

		this.fnDeleteRow = function( target, callback, redraw )
		{
			var api      = this.api( true );
			var rows     = api.rows( target );
			var settings = rows.settings()[0];
			var data     = settings.aoData[ rows[0][0] ];

			rows.remove();

			if ( callback ) {
				callback.call( this, settings, data );
			}

			if ( redraw === undefined || redraw ) {
				api.draw();
			}

			return data;
		};

		this.fnDestroy = function ( remove )
		{
			this.api( true ).destroy( remove );
		};

		this.fnDraw = function( complete )
		{

			this.api( true ).draw( complete );
		};

		this.fnFilter = function( sInput, iColumn, bRegex, bSmart, bShowGlobal, bCaseInsensitive )
		{
			var api = this.api( true );

			if ( iColumn === null || iColumn === undefined ) {
				api.search( sInput, bRegex, bSmart, bCaseInsensitive );
			} else {
				api.column( iColumn ).search( sInput, bRegex, bSmart, bCaseInsensitive );
			}

			api.draw();
		};

		this.fnGetData = function( src, col )
		{
			var api = this.api( true );

			if ( src !== undefined ) {
				var type = src.nodeName ? src.nodeName.toLowerCase() : '';

				return col !== undefined || type == 'td' || type == 'th' ?
					api.cell( src, col ).data() :
					api.row( src ).data() || null;
			}

			return api.data().toArray();
		};

		this.fnGetNodes = function( iRow )
		{
			var api = this.api( true );

			return iRow !== undefined ?
				api.row( iRow ).node() :
				api.rows().nodes().flatten().toArray();
		};

		this.fnGetPosition = function( node )
		{
			var api      = this.api( true );
			var nodeName = node.nodeName.toUpperCase();

			if ( nodeName == 'TR' ) {
				return api.row( node ).index();
			} else if ( nodeName == 'TD' || nodeName == 'TH' ) {
				var cell = api.cell( node ).index();

				return [
					cell.row,
					cell.columnVisible,
					cell.column
				];
			}
			return null;
		};

		this.fnIsOpen = function( nTr )
		{
			return this.api( true ).row( nTr ).child.isShown();
		};

		this.fnOpen = function( nTr, mHtml, sClass )
		{
			return this.api( true )
				.row( nTr )
				.child( mHtml, sClass )
				.show()
				.child()[0];
		};

		this.fnPageChange = function ( mAction, bRedraw )
		{
			var api = this.api( true ).page( mAction );

			if ( bRedraw === undefined || bRedraw ) {
				api.draw( false );
			}
		};

		this.fnSetColumnVis = function ( iCol, bShow, bRedraw )
		{
			var api = this.api( true ).column( iCol ).visible( bShow );

			if ( bRedraw === undefined || bRedraw ) {
				api.columns.adjust().draw();
			}
		};

		this.fnSettings = function()
		{
			return _fnSettingsFromNode( this[_ext.iApiIndex] );
		};

		this.fnSort = function( aaSort )
		{
			this.api( true ).order( aaSort ).draw();
		};

		this.fnSortListener = function( nNode, iColumn, fnCallback )
		{
			this.api( true ).order.listener( nNode, iColumn, fnCallback );
		};

		this.fnUpdate = function( mData, mRow, iColumn, bRedraw, bAction )
		{
			var api = this.api( true );

			if ( iColumn === undefined || iColumn === null ) {
				api.row( mRow ).data( mData );
			} else {
				api.cell( mRow, iColumn ).data( mData );
			}

			if ( bAction === undefined || bAction ) {
				api.columns.adjust();
			}

			if ( bRedraw === undefined || bRedraw ) {
				api.draw();
			}
			return 0;
		};

		this.fnVersionCheck = _ext.fnVersionCheck;

		var _that     = this;
		var emptyInit = options === undefined;
		var len       = this.length;

		if ( emptyInit ) {
			options = {};
		}

		this.oApi = this.internal = _ext.internal;

		for ( var fn in DataTable.ext.internal ) {
			if ( fn ) {
				this[fn] = _fnExternApiFunc( fn );
			}
		}

		this.each(
			function() {

				var o     = {};
				var oInit = len > 1 ?
				_fnExtend( o, options, true ) :
				options;

				var i              = 0, iLen, j, jLen, k, kLen;
				var sId            = this.getAttribute( 'id' );
				var bInitHandedOff = false;
				var defaults       = DataTable.defaults;
				var $this          = $( this );

				if ( this.nodeName.toLowerCase() != 'table' ) {
					_fnLog( null, 0, 'Non-table node initialisation (' + this.nodeName + ')', 2 );
					return;
				}

				_fnCompatOpts( defaults );
				_fnCompatCols( defaults.column );

				_fnCamelToHungarian( defaults, defaults, true );
				_fnCamelToHungarian( defaults.column, defaults.column, true );

				_fnCamelToHungarian( defaults, $.extend( oInit, $this.data() ), true );

				var allSettings = DataTable.settings;
				for ( i = 0, iLen = allSettings.length; i < iLen; i++ ) {
					var s = allSettings[i];

					if (
					s.nTable == this ||
					(s.nTHead && s.nTHead.parentNode == this) ||
					(s.nTFoot && s.nTFoot.parentNode == this)
					) {
						var bRetrieve = oInit.bRetrieve !== undefined ? oInit.bRetrieve : defaults.bRetrieve;
						var bDestroy  = oInit.bDestroy !== undefined ? oInit.bDestroy : defaults.bDestroy;

						if ( emptyInit || bRetrieve ) {
							return s.oInstance;
						} else if ( bDestroy ) {
							s.oInstance.fnDestroy();
							break;
						} else {
							_fnLog( s, 0, 'Cannot reinitialise DataTable', 3 );
							return;
						}
					}

					if ( s.sTableId == this.id ) {
						allSettings.splice( i, 1 );
						break;
					}
				}

				if ( sId === null || sId === "" ) {
					sId     = "DataTables_Table_" + (DataTable.ext._unique++);
					this.id = sId;
				}

				var oSettings    = $.extend(
					true,
					{},
					DataTable.models.oSettings,
					{
						"sDestroyWidth": $this[0].style.width,
						"sInstance":     sId,
						"sTableId":      sId
					}
				);
				oSettings.nTable = this;
				oSettings.oApi   = _that.internal;
				oSettings.oInit  = oInit;

				allSettings.push( oSettings );

				oSettings.oInstance = (_that.length === 1) ? _that : $this.dataTable();

				_fnCompatOpts( oInit );
				_fnLanguageCompat( oInit.oLanguage );

				if ( oInit.aLengthMenu && ! oInit.iDisplayLength ) {
					oInit.iDisplayLength = $.isArray( oInit.aLengthMenu[0] ) ?
					oInit.aLengthMenu[0][0] : oInit.aLengthMenu[0];
				}

				oInit = _fnExtend( $.extend( true, {}, defaults ), oInit );

				_fnMap(
					oSettings.oFeatures,
					oInit,
					[
					"bPaginate",
					"bLengthChange",
					"bFilter",
					"bSort",
					"bSortMulti",
					"bInfo",
					"bProcessing",
					"bAutoWidth",
					"bSortClasses",
					"bServerSide",
					"bDeferRender"
					]
				);
				_fnMap(
					oSettings,
					oInit,
					[
					"asStripeClasses",
					"ajax",
					"fnServerData",
					"fnFormatNumber",
					"sServerMethod",
					"aaSorting",
					"aaSortingFixed",
					"aLengthMenu",
					"sPaginationType",
					"sAjaxSource",
					"sAjaxDataProp",
					"iStateDuration",
					"sDom",
					"bSortCellsTop",
					"iTabIndex",
					"fnStateLoadCallback",
					"fnStateSaveCallback",
					"renderer",
					"searchDelay",
					"rowId",
					[ "iCookieDuration", "iStateDuration" ],
					[ "oSearch", "oPreviousSearch" ],
					[ "aoSearchCols", "aoPreSearchCols" ],
					[ "iDisplayLength", "_iDisplayLength" ]
					]
				);
				_fnMap(
					oSettings.oScroll,
					oInit,
					[
					[ "sScrollX", "sX" ],
					[ "sScrollXInner", "sXInner" ],
					[ "sScrollY", "sY" ],
					[ "bScrollCollapse", "bCollapse" ]
					]
				);
				_fnMap( oSettings.oLanguage, oInit, "fnInfoCallback" );

				_fnCallbackReg( oSettings, 'aoDrawCallback',       oInit.fnDrawCallback,      'user' );
				_fnCallbackReg( oSettings, 'aoServerParams',       oInit.fnServerParams,      'user' );
				_fnCallbackReg( oSettings, 'aoStateSaveParams',    oInit.fnStateSaveParams,   'user' );
				_fnCallbackReg( oSettings, 'aoStateLoadParams',    oInit.fnStateLoadParams,   'user' );
				_fnCallbackReg( oSettings, 'aoStateLoaded',        oInit.fnStateLoaded,       'user' );
				_fnCallbackReg( oSettings, 'aoRowCallback',        oInit.fnRowCallback,       'user' );
				_fnCallbackReg( oSettings, 'aoRowCreatedCallback', oInit.fnCreatedRow,        'user' );
				_fnCallbackReg( oSettings, 'aoHeaderCallback',     oInit.fnHeaderCallback,    'user' );
				_fnCallbackReg( oSettings, 'aoFooterCallback',     oInit.fnFooterCallback,    'user' );
				_fnCallbackReg( oSettings, 'aoInitComplete',       oInit.fnInitComplete,      'user' );
				_fnCallbackReg( oSettings, 'aoPreDrawCallback',    oInit.fnPreDrawCallback,   'user' );

				oSettings.rowIdFn = _fnGetObjectDataFn( oInit.rowId );

				_fnBrowserDetect( oSettings );

				var oClasses = oSettings.oClasses;

				$.extend( oClasses, DataTable.ext.classes, oInit.oClasses );
				$this.addClass( oClasses.sTable );

				if ( oSettings.iInitDisplayStart === undefined ) {

					oSettings.iInitDisplayStart = oInit.iDisplayStart;
					oSettings._iDisplayStart    = oInit.iDisplayStart;
				}

				if ( oInit.iDeferLoading !== null ) {
					oSettings.bDeferLoading    = true;
					var tmp                    = $.isArray( oInit.iDeferLoading );
					oSettings._iRecordsDisplay = tmp ? oInit.iDeferLoading[0] : oInit.iDeferLoading;
					oSettings._iRecordsTotal   = tmp ? oInit.iDeferLoading[1] : oInit.iDeferLoading;
				}

				var oLanguage = oSettings.oLanguage;
				$.extend( true, oLanguage, oInit.oLanguage );

				if ( oLanguage.sUrl ) {

					$.ajax(
						{
							dataType: 'json',
							url: oLanguage.sUrl,
							success: function ( json ) {
								_fnLanguageCompat( json );
								_fnCamelToHungarian( defaults.oLanguage, json );
								$.extend( true, oLanguage, json );
								_fnInitialise( oSettings );
							},
							error: function () {

								_fnInitialise( oSettings );
							}
						}
					);
					bInitHandedOff = true;
				}

				if ( oInit.asStripeClasses === null ) {
					oSettings.asStripeClasses = [
					oClasses.sStripeOdd,
					oClasses.sStripeEven
					];
				}

				var stripeClasses = oSettings.asStripeClasses;
				var rowOne        = $this.children( 'tbody' ).find( 'tr' ).eq( 0 );
				if ( $.inArray(
					true,
					$.map(
						stripeClasses,
						function(el, i) {
							return rowOne.hasClass( el );
						}
					)
				) !== -1 ) {
					$( 'tbody tr', this ).removeClass( stripeClasses.join( ' ' ) );
					oSettings.asDestroyStripes = stripeClasses.slice();
				}

				var anThs = [];
				var aoColumnsInit;
				var nThead = this.getElementsByTagName( 'thead' );
				if ( nThead.length !== 0 ) {
					_fnDetectHeader( oSettings.aoHeader, nThead[0] );
					anThs = _fnGetUniqueThs( oSettings );
				}

				if ( oInit.aoColumns === null ) {
					aoColumnsInit = [];
					for ( i = 0, iLen = anThs.length; i < iLen; i++ ) {
						aoColumnsInit.push( null );
					}
				} else {
					aoColumnsInit = oInit.aoColumns;
				}

				for ( i = 0, iLen = aoColumnsInit.length; i < iLen; i++ ) {
					_fnAddColumn( oSettings, anThs ? anThs[i] : null );
				}

				_fnApplyColumnDefs(
					oSettings,
					oInit.aoColumnDefs,
					aoColumnsInit,
					function (iCol, oDef) {
						_fnColumnOptions( oSettings, iCol, oDef );
					}
				);

				if ( rowOne.length ) {
					var a = function ( cell, name ) {
						return cell.getAttribute( 'data-' + name ) !== null ? name : null;
					};

					$( rowOne[0] ).children( 'th, td' ).each(
						function (i, cell) {
							var col = oSettings.aoColumns[i];

							if ( col.mData === i ) {
								  var sort   = a( cell, 'sort' ) || a( cell, 'order' );
								  var filter = a( cell, 'filter' ) || a( cell, 'search' );

								if ( sort !== null || filter !== null ) {
									col.mData = {
										_:      i + '.display',
										sort:   sort !== null ? i + '.@data-' + sort : undefined,
										type:   sort !== null ? i + '.@data-' + sort : undefined,
										filter: filter !== null ? i + '.@data-' + filter : undefined
									};

									_fnColumnOptions( oSettings, i );
								}
							}
						}
					);
				}

				var features   = oSettings.oFeatures;
				var loadedInit = function () {

					if ( oInit.aaSorting === undefined ) {
						var sorting = oSettings.aaSorting;
						for ( i = 0, iLen = sorting.length; i < iLen; i++ ) {
							sorting[i][1] = oSettings.aoColumns[ i ].asSorting[0];
						}
					}

					_fnSortingClasses( oSettings );

					if ( features.bSort ) {
						_fnCallbackReg(
							oSettings,
							'aoDrawCallback',
							function () {
								if ( oSettings.bSorted ) {
									var aSort         = _fnSortFlatten( oSettings );
									var sortedColumns = {};

									$.each(
										aSort,
										function (i, val) {
											sortedColumns[ val.src ] = val.dir;
										}
									);

									_fnCallbackFire( oSettings, null, 'order', [oSettings, aSort, sortedColumns] );
									_fnSortAria( oSettings );
								}
							}
						);
					}

					_fnCallbackReg(
						oSettings,
						'aoDrawCallback',
						function () {
							if ( oSettings.bSorted || _fnDataSource( oSettings ) === 'ssp' || features.bDeferRender ) {
								_fnSortingClasses( oSettings );
							}
						},
						'sc'
					);

					var captions = $this.children( 'caption' ).each(
						function () {
							this._captionSide = $( this ).css( 'caption-side' );
						}
					);

					var thead = $this.children( 'thead' );
					if ( thead.length === 0 ) {
						thead = $( '<thead/>' ).appendTo( $this );
					}
					oSettings.nTHead = thead[0];

					var tbody = $this.children( 'tbody' );
					if ( tbody.length === 0 ) {
						tbody = $( '<tbody/>' ).appendTo( $this );
					}
					oSettings.nTBody = tbody[0];

					var tfoot = $this.children( 'tfoot' );
					if ( tfoot.length === 0 && captions.length > 0 && (oSettings.oScroll.sX !== "" || oSettings.oScroll.sY !== "") ) {

						tfoot = $( '<tfoot/>' ).appendTo( $this );
					}

					if ( tfoot.length === 0 || tfoot.children().length === 0 ) {
						$this.addClass( oClasses.sNoFooter );
					} else if ( tfoot.length > 0 ) {
						oSettings.nTFoot = tfoot[0];
						_fnDetectHeader( oSettings.aoFooter, oSettings.nTFoot );
					}

					if ( oInit.aaData ) {
						for ( i = 0; i < oInit.aaData.length; i++ ) {
							_fnAddData( oSettings, oInit.aaData[ i ] );
						}
					} else if ( oSettings.bDeferLoading || _fnDataSource( oSettings ) == 'dom' ) {

						_fnAddTr( oSettings, $( oSettings.nTBody ).children( 'tr' ) );
					}

					oSettings.aiDisplay = oSettings.aiDisplayMaster.slice();

					oSettings.bInitialised = true;

					if ( bInitHandedOff === false ) {
						_fnInitialise( oSettings );
					}
				};

				if ( oInit.bStateSave ) {
					features.bStateSave = true;
					_fnCallbackReg( oSettings, 'aoDrawCallback', _fnSaveState, 'state_save' );
					_fnLoadState( oSettings, oInit, loadedInit );
				} else {
					loadedInit();
				}

			}
		);
		_that = null;
		return this;
	};

	var _ext;
	var _Api;
	var _api_register;
	var _api_registerPlural;

	var _re_dic       = {};
	var _re_new_lines = /[\r\n\u2028]/g;
	var _re_html      = /<.*?>/g;

	var _re_date = /^\d{2,4}[\.\/\-]\d{1,2}[\.\/\-]\d{1,2}([T ]{1}\d{1,2}[:\.]\d{2}([\.:]\d{2})?)?$/;

	var _re_escape_regex = new RegExp( '(\\' + [ '/', '.', '*', '+', '?', '|', '(', ')', '[', ']', '{', '}', '\\', '$', '^', '-' ].join( '|\\' ) + ')', 'g' );

	var _re_formatted_numeric = /[',$£€¥%\u2009\u202F\u20BD\u20a9\u20BArfkɃΞ]/gi;

	var _empty = function ( d ) {
		return ! d || d === true || d === '-' ? true : false;
	};

	var _intVal = function ( s ) {
		var integer = parseInt( s, 10 );
		return ! isNaN( integer ) && isFinite( s ) ? integer : null;
	};

	var _numToDecimal = function ( num, decimalPoint ) {

		if ( ! _re_dic[ decimalPoint ] ) {
			_re_dic[ decimalPoint ] = new RegExp( _fnEscapeRegex( decimalPoint ), 'g' );
		}
		return typeof num === 'string' && decimalPoint !== '.' ?
			num.replace( /\./g, '' ).replace( _re_dic[ decimalPoint ], '.' ) :
			num;
	};

	var _isNumber = function ( d, decimalPoint, formatted ) {
		var strType = typeof d === 'string';

		if ( _empty( d ) ) {
			return true;
		}

		if ( decimalPoint && strType ) {
			d = _numToDecimal( d, decimalPoint );
		}

		if ( formatted && strType ) {
			d = d.replace( _re_formatted_numeric, '' );
		}

		return ! isNaN( parseFloat( d ) ) && isFinite( d );
	};

	var _isHtml = function ( d ) {
		return _empty( d ) || typeof d === 'string';
	};

	var _htmlNumeric = function ( d, decimalPoint, formatted ) {
		if ( _empty( d ) ) {
			return true;
		}

		var html = _isHtml( d );
		return ! html ?
			null :
			_isNumber( _stripHtml( d ), decimalPoint, formatted ) ?
				true :
				null;
	};

	var _pluck = function ( a, prop, prop2 ) {
		var out = [];
		var i   = 0, ien = a.length;

		if ( prop2 !== undefined ) {
			for ( ; i < ien; i++ ) {
				if ( a[i] && a[i][ prop ] ) {
					out.push( a[i][ prop ][ prop2 ] );
				}
			}
		} else {
			for ( ; i < ien; i++ ) {
				if ( a[i] ) {
					out.push( a[i][ prop ] );
				}
			}
		}

		return out;
	};

	var _pluck_order = function ( a, order, prop, prop2 )
	{
		var out = [];
		var i   = 0, ien = order.length;

		if ( prop2 !== undefined ) {
			for ( ; i < ien; i++ ) {
				if ( a[ order[i] ][ prop ] ) {
					out.push( a[ order[i] ][ prop ][ prop2 ] );
				}
			}
		} else {
			for ( ; i < ien; i++ ) {
				out.push( a[ order[i] ][ prop ] );
			}
		}

		return out;
	};

	var _range = function ( len, start )
	{
		var out = [];
		var end;

		if ( start === undefined ) {
			start = 0;
			end   = len;
		} else {
			end   = start;
			start = len;
		}

		for ( var i = start; i < end; i++ ) {
			out.push( i );
		}

		return out;
	};

	var _removeEmpty = function ( a )
	{
		var out = [];

		for ( var i = 0, ien = a.length; i < ien; i++ ) {
			if ( a[i] ) {
				out.push( a[i] );
			}
		}

		return out;
	};

	var _stripHtml = function ( d ) {
		return d.replace( _re_html, '' );
	};

	var _areAllUnique = function ( src ) {
		if ( src.length < 2 ) {
			return true;
		}

		var sorted = src.slice().sort();
		var last   = sorted[0];

		for ( var i = 1, ien = sorted.length; i < ien; i++ ) {
			if ( sorted[i] === last ) {
				return false;
			}

			last = sorted[i];
		}

		return true;
	};

	var _unique = function ( src )
	{
		if ( _areAllUnique( src ) ) {
			return src.slice();
		}

		var
			out    = [],
			val,
			i, ien = src.length,
			j, k   = 0;

		again: for ( i = 0; i < ien; i++ ) {
			val = src[i];

			for ( j = 0; j < k; j++ ) {
				if ( out[j] === val ) {
					continue again;
				}
			}

			out.push( val );
			k++;
		}

		return out;
	};

	DataTable.util = {

		throttle: function ( fn, freq ) {
			var
				frequency = freq !== undefined ? freq : 200,
				last,
				timer;

			return function () {
				var
					that = this,
					now  = +new Date(),
					args = arguments;

				if ( last && now < last + frequency ) {
					clearTimeout( timer );

					timer = setTimeout(
						function () {
							last = undefined;
							fn.apply( that, args );
						},
						frequency
					);
				} else {
					last = now;
				}
			};
		},

		escapeRegex: function ( val ) {
			return val.replace( _re_escape_regex, '\\$1' );
		}
	};

	function _fnHungarianMap ( o )
	{
		var
			hungarian = 'a aa ai ao as b fn i m o s ',
			match,
			newKey,
			map       = {};

		$.each(
			o,
			function (key, val) {
				match = key.match( /^([^A-Z]+?)([A-Z])/ );

				if ( match && hungarian.indexOf( match[1] + ' ' ) !== -1 ) {
					newKey        = key.replace( match[0], match[2].toLowerCase() );
					map[ newKey ] = key;

					if ( match[1] === 'o' ) {
						_fnHungarianMap( o[key] );
					}
				}
			}
		);

		o._hungarianMap = map;
	}

	function _fnCamelToHungarian ( src, user, force )
	{
		if ( ! src._hungarianMap ) {
			_fnHungarianMap( src );
		}

		var hungarianKey;

		$.each(
			user,
			function (key, val) {
				hungarianKey = src._hungarianMap[ key ];

				if ( hungarianKey !== undefined && (force || user[hungarianKey] === undefined) ) {

					if ( hungarianKey.charAt( 0 ) === 'o' ) {

						if ( ! user[ hungarianKey ] ) {
							user[ hungarianKey ] = {};
						}
						$.extend( true, user[hungarianKey], user[key] );

						_fnCamelToHungarian( src[hungarianKey], user[hungarianKey], force );
					} else {
						user[hungarianKey] = user[ key ];
					}
				}
			}
		);
	}

	function _fnLanguageCompat( lang )
	{

		var defaults = DataTable.defaults.oLanguage;

		var defaultDecimal = defaults.sDecimal;
		if ( defaultDecimal ) {
			_addNumericSort( defaultDecimal );
		}

		if ( lang ) {
			var zeroRecords = lang.sZeroRecords;

			if ( ! lang.sEmptyTable && zeroRecords &&
				defaults.sEmptyTable === "No data available in table" ) {
				_fnMap( lang, lang, 'sZeroRecords', 'sEmptyTable' );
			}

			if ( ! lang.sLoadingRecords && zeroRecords &&
				defaults.sLoadingRecords === "Loading..." ) {
				_fnMap( lang, lang, 'sZeroRecords', 'sLoadingRecords' );
			}

			if ( lang.sInfoThousands ) {
				lang.sThousands = lang.sInfoThousands;
			}

			var decimal = lang.sDecimal;
			if ( decimal && defaultDecimal !== decimal ) {
				_addNumericSort( decimal );
			}
		}
	}

	var _fnCompatMap = function ( o, knew, old ) {
		if ( o[ knew ] !== undefined ) {
			o[ old ] = o[ knew ];
		}
	};

	function _fnCompatOpts ( init )
	{
		_fnCompatMap( init, 'ordering',      'bSort' );
		_fnCompatMap( init, 'orderMulti',    'bSortMulti' );
		_fnCompatMap( init, 'orderClasses',  'bSortClasses' );
		_fnCompatMap( init, 'orderCellsTop', 'bSortCellsTop' );
		_fnCompatMap( init, 'order',         'aaSorting' );
		_fnCompatMap( init, 'orderFixed',    'aaSortingFixed' );
		_fnCompatMap( init, 'paging',        'bPaginate' );
		_fnCompatMap( init, 'pagingType',    'sPaginationType' );
		_fnCompatMap( init, 'pageLength',    'iDisplayLength' );
		_fnCompatMap( init, 'searching',     'bFilter' );

		if ( typeof init.sScrollX === 'boolean' ) {
			init.sScrollX = init.sScrollX ? '100%' : '';
		}
		if ( typeof init.scrollX === 'boolean' ) {
			init.scrollX = init.scrollX ? '100%' : '';
		}

		var searchCols = init.aoSearchCols;

		if ( searchCols ) {
			for ( var i = 0, ien = searchCols.length; i < ien; i++ ) {
				if ( searchCols[i] ) {
					_fnCamelToHungarian( DataTable.models.oSearch, searchCols[i] );
				}
			}
		}
	}

	function _fnCompatCols ( init )
	{
		_fnCompatMap( init, 'orderable',     'bSortable' );
		_fnCompatMap( init, 'orderData',     'aDataSort' );
		_fnCompatMap( init, 'orderSequence', 'asSorting' );
		_fnCompatMap( init, 'orderDataType', 'sortDataType' );

		var dataSort = init.aDataSort;
		if ( typeof dataSort === 'number' && ! $.isArray( dataSort ) ) {
			init.aDataSort = [ dataSort ];
		}
	}

	function _fnBrowserDetect( settings )
	{

		if ( ! DataTable.__browser ) {
			var browser         = {};
			DataTable.__browser = browser;

			var n = $( '<div/>' )
				.css(
					{
						position: 'fixed',
						top: 0,
						left: $( window ).scrollLeft() * -1,
						height: 1,
						width: 1,
						overflow: 'hidden'
					}
				)
				.append(
					$( '<div/>' )
						.css(
							{
								position: 'absolute',
								top: 1,
								left: 1,
								width: 100,
								overflow: 'scroll'
							}
						)
						.append(
							$( '<div/>' )
								.css(
									{
										width: '100%',
										height: 10
									}
								)
						)
				)
				.appendTo( 'body' );

			var outer = n.children();
			var inner = outer.children();

			browser.barWidth = outer[0].offsetWidth - outer[0].clientWidth;

			browser.bScrollOversize = inner[0].offsetWidth === 100 && outer[0].clientWidth !== 100;

			browser.bScrollbarLeft = Math.round( inner.offset().left ) !== 1;

			browser.bBounding = n[0].getBoundingClientRect().width ? true : false;

			n.remove();
		}

		$.extend( settings.oBrowser, DataTable.__browser );
		settings.oScroll.iBarWidth = DataTable.__browser.barWidth;
	}

	function _fnReduce ( that, fn, init, start, end, inc )
	{
		var
			i     = start,
			value,
			isSet = false;

		if ( init !== undefined ) {
			value = init;
			isSet = true;
		}

		while ( i !== end ) {
			if ( ! that.hasOwnProperty( i ) ) {
				continue;
			}

			value = isSet ?
				fn( value, that[i], i, that ) :
				that[i];

			isSet = true;
			i    += inc;
		}

		return value;
	}

	function _fnAddColumn( oSettings, nTh )
	{

		var oDefaults = DataTable.defaults.column;
		var iCol      = oSettings.aoColumns.length;
		var oCol      = $.extend(
			{},
			DataTable.models.oColumn,
			oDefaults,
			{
				"nTh": nTh ? nTh : document.createElement( 'th' ),
				"sTitle":    oDefaults.sTitle ? oDefaults.sTitle : nTh ? nTh.innerHTML : '',
				"aDataSort": oDefaults.aDataSort ? oDefaults.aDataSort : [iCol],
				"mData": oDefaults.mData ? oDefaults.mData : iCol,
				idx: iCol
			}
		);
		oSettings.aoColumns.push( oCol );

		var searchCols     = oSettings.aoPreSearchCols;
		searchCols[ iCol ] = $.extend( {}, DataTable.models.oSearch, searchCols[ iCol ] );

		_fnColumnOptions( oSettings, iCol, $( nTh ).data() );
	}

	function _fnColumnOptions( oSettings, iCol, oOptions )
	{
		var oCol     = oSettings.aoColumns[ iCol ];
		var oClasses = oSettings.oClasses;
		var th       = $( oCol.nTh );

		if ( ! oCol.sWidthOrig ) {

			oCol.sWidthOrig = th.attr( 'width' ) || null;

			var t = (th.attr( 'style' ) || '').match( /width:\s*(\d+[pxem%]+)/ );
			if ( t ) {
				oCol.sWidthOrig = t[1];
			}
		}

		if ( oOptions !== undefined && oOptions !== null ) {

			_fnCompatCols( oOptions );

			_fnCamelToHungarian( DataTable.defaults.column, oOptions, true );

			if ( oOptions.mDataProp !== undefined && ! oOptions.mData ) {
				oOptions.mData = oOptions.mDataProp;
			}

			if ( oOptions.sType ) {
				oCol._sManualType = oOptions.sType;
			}

			if ( oOptions.className && ! oOptions.sClass ) {
				oOptions.sClass = oOptions.className;
			}
			if ( oOptions.sClass ) {
				th.addClass( oOptions.sClass );
			}

			$.extend( oCol, oOptions );
			_fnMap( oCol, oOptions, "sWidth", "sWidthOrig" );

			if ( oOptions.iDataSort !== undefined ) {
				oCol.aDataSort = [ oOptions.iDataSort ];
			}
			_fnMap( oCol, oOptions, "aDataSort" );
		}

		var mDataSrc = oCol.mData;
		var mData    = _fnGetObjectDataFn( mDataSrc );
		var mRender  = oCol.mRender ? _fnGetObjectDataFn( oCol.mRender ) : null;

		var attrTest   = function( src ) {
			return typeof src === 'string' && src.indexOf( '@' ) !== -1;
		};
		oCol._bAttrSrc = $.isPlainObject( mDataSrc ) && (
			attrTest( mDataSrc.sort ) || attrTest( mDataSrc.type ) || attrTest( mDataSrc.filter )
		);
		oCol._setter   = null;

		oCol.fnGetData = function (rowData, type, meta) {
			var innerData = mData( rowData, type, undefined, meta );

			return mRender && type ?
				mRender( innerData, type, rowData, meta ) :
				innerData;
		};
		oCol.fnSetData = function ( rowData, val, meta ) {
			return _fnSetObjectDataFn( mDataSrc )( rowData, val, meta );
		};

		if ( typeof mDataSrc !== 'number' ) {
			oSettings._rowReadObject = true;
		}

		if ( ! oSettings.oFeatures.bSort ) {
			oCol.bSortable = false;
			th.addClass( oClasses.sSortableNone );
		}

		var bAsc  = $.inArray( 'asc', oCol.asSorting ) !== -1;
		var bDesc = $.inArray( 'desc', oCol.asSorting ) !== -1;
		if ( ! oCol.bSortable || ( ! bAsc && ! bDesc) ) {
			oCol.sSortingClass    = oClasses.sSortableNone;
			oCol.sSortingClassJUI = "";
		} else if ( bAsc && ! bDesc ) {
			oCol.sSortingClass    = oClasses.sSortableAsc;
			oCol.sSortingClassJUI = oClasses.sSortJUIAscAllowed;
		} else if ( ! bAsc && bDesc ) {
			oCol.sSortingClass    = oClasses.sSortableDesc;
			oCol.sSortingClassJUI = oClasses.sSortJUIDescAllowed;
		} else {
			oCol.sSortingClass    = oClasses.sSortable;
			oCol.sSortingClassJUI = oClasses.sSortJUI;
		}
	}

	function _fnAdjustColumnSizing ( settings )
	{

		if ( settings.oFeatures.bAutoWidth !== false ) {
			var columns = settings.aoColumns;

			_fnCalculateColumnWidths( settings );
			for ( var i = 0 , iLen = columns.length; i < iLen; i++ ) {
				columns[i].nTh.style.width = columns[i].sWidth;
			}
		}

		var scroll = settings.oScroll;
		if ( scroll.sY !== '' || scroll.sX !== '') {
			_fnScrollDraw( settings );
		}

		_fnCallbackFire( settings, null, 'column-sizing', [settings] );
	}

	function _fnVisibleToColumnIndex( oSettings, iMatch )
	{
		var aiVis = _fnGetColumns( oSettings, 'bVisible' );

		return typeof aiVis[iMatch] === 'number' ?
			aiVis[iMatch] :
			null;
	}

	function _fnColumnIndexToVisible( oSettings, iMatch )
	{
		var aiVis = _fnGetColumns( oSettings, 'bVisible' );
		var iPos  = $.inArray( iMatch, aiVis );

		return iPos !== -1 ? iPos : null;
	}

	function _fnVisbleColumns( oSettings )
	{
		var vis = 0;

		$.each(
			oSettings.aoColumns,
			function ( i, col ) {
				if ( col.bVisible && $( col.nTh ).css( 'display' ) !== 'none' ) {
					vis++;
				}
			}
		);

		return vis;
	}

	function _fnGetColumns( oSettings, sParam )
	{
		var a = [];

		$.map(
			oSettings.aoColumns,
			function(val, i) {
				if ( val[sParam] ) {
					a.push( i );
				}
			}
		);

		return a;
	}

	function _fnColumnTypes ( settings )
	{
		var columns = settings.aoColumns;
		var data    = settings.aoData;
		var types   = DataTable.ext.type.detect;
		var i, ien, j, jen, k, ken;
		var col, cell, detectedType, cache;

		for ( i = 0, ien = columns.length; i < ien; i++ ) {
			col   = columns[i];
			cache = [];

			if ( ! col.sType && col._sManualType ) {
				col.sType = col._sManualType;
			} else if ( ! col.sType ) {
				for ( j = 0, jen = types.length; j < jen; j++ ) {
					for ( k = 0, ken = data.length; k < ken; k++ ) {

						if ( cache[k] === undefined ) {
							cache[k] = _fnGetCellData( settings, k, i, 'type' );
						}

						detectedType = types[j]( cache[k], settings );

						if ( ! detectedType && j !== types.length - 1 ) {
							break;
						}

						if ( detectedType === 'html' ) {
							break;
						}
					}

					if ( detectedType ) {
						col.sType = detectedType;
						break;
					}
				}

				if ( ! col.sType ) {
					col.sType = 'string';
				}
			}
		}
	}

	function _fnApplyColumnDefs( oSettings, aoColDefs, aoCols, fn )
	{
		var i, iLen, j, jLen, k, kLen, def;
		var columns = oSettings.aoColumns;

		if ( aoColDefs ) {

			for ( i = aoColDefs.length - 1; i >= 0; i-- ) {
				def = aoColDefs[i];

				var aTargets = def.targets !== undefined ?
					def.targets :
					def.aTargets;

				if ( ! $.isArray( aTargets ) ) {
					aTargets = [ aTargets ];
				}

				for ( j = 0, jLen = aTargets.length; j < jLen; j++ ) {
					if ( typeof aTargets[j] === 'number' && aTargets[j] >= 0 ) {

						while ( columns.length <= aTargets[j] ) {
							_fnAddColumn( oSettings );
						}

						fn( aTargets[j], def );
					} else if ( typeof aTargets[j] === 'number' && aTargets[j] < 0 ) {

						fn( columns.length + aTargets[j], def );
					} else if ( typeof aTargets[j] === 'string' ) {

						for ( k = 0, kLen = columns.length; k < kLen; k++ ) {
							if ( aTargets[j] == "_all" ||
								 $( columns[k].nTh ).hasClass( aTargets[j] ) ) {
								fn( k, def );
							}
						}
					}
				}
			}
		}

		if ( aoCols ) {
			for ( i = 0, iLen = aoCols.length; i < iLen; i++ ) {
				fn( i, aoCols[i] );
			}
		}
	}

	function _fnAddData ( oSettings, aDataIn, nTr, anTds )
	{

		var iRow  = oSettings.aoData.length;
		var oData = $.extend(
			true,
			{},
			DataTable.models.oRow,
			{
				src: nTr ? 'dom' : 'data',
				idx: iRow
			}
		);

		oData._aData = aDataIn;
		oSettings.aoData.push( oData );

		var nTd, sThisType;
		var columns = oSettings.aoColumns;

		for ( var i = 0, iLen = columns.length; i < iLen; i++ ) {
			columns[i].sType = null;
		}

		oSettings.aiDisplayMaster.push( iRow );

		var id = oSettings.rowIdFn( aDataIn );
		if ( id !== undefined ) {
			oSettings.aIds[ id ] = oData;
		}

		if ( nTr || ! oSettings.oFeatures.bDeferRender ) {
			_fnCreateTr( oSettings, iRow, nTr, anTds );
		}

		return iRow;
	}

	function _fnAddTr( settings, trs )
	{
		var row;

		if ( ! (trs instanceof $) ) {
			trs = $( trs );
		}

		return trs.map(
			function (i, el) {
				row = _fnGetRowElements( settings, el );
				return _fnAddData( settings, row.data, el, row.cells );
			}
		);
	}

	function _fnNodeToDataIndex( oSettings, n )
	{
		return (n._DT_RowIndex !== undefined) ? n._DT_RowIndex : null;
	}

	function _fnNodeToColumnIndex( oSettings, iRow, n )
	{
		return $.inArray( n, oSettings.aoData[ iRow ].anCells );
	}

	function _fnGetCellData( settings, rowIdx, colIdx, type )
	{
		var draw           = settings.iDraw;
		var col            = settings.aoColumns[colIdx];
		var rowData        = settings.aoData[rowIdx]._aData;
		var defaultContent = col.sDefaultContent;
		var cellData       = col.fnGetData(
			rowData,
			type,
			{
				settings: settings,
				row:      rowIdx,
				col:      colIdx
			}
		);

		if ( cellData === undefined ) {
			if ( settings.iDrawError != draw && defaultContent === null ) {
				_fnLog(
					settings,
					0,
					"Requested unknown parameter " +
					(typeof col.mData == 'function' ? '{function}' : "'" + col.mData + "'") +
					" for row " + rowIdx + ", column " + colIdx,
					4
				);
				settings.iDrawError = draw;
			}
			return defaultContent;
		}

		if ( (cellData === rowData || cellData === null) && defaultContent !== null && type !== undefined ) {
			cellData = defaultContent;
		} else if ( typeof cellData === 'function' ) {

			return cellData.call( rowData );
		}

		if ( cellData === null && type == 'display' ) {
			return '';
		}
		return cellData;
	}

	function _fnSetCellData( settings, rowIdx, colIdx, val )
	{
		var col     = settings.aoColumns[colIdx];
		var rowData = settings.aoData[rowIdx]._aData;

		col.fnSetData(
			rowData,
			val,
			{
				settings: settings,
				row:      rowIdx,
				col:      colIdx
			}
		);
	}

	var __reArray = /\[.*?\]$/;
	var __reFn    = /\(\)$/;

	function _fnSplitObjNotation( str )
	{
		return $.map(
			str.match( /(\\.|[^\.])+/g ) || [''],
			function ( s ) {
				return s.replace( /\\\./g, '.' );
			}
		);
	}

	function _fnGetObjectDataFn( mSource )
	{
		if ( $.isPlainObject( mSource ) ) {

			var o = {};
			$.each(
				mSource,
				function (key, val) {
					if ( val ) {
						o[key] = _fnGetObjectDataFn( val );
					}
				}
			);

			return function (data, type, row, meta) {
				var t = o[type] || o._;
				return t !== undefined ?
					t( data, type, row, meta ) :
					data;
			};
		} else if ( mSource === null ) {

			return function (data) {
				return data;
			};
		} else if ( typeof mSource === 'function' ) {
			return function (data, type, row, meta) {
				return mSource( data, type, row, meta );
			};
		} else if ( typeof mSource === 'string' && (mSource.indexOf( '.' ) !== -1 ||
				  mSource.indexOf( '[' ) !== -1 || mSource.indexOf( '(' ) !== -1) ) {

			var fetchData = function (data, type, src) {
				var arrayNotation, funcNotation, out, innerSrc;

				if ( src !== "" ) {
					var a = _fnSplitObjNotation( src );

					for ( var i = 0, iLen = a.length; i < iLen; i++ ) {

						arrayNotation = a[i].match( __reArray );
						funcNotation  = a[i].match( __reFn );

						if ( arrayNotation ) {

							a[i] = a[i].replace( __reArray, '' );

							if ( a[i] !== "" ) {
								data = data[ a[i] ];
							}
							out = [];

							a.splice( 0, i + 1 );
							innerSrc = a.join( '.' );

							if ( $.isArray( data ) ) {
								for ( var j = 0, jLen = data.length; j < jLen; j++ ) {
									out.push( fetchData( data[j], type, innerSrc ) );
								}
							}

							var join = arrayNotation[0].substring( 1, arrayNotation[0].length - 1 );
							data     = (join === "") ? out : out.join( join );

							break;
						} else if ( funcNotation ) {

							a[i] = a[i].replace( __reFn, '' );
							data = data[ a[i] ]();
							continue;
						}

						if ( data === null || data[ a[i] ] === undefined ) {
							return undefined;
						}
						data = data[ a[i] ];
					}
				}

				return data;
			};

			return function (data, type) {
				return fetchData( data, type, mSource );
			};
		} else {

			return function (data, type) {
				return data[mSource];
			};
		}
	}

	function _fnSetObjectDataFn( mSource )
	{
		if ( $.isPlainObject( mSource ) ) {

			return _fnSetObjectDataFn( mSource._ );
		} else if ( mSource === null ) {

			return function () {};
		} else if ( typeof mSource === 'function' ) {
			return function (data, val, meta) {
				mSource( data, 'set', val, meta );
			};
		} else if ( typeof mSource === 'string' && (mSource.indexOf( '.' ) !== -1 ||
				  mSource.indexOf( '[' ) !== -1 || mSource.indexOf( '(' ) !== -1) ) {

			var setData = function (data, val, src) {
				var a     = _fnSplitObjNotation( src ), b;
				var aLast = a[a.length - 1];
				var arrayNotation, funcNotation, o, innerSrc;

				for ( var i = 0, iLen = a.length - 1; i < iLen; i++ ) {

					arrayNotation = a[i].match( __reArray );
					funcNotation  = a[i].match( __reFn );

					if ( arrayNotation ) {
						a[i]         = a[i].replace( __reArray, '' );
						data[ a[i] ] = [];

						b = a.slice();
						b.splice( 0, i + 1 );
						innerSrc = b.join( '.' );

						if ( $.isArray( val ) ) {
							for ( var j = 0, jLen = val.length; j < jLen; j++ ) {
								o = {};
								setData( o, val[j], innerSrc );
								data[ a[i] ].push( o );
							}
						} else {

							data[ a[i] ] = val;
						}

						return;
					} else if ( funcNotation ) {

						a[i] = a[i].replace( __reFn, '' );
						data = data[ a[i] ]( val );
					}

					if ( data[ a[i] ] === null || data[ a[i] ] === undefined ) {
						data[ a[i] ] = {};
					}
					data = data[ a[i] ];
				}

				if ( aLast.match( __reFn ) ) {

					data = data[ aLast.replace( __reFn, '' ) ]( val );
				} else {

					data[ aLast.replace( __reArray, '' ) ] = val;
				}
			};

			return function (data, val) {
				return setData( data, val, mSource );
			};
		} else {

			return function (data, val) {
				data[mSource] = val;
			};
		}
	}

	function _fnGetDataMaster ( settings )
	{
		return _pluck( settings.aoData, '_aData' );
	}

	function _fnClearTable( settings )
	{
		settings.aoData.length          = 0;
		settings.aiDisplayMaster.length = 0;
		settings.aiDisplay.length       = 0;
		settings.aIds                   = {};
	}

	function _fnDeleteIndex( a, iTarget, splice )
	{
		var iTargetIndex = -1;

		for ( var i = 0, iLen = a.length; i < iLen; i++ ) {
			if ( a[i] == iTarget ) {
				iTargetIndex = i;
			} else if ( a[i] > iTarget ) {
				a[i]--;
			}
		}

		if ( iTargetIndex != -1 && splice === undefined ) {
			a.splice( iTargetIndex, 1 );
		}
	}

	function _fnInvalidate( settings, rowIdx, src, colIdx )
	{
		var row = settings.aoData[ rowIdx ];
		var i, ien;
		var cellWrite = function ( cell, col ) {

			while ( cell.childNodes.length ) {
				cell.removeChild( cell.firstChild );
			}

			cell.innerHTML = _fnGetCellData( settings, rowIdx, col, 'display' );
		};

		if ( src === 'dom' || (( ! src || src === 'auto') && row.src === 'dom') ) {

			row._aData = _fnGetRowElements(
				settings,
				row,
				colIdx,
				colIdx === undefined ? undefined : row._aData
			)
				.data;
		} else {

			var cells = row.anCells;

			if ( cells ) {
				if ( colIdx !== undefined ) {
					cellWrite( cells[colIdx], colIdx );
				} else {
					for ( i = 0, ien = cells.length; i < ien; i++ ) {
						cellWrite( cells[i], i );
					}
				}
			}
		}

		row._aSortData   = null;
		row._aFilterData = null;

		var cols = settings.aoColumns;
		if ( colIdx !== undefined ) {
			cols[ colIdx ].sType = null;
		} else {
			for ( i = 0, ien = cols.length; i < ien; i++ ) {
				cols[i].sType = null;
			}

			_fnRowAttributes( settings, row );
		}
	}

	function _fnGetRowElements( settings, row, colIdx, d )
	{
		var
			tds             = [],
			td              = row.firstChild,
			name, col, o, i = 0, contents,
			columns         = settings.aoColumns,
			objectRead      = settings._rowReadObject;

		d = d !== undefined ?
			d :
			objectRead ?
				{} :
				[];

		var attr = function ( str, td  ) {
			if ( typeof str === 'string' ) {
				var idx = str.indexOf( '@' );

				if ( idx !== -1 ) {
					var attr   = str.substring( idx + 1 );
					var setter = _fnSetObjectDataFn( str );
					setter( d, td.getAttribute( attr ) );
				}
			}
		};

		var cellProcess = function ( cell ) {
			if ( colIdx === undefined || colIdx === i ) {
				col      = columns[i];
				contents = $.trim( cell.innerHTML );

				if ( col && col._bAttrSrc ) {
					var setter = _fnSetObjectDataFn( col.mData._ );
					setter( d, contents );

					attr( col.mData.sort, cell );
					attr( col.mData.type, cell );
					attr( col.mData.filter, cell );
				} else {

					if ( objectRead ) {
						if ( ! col._setter ) {

							col._setter = _fnSetObjectDataFn( col.mData );
						}
						col._setter( d, contents );
					} else {
						d[i] = contents;
					}
				}
			}

			i++;
		};

		if ( td ) {

			while ( td ) {
				name = td.nodeName.toUpperCase();

				if ( name == "TD" || name == "TH" ) {
					cellProcess( td );
					tds.push( td );
				}

				td = td.nextSibling;
			}
		} else {

			tds = row.anCells;

			for ( var j = 0, jen = tds.length; j < jen; j++ ) {
				cellProcess( tds[j] );
			}
		}

		var rowNode = row.firstChild ? row : row.nTr;

		if ( rowNode ) {
			var id = rowNode.getAttribute( 'id' );

			if ( id ) {
				_fnSetObjectDataFn( settings.rowId )( d, id );
			}
		}

		return {
			data: d,
			cells: tds
		};
	}

	function _fnCreateTr ( oSettings, iRow, nTrIn, anTds )
	{
		var
			row     = oSettings.aoData[iRow],
			rowData = row._aData,
			cells   = [],
			nTr, nTd, oCol,
			i, iLen, create;

		if ( row.nTr === null ) {
			nTr = nTrIn || document.createElement( 'tr' );

			row.nTr     = nTr;
			row.anCells = cells;

			nTr._DT_RowIndex = iRow;

			_fnRowAttributes( oSettings, row );

			for ( i = 0, iLen = oSettings.aoColumns.length; i < iLen; i++ ) {
				oCol   = oSettings.aoColumns[i];
				create = nTrIn ? false : true;

				nTd               = create ? document.createElement( oCol.sCellType ) : anTds[i];
				nTd._DT_CellIndex = {
					row: iRow,
					column: i
				};

				cells.push( nTd );

				if ( create || (( ! nTrIn || oCol.mRender || oCol.mData !== i) &&
					 ( ! $.isPlainObject( oCol.mData ) || oCol.mData._ !== i + '.display')
				)) {
					nTd.innerHTML = _fnGetCellData( oSettings, iRow, i, 'display' );
				}

				if ( oCol.sClass ) {
					nTd.className += ' ' + oCol.sClass;
				}

				if ( oCol.bVisible && ! nTrIn ) {
					nTr.appendChild( nTd );
				} else if ( ! oCol.bVisible && nTrIn ) {
					nTd.parentNode.removeChild( nTd );
				}

				if ( oCol.fnCreatedCell ) {
					oCol.fnCreatedCell.call(
						oSettings.oInstance,
						nTd,
						_fnGetCellData( oSettings, iRow, i ),
						rowData,
						iRow,
						i
					);
				}
			}

			_fnCallbackFire( oSettings, 'aoRowCreatedCallback', null, [nTr, rowData, iRow, cells] );
		}

		row.nTr.setAttribute( 'role', 'row' );
	}

	function _fnRowAttributes( settings, row )
	{
		var tr   = row.nTr;
		var data = row._aData;

		if ( tr ) {
			var id = settings.rowIdFn( data );

			if ( id ) {
				tr.id = id;
			}

			if ( data.DT_RowClass ) {

				var a      = data.DT_RowClass.split( ' ' );
				row.__rowc = row.__rowc ?
					_unique( row.__rowc.concat( a ) ) :
					a;

				$( tr )
					.removeClass( row.__rowc.join( ' ' ) )
					.addClass( data.DT_RowClass );
			}

			if ( data.DT_RowAttr ) {
				$( tr ).attr( data.DT_RowAttr );
			}

			if ( data.DT_RowData ) {
				$( tr ).data( data.DT_RowData );
			}
		}
	}

	function _fnBuildHead( oSettings )
	{
		var i, ien, cell, row, column;
		var thead        = oSettings.nTHead;
		var tfoot        = oSettings.nTFoot;
		var createHeader = $( 'th, td', thead ).length === 0;
		var classes      = oSettings.oClasses;
		var columns      = oSettings.aoColumns;

		if ( createHeader ) {
			row = $( '<tr/>' ).appendTo( thead );
		}

		for ( i = 0, ien = columns.length; i < ien; i++ ) {
			column = columns[i];
			cell   = $( column.nTh ).addClass( column.sClass );

			if ( createHeader ) {
				cell.appendTo( row );
			}

			if ( oSettings.oFeatures.bSort ) {
				cell.addClass( column.sSortingClass );

				if ( column.bSortable !== false ) {
					cell
						.attr( 'tabindex', oSettings.iTabIndex )
						.attr( 'aria-controls', oSettings.sTableId );

					_fnSortAttachListener( oSettings, column.nTh, i );
				}
			}

			if ( column.sTitle != cell[0].innerHTML ) {
				cell.html( column.sTitle );
			}

			_fnRenderer( oSettings, 'header' )(
				oSettings,
				cell,
				column,
				classes
			);
		}

		if ( createHeader ) {
			_fnDetectHeader( oSettings.aoHeader, thead );
		}

		$( thead ).find( '>tr' ).attr( 'role', 'row' );

		$( thead ).find( '>tr>th, >tr>td' ).addClass( classes.sHeaderTH );
		$( tfoot ).find( '>tr>th, >tr>td' ).addClass( classes.sFooterTH );

		if ( tfoot !== null ) {
			var cells = oSettings.aoFooter[0];

			for ( i = 0, ien = cells.length; i < ien; i++ ) {
				column     = columns[i];
				column.nTf = cells[i].cell;

				if ( column.sClass ) {
					$( column.nTf ).addClass( column.sClass );
				}
			}
		}
	}

	function _fnDrawHead( oSettings, aoSource, bIncludeHidden )
	{
		var i, iLen, j, jLen, k, kLen, n, nLocalTr;
		var aoLocal  = [];
		var aApplied = [];
		var iColumns = oSettings.aoColumns.length;
		var iRowspan, iColspan;

		if ( ! aoSource ) {
			return;
		}

		if (  bIncludeHidden === undefined ) {
			bIncludeHidden = false;
		}

		for ( i = 0, iLen = aoSource.length; i < iLen; i++ ) {
			aoLocal[i]     = aoSource[i].slice();
			aoLocal[i].nTr = aoSource[i].nTr;

			for ( j = iColumns - 1; j >= 0; j-- ) {
				if ( ! oSettings.aoColumns[j].bVisible && ! bIncludeHidden ) {
					aoLocal[i].splice( j, 1 );
				}
			}

			aApplied.push( [] );
		}

		for ( i = 0, iLen = aoLocal.length; i < iLen; i++ ) {
			nLocalTr = aoLocal[i].nTr;

			if ( nLocalTr ) {
				while ( (n = nLocalTr.firstChild) ) {
					nLocalTr.removeChild( n );
				}
			}

			for ( j = 0, jLen = aoLocal[i].length; j < jLen; j++ ) {
				iRowspan = 1;
				iColspan = 1;

				if ( aApplied[i][j] === undefined ) {
					nLocalTr.appendChild( aoLocal[i][j].cell );
					aApplied[i][j] = 1;

					while ( aoLocal[i + iRowspan] !== undefined &&
							aoLocal[i][j].cell == aoLocal[i + iRowspan][j].cell ) {
						aApplied[i + iRowspan][j] = 1;
						iRowspan++;
					}

					while ( aoLocal[i][j + iColspan] !== undefined &&
							aoLocal[i][j].cell == aoLocal[i][j + iColspan].cell ) {

						for ( k = 0; k < iRowspan; k++ ) {
							aApplied[i + k][j + iColspan] = 1;
						}
						iColspan++;
					}

					$( aoLocal[i][j].cell )
						.attr( 'rowspan', iRowspan )
						.attr( 'colspan', iColspan );
				}
			}
		}
	}

	function _fnDraw( oSettings )
	{

		var aPreDraw = _fnCallbackFire( oSettings, 'aoPreDrawCallback', 'preDraw', [oSettings] );
		if ( $.inArray( false, aPreDraw ) !== -1 ) {
			_fnProcessingDisplay( oSettings, false );
			return;
		}

		var i, iLen, n;
		var anRows            = [];
		var iRowCount         = 0;
		var asStripeClasses   = oSettings.asStripeClasses;
		var iStripes          = asStripeClasses.length;
		var iOpenRows         = oSettings.aoOpenRows.length;
		var oLang             = oSettings.oLanguage;
		var iInitDisplayStart = oSettings.iInitDisplayStart;
		var bServerSide       = _fnDataSource( oSettings ) == 'ssp';
		var aiDisplay         = oSettings.aiDisplay;

		oSettings.bDrawing = true;

		if ( iInitDisplayStart !== undefined && iInitDisplayStart !== -1 ) {
			oSettings._iDisplayStart = bServerSide ?
				iInitDisplayStart :
				iInitDisplayStart >= oSettings.fnRecordsDisplay() ?
					0 :
					iInitDisplayStart;

			oSettings.iInitDisplayStart = -1;
		}

		var iDisplayStart = oSettings._iDisplayStart;
		var iDisplayEnd   = oSettings.fnDisplayEnd();

		if ( oSettings.bDeferLoading ) {
			oSettings.bDeferLoading = false;
			oSettings.iDraw++;
			_fnProcessingDisplay( oSettings, false );
		} else if ( ! bServerSide ) {
			oSettings.iDraw++;
		} else if ( ! oSettings.bDestroying && ! _fnAjaxUpdate( oSettings ) ) {
			return;
		}

		if ( aiDisplay.length !== 0 ) {
			var iStart = bServerSide ? 0 : iDisplayStart;
			var iEnd   = bServerSide ? oSettings.aoData.length : iDisplayEnd;

			for ( var j = iStart; j < iEnd; j++ ) {
				var iDataIndex = aiDisplay[j];
				var aoData     = oSettings.aoData[ iDataIndex ];
				if ( aoData.nTr === null ) {
					_fnCreateTr( oSettings, iDataIndex );
				}

				var nRow = aoData.nTr;

				if ( iStripes !== 0 ) {
					var sStripe = asStripeClasses[ iRowCount % iStripes ];
					if ( aoData._sRowStripe != sStripe ) {
						$( nRow ).removeClass( aoData._sRowStripe ).addClass( sStripe );
						aoData._sRowStripe = sStripe;
					}
				}

				_fnCallbackFire(
					oSettings,
					'aoRowCallback',
					null,
					[nRow, aoData._aData, iRowCount, j, iDataIndex]
				);

				anRows.push( nRow );
				iRowCount++;
			}
		} else {

			var sZero = oLang.sZeroRecords;
			if ( oSettings.iDraw == 1 && _fnDataSource( oSettings ) == 'ajax' ) {
				sZero = oLang.sLoadingRecords;
			} else if ( oLang.sEmptyTable && oSettings.fnRecordsTotal() === 0 ) {
				sZero = oLang.sEmptyTable;
			}

			anRows[ 0 ] = $( '<tr/>', { 'class': iStripes ? asStripeClasses[0] : '' } )
				.append(
					$(
						'<td />',
						{
							'valign':  'top',
							'colSpan': _fnVisbleColumns( oSettings ),
							'class':   oSettings.oClasses.sRowEmpty
						}
					).html( sZero )
				)[0];
		}

		_fnCallbackFire(
			oSettings,
			'aoHeaderCallback',
			'header',
			[ $( oSettings.nTHead ).children( 'tr' )[0],
			_fnGetDataMaster( oSettings ),
			iDisplayStart,
			iDisplayEnd,
			aiDisplay ]
		);

		_fnCallbackFire(
			oSettings,
			'aoFooterCallback',
			'footer',
			[ $( oSettings.nTFoot ).children( 'tr' )[0],
			_fnGetDataMaster( oSettings ),
			iDisplayStart,
			iDisplayEnd,
			aiDisplay ]
		);

		var body = $( oSettings.nTBody );

		body.children().detach();
		body.append( $( anRows ) );

		_fnCallbackFire( oSettings, 'aoDrawCallback', 'draw', [oSettings] );

		oSettings.bSorted   = false;
		oSettings.bFiltered = false;
		oSettings.bDrawing  = false;
	}

	function _fnReDraw( settings, holdPosition )
	{
		var
			features = settings.oFeatures,
			sort     = features.bSort,
			filter   = features.bFilter;

		if ( sort ) {
			_fnSort( settings );
		}

		if ( filter ) {
			_fnFilterComplete( settings, settings.oPreviousSearch );
		} else {

			settings.aiDisplay = settings.aiDisplayMaster.slice();
		}

		if ( holdPosition !== true ) {
			settings._iDisplayStart = 0;
		}

		settings._drawHold = holdPosition;

		_fnDraw( settings );

		settings._drawHold = false;
	}

	function _fnAddOptionsHtml ( oSettings )
	{
		var classes  = oSettings.oClasses;
		var table    = $( oSettings.nTable );
		var holding  = $( '<div/>' ).insertBefore( table );
		var features = oSettings.oFeatures;

		var insert = $(
			'<div/>',
			{
				id:      oSettings.sTableId + '_wrapper',
				'class': classes.sWrapper + (oSettings.nTFoot ? '' : ' ' + classes.sNoFooter)
			}
		);

		oSettings.nHolding             = holding[0];
		oSettings.nTableWrapper        = insert[0];
		oSettings.nTableReinsertBefore = oSettings.nTable.nextSibling;

		var aDom = oSettings.sDom.split( '' );
		var featureNode, cOption, nNewNode, cNext, sAttr, j;
		for ( var i = 0; i < aDom.length; i++ ) {
			featureNode = null;
			cOption     = aDom[i];

			if ( cOption == '<' ) {

				nNewNode = $( '<div/>' )[0];

				cNext = aDom[i + 1];
				if ( cNext == "'" || cNext == '"' ) {
					sAttr = "";
					j     = 2;
					while ( aDom[i + j] != cNext ) {
						sAttr += aDom[i + j];
						j++;
					}

					if ( sAttr == "H" ) {
						sAttr = classes.sJUIHeader;
					} else if ( sAttr == "F" ) {
						sAttr = classes.sJUIFooter;
					}

					if ( sAttr.indexOf( '.' ) != -1 ) {
						var aSplit         = sAttr.split( '.' );
						nNewNode.id        = aSplit[0].substr( 1, aSplit[0].length - 1 );
						nNewNode.className = aSplit[1];
					} else if ( sAttr.charAt( 0 ) == "#" ) {
						nNewNode.id = sAttr.substr( 1, sAttr.length - 1 );
					} else {
						nNewNode.className = sAttr;
					}

					i += j;
				}

				insert.append( nNewNode );
				insert = $( nNewNode );
			} else if ( cOption == '>' ) {

				insert = insert.parent();
			} else if ( cOption == 'l' && features.bPaginate && features.bLengthChange ) {

				featureNode = _fnFeatureHtmlLength( oSettings );
			} else if ( cOption == 'f' && features.bFilter ) {

				featureNode = _fnFeatureHtmlFilter( oSettings );
			} else if ( cOption == 'r' && features.bProcessing ) {

				featureNode = _fnFeatureHtmlProcessing( oSettings );
			} else if ( cOption == 't' ) {

				featureNode = _fnFeatureHtmlTable( oSettings );
			} else if ( cOption == 'i' && features.bInfo ) {

				featureNode = _fnFeatureHtmlInfo( oSettings );
			} else if ( cOption == 'p' && features.bPaginate ) {

				featureNode = _fnFeatureHtmlPaginate( oSettings );
			} else if ( DataTable.ext.feature.length !== 0 ) {

				var aoFeatures = DataTable.ext.feature;
				for ( var k = 0, kLen = aoFeatures.length; k < kLen; k++ ) {
					if ( cOption == aoFeatures[k].cFeature ) {
						featureNode = aoFeatures[k].fnInit( oSettings );
						break;
					}
				}
			}

			if ( featureNode ) {
				var aanFeatures = oSettings.aanFeatures;

				if ( ! aanFeatures[cOption] ) {
					aanFeatures[cOption] = [];
				}

				aanFeatures[cOption].push( featureNode );
				insert.append( featureNode );
			}
		}

		holding.replaceWith( insert );
		oSettings.nHolding = null;
	}

	function _fnDetectHeader ( aLayout, nThead )
	{
		var nTrs = $( nThead ).children( 'tr' );
		var nTr, nCell;
		var i, k, l, iLen, jLen, iColShifted, iColumn, iColspan, iRowspan;
		var bUnique;
		var fnShiftCol = function ( a, i, j ) {
			var k = a[i];
			while ( k[j] ) {
				j++;
			}
			return j;
		};

		aLayout.splice( 0, aLayout.length );

		for ( i = 0, iLen = nTrs.length; i < iLen; i++ ) {
			aLayout.push( [] );
		}

		for ( i = 0, iLen = nTrs.length; i < iLen; i++ ) {
			nTr     = nTrs[i];
			iColumn = 0;

			nCell = nTr.firstChild;
			while ( nCell ) {
				if ( nCell.nodeName.toUpperCase() == "TD" ||
					 nCell.nodeName.toUpperCase() == "TH" ) {

					iColspan = nCell.getAttribute( 'colspan' ) * 1;
					iRowspan = nCell.getAttribute( 'rowspan' ) * 1;
					iColspan = ( ! iColspan || iColspan === 0 || iColspan === 1) ? 1 : iColspan;
					iRowspan = ( ! iRowspan || iRowspan === 0 || iRowspan === 1) ? 1 : iRowspan;

					iColShifted = fnShiftCol( aLayout, i, iColumn );

					bUnique = iColspan === 1 ? true : false;

					for ( l = 0; l < iColspan; l++ ) {
						for ( k = 0; k < iRowspan; k++ ) {
							aLayout[i + k][iColShifted + l] = {
								"cell": nCell,
								"unique": bUnique
							};
							aLayout[i + k].nTr              = nTr;
						}
					}
				}
				nCell = nCell.nextSibling;
			}
		}
	}

	function _fnGetUniqueThs ( oSettings, nHeader, aLayout )
	{
		var aReturn = [];
		if ( ! aLayout ) {
			aLayout = oSettings.aoHeader;
			if ( nHeader ) {
				aLayout = [];
				_fnDetectHeader( aLayout, nHeader );
			}
		}

		for ( var i = 0, iLen = aLayout.length; i < iLen; i++ ) {
			for ( var j = 0, jLen = aLayout[i].length; j < jLen; j++ ) {
				if ( aLayout[i][j].unique &&
					 ( ! aReturn[j] || ! oSettings.bSortCellsTop) ) {
					aReturn[j] = aLayout[i][j].cell;
				}
			}
		}

		return aReturn;
	}

	function _fnBuildAjax( oSettings, data, fn )
	{

		_fnCallbackFire( oSettings, 'aoServerParams', 'serverParams', [data] );

		if ( data && $.isArray( data ) ) {
			var tmp      = {};
			var rbracket = /(.*?)\[\]$/;

			$.each(
				data,
				function (key, val) {
					var match = val.name.match( rbracket );

					if ( match ) {

						var name = match[0];

						if ( ! tmp[ name ] ) {
							tmp[ name ] = [];
						}
						tmp[ name ].push( val.value );
					} else {
						tmp[val.name] = val.value;
					}
				}
			);
			data = tmp;
		}

		var ajaxData;
		var ajax     = oSettings.ajax;
		var instance = oSettings.oInstance;

		var callback = function ( json ) {
			_fnCallbackFire( oSettings, null, 'xhr', [oSettings, json, oSettings.jqXHR] );
			fn( json );
		};

		if ( $.isPlainObject( ajax ) && ajax.data ) {
			ajaxData    = ajax.data;
			var newData = typeof ajaxData === 'function' ?
				ajaxData( data, oSettings ) :
				ajaxData;

			data = typeof ajaxData === 'function' && newData ?
				newData :
				$.extend( true, data, newData );

			delete ajax.data;
		}

		var actions_list = ['arf_retrieve_form_entry', 'arf_retrieve_form_incomplete_entry','arf_retrieve_form_data', 'arflite_retrieve_form_entry', 'arflite_retrieve_form_data', 'arforms_retrieve_schdular_status'];

		var action_arr = wp.hooks.applyFilters( 'arf_modify_datatable_action_outside', actions_list );

		var _wpnonce_arforms = jQuery('#arforms_wp_nonce').val();

		if ( action_arr.indexOf( data.action ) > -1 ) {
			var json_data     = {
				"aoData": data
			}
			var filtered_data = "action=" + data.action + "&_wpnonce_arforms="+ _wpnonce_arforms + "&data=" + JSON.stringify( json_data );
			data              = filtered_data;
		}

		var baseAjax = {
			"data": data,
			"success": function (json) {
				var error = json.error || json.sError;
				if ( error ) {
					_fnLog( oSettings, 0, error );
				}

				oSettings.json = json;
				callback( json );
			},
			"dataType": "json",
			"cache": false,
			"type": oSettings.sServerMethod,
			"error": function (xhr, error, thrown) {
				var ret = _fnCallbackFire( oSettings, null, 'xhr', [oSettings, null, oSettings.jqXHR] );

				if ( $.inArray( true, ret ) === -1 ) {
					if ( error == "parsererror" ) {
						_fnLog( oSettings, 0, 'Invalid JSON response', 1 );
					} else if ( xhr.readyState === 4 ) {
						_fnLog( oSettings, 0, 'Ajax error', 7 );
					}
				}

				_fnProcessingDisplay( oSettings, false );
			}
		};

		oSettings.oAjaxData = data;

		_fnCallbackFire( oSettings, null, 'preXhr', [oSettings, data] );

		if ( oSettings.fnServerData ) {

			oSettings.fnServerData.call(
				instance,
				oSettings.sAjaxSource,
				$.map(
					data,
					function (val, key) {
						return { name: key, value: val };
					}
				),
				callback,
				oSettings
			);
		} else if ( oSettings.sAjaxSource || typeof ajax === 'string' ) {

			oSettings.jqXHR = $.ajax(
				$.extend(
					baseAjax,
					{
						url: ajax || oSettings.sAjaxSource
					}
				)
			);
		} else if ( typeof ajax === 'function' ) {

			oSettings.jqXHR = ajax.call( instance, data, callback, oSettings );
		} else {

			oSettings.jqXHR = $.ajax( $.extend( baseAjax, ajax ) );

			ajax.data = ajaxData;
		}
	}

	function _fnAjaxUpdate( settings )
	{
		if ( settings.bAjaxDataGet ) {
			settings.iDraw++;
			_fnProcessingDisplay( settings, true );

			_fnBuildAjax(
				settings,
				_fnAjaxParameters( settings ),
				function(json) {
					_fnAjaxUpdateDraw( settings, json );
				}
			);

			return false;
		}
		return true;
	}

	function _fnAjaxParameters( settings )
	{
		var
			columns       = settings.aoColumns,
			columnCount   = columns.length,
			features      = settings.oFeatures,
			preSearch     = settings.oPreviousSearch,
			preColSearch  = settings.aoPreSearchCols,
			i, data       = [], dataProp, column, columnSearch,
			sort          = _fnSortFlatten( settings ),
			displayStart  = settings._iDisplayStart,
			displayLength = features.bPaginate !== false ?
				settings._iDisplayLength :
				-1;

		var param = function ( name, value ) {
			data.push( { 'name': name, 'value': value } );
		};

		param( 'sEcho',          settings.iDraw );
		param( 'iColumns',       columnCount );
		param( 'sColumns',       _pluck( columns, 'sName' ).join( ',' ) );
		param( 'iDisplayStart',  displayStart );
		param( 'iDisplayLength', displayLength );

		var d = {
			draw:    settings.iDraw,
			columns: [],
			order:   [],
			start:   displayStart,
			length:  displayLength,
			search:  {
				value: preSearch.sSearch,
				regex: preSearch.bRegex
			}
		};

		for ( i = 0; i < columnCount; i++ ) {
			column       = columns[i];
			columnSearch = preColSearch[i];
			dataProp     = typeof column.mData == "function" ? 'function' : column.mData;

			d.columns.push(
				{
					data:       dataProp,
					name:       column.sName,
					searchable: column.bSearchable,
					orderable:  column.bSortable,
					search:     {
						value: columnSearch.sSearch,
						regex: columnSearch.bRegex
					}
				}
			);

			param( "mDataProp_" + i, dataProp );

			if ( features.bFilter ) {
				param( 'sSearch_' + i,     columnSearch.sSearch );
				param( 'bRegex_' + i,      columnSearch.bRegex );
				param( 'bSearchable_' + i, column.bSearchable );
			}

			if ( features.bSort ) {
				param( 'bSortable_' + i, column.bSortable );
			}
		}

		if ( features.bFilter ) {
			param( 'sSearch', preSearch.sSearch );
			param( 'bRegex', preSearch.bRegex );
		}

		if ( features.bSort ) {
			$.each(
				sort,
				function ( i, val ) {
					d.order.push( { column: val.col, dir: val.dir } );

					param( 'iSortCol_' + i, val.col );
					param( 'sSortDir_' + i, val.dir );
				}
			);

			param( 'iSortingCols', sort.length );
		}

		var legacy = DataTable.ext.legacy.ajax;
		if ( legacy === null ) {
			return settings.sAjaxSource ? data : d;
		}

		return legacy ? data : d;
	}

	function _fnAjaxUpdateDraw ( settings, json )
	{

		var compat = function ( old, modern ) {
			return json[old] !== undefined ? json[old] : json[modern];
		};

		var data            = _fnAjaxDataSrc( settings, json );
		var draw            = compat( 'sEcho',                'draw' );
		var recordsTotal    = compat( 'iTotalRecords',        'recordsTotal' );
		var recordsFiltered = compat( 'iTotalDisplayRecords', 'recordsFiltered' );

		if ( draw !== undefined ) {

			if ( draw * 1 < settings.iDraw ) {
				return;
			}
			settings.iDraw = draw * 1;
		}

		_fnClearTable( settings );
		settings._iRecordsTotal   = parseInt( recordsTotal, 10 );
		settings._iRecordsDisplay = parseInt( recordsFiltered, 10 );

		for ( var i = 0, ien = data.length; i < ien; i++ ) {
			_fnAddData( settings, data[i] );
		}
		settings.aiDisplay = settings.aiDisplayMaster.slice();

		settings.bAjaxDataGet = false;
		_fnDraw( settings );

		if ( ! settings._bInitComplete ) {
			_fnInitComplete( settings, json );
		}

		settings.bAjaxDataGet = true;
		_fnProcessingDisplay( settings, false );
	}

	function _fnAjaxDataSrc ( oSettings, json )
	{
		var dataSrc = $.isPlainObject( oSettings.ajax ) && oSettings.ajax.dataSrc !== undefined ?
			oSettings.ajax.dataSrc :
			oSettings.sAjaxDataProp;

		if ( dataSrc === 'data' ) {
			return json.aaData || json[dataSrc];
		}

		return dataSrc !== "" ?
			_fnGetObjectDataFn( dataSrc )( json ) :
			json;
	}

	function _fnFeatureHtmlFilter ( settings )
	{
		var classes        = settings.oClasses;
		var tableId        = settings.sTableId;
		var language       = settings.oLanguage;
		var previousSearch = settings.oPreviousSearch;
		var features       = settings.aanFeatures;
		var input          = '<input type="search" class="' + classes.sFilterInput + '"/>';

		var str = language.sSearch;
		str     = str.match( /_INPUT_/ ) ?
			str.replace( '_INPUT_', input ) :
			str + input;

		var filter = $(
			'<div/>',
			{
				'id': ! features.f ? tableId + '_filter' : null,
				'class': classes.sFilter
			}
		)
			.append( $( '<label/>' ).append( str ) );

		var searchFn = function() {

			var n   = features.f;
			var val = ! this.value ? "" : this.value;

			if ( val != previousSearch.sSearch ) {
				_fnFilterComplete(
					settings,
					{
						"sSearch": val,
						"bRegex": previousSearch.bRegex,
						"bSmart": previousSearch.bSmart ,
						"bCaseInsensitive": previousSearch.bCaseInsensitive
					}
				);

				settings._iDisplayStart = 0;
				_fnDraw( settings );
			}
		};

		var searchDelay = settings.searchDelay !== null ?
			settings.searchDelay :
			_fnDataSource( settings ) === 'ssp' ?
				400 :
				0;

		var jqFilter = $( 'input', filter )
			.val( previousSearch.sSearch )
			.attr( 'placeholder', language.sSearchPlaceholder )
			.on(
				'keyup.DT search.DT input.DT paste.DT cut.DT',
				searchDelay ?
					_fnThrottle( searchFn, searchDelay ) :
					searchFn
			)
			.on(
				'mouseup',
				function(e) {

					setTimeout(
						function () {
							searchFn.call( jqFilter[0] );
						},
						10
					);
				}
			)
			.on(
				'keypress.DT',
				function(e) {

					if ( e.keyCode == 13 ) {
						return false;
					}
				}
			)
			.attr( 'aria-controls', tableId );

		$( settings.nTable ).on(
			'search.dt.DT',
			function ( ev, s ) {
				if ( settings === s ) {

					try {
						if ( jqFilter[0] !== document.activeElement ) {
							jqFilter.val( previousSearch.sSearch );
						}
					} catch ( e ) {
					}
				}
			}
		);

		return filter[0];
	}

	function _fnFilterComplete ( oSettings, oInput, iForce )
	{
		var oPrevSearch  = oSettings.oPreviousSearch;
		var aoPrevSearch = oSettings.aoPreSearchCols;
		var fnSaveFilter = function ( oFilter ) {

			oPrevSearch.sSearch          = oFilter.sSearch;
			oPrevSearch.bRegex           = oFilter.bRegex;
			oPrevSearch.bSmart           = oFilter.bSmart;
			oPrevSearch.bCaseInsensitive = oFilter.bCaseInsensitive;
		};
		var fnRegex      = function ( o ) {

			return o.bEscapeRegex !== undefined ? ! o.bEscapeRegex : o.bRegex;
		};

		_fnColumnTypes( oSettings );

		if ( _fnDataSource( oSettings ) != 'ssp' ) {

			_fnFilter( oSettings, oInput.sSearch, iForce, fnRegex( oInput ), oInput.bSmart, oInput.bCaseInsensitive );
			fnSaveFilter( oInput );

			for ( var i = 0; i < aoPrevSearch.length; i++ ) {
				_fnFilterColumn(
					oSettings,
					aoPrevSearch[i].sSearch,
					i,
					fnRegex( aoPrevSearch[i] ),
					aoPrevSearch[i].bSmart,
					aoPrevSearch[i].bCaseInsensitive
				);
			}

			_fnFilterCustom( oSettings );
		} else {
			fnSaveFilter( oInput );
		}

		oSettings.bFiltered = true;
		_fnCallbackFire( oSettings, null, 'search', [oSettings] );
	}

	function _fnFilterCustom( settings )
	{
		var filters     = DataTable.ext.search;
		var displayRows = settings.aiDisplay;
		var row, rowIdx;

		for ( var i = 0, ien = filters.length; i < ien; i++ ) {
			var rows = [];

			for ( var j = 0, jen = displayRows.length; j < jen; j++ ) {
				rowIdx = displayRows[ j ];
				row    = settings.aoData[ rowIdx ];

				if ( filters[i]( settings, row._aFilterData, rowIdx, row._aData, j ) ) {
					rows.push( rowIdx );
				}
			}

			displayRows.length = 0;
			$.merge( displayRows, rows );
		}
	}

	function _fnFilterColumn ( settings, searchStr, colIdx, regex, smart, caseInsensitive )
	{
		if ( searchStr === '' ) {
			return;
		}

		var data;
		var out      = [];
		var display  = settings.aiDisplay;
		var rpSearch = _fnFilterCreateSearch( searchStr, regex, smart, caseInsensitive );

		for ( var i = 0; i < display.length; i++ ) {
			data = settings.aoData[ display[i] ]._aFilterData[ colIdx ];

			if ( rpSearch.test( data ) ) {
				out.push( display[i] );
			}
		}

		settings.aiDisplay = out;
	}

	function _fnFilter( settings, input, force, regex, smart, caseInsensitive )
	{
		var rpSearch      = _fnFilterCreateSearch( input, regex, smart, caseInsensitive );
		var prevSearch    = settings.oPreviousSearch.sSearch;
		var displayMaster = settings.aiDisplayMaster;
		var display, invalidated, i;
		var filtered = [];

		if ( DataTable.ext.search.length !== 0 ) {
			force = true;
		}

		invalidated = _fnFilterData( settings );

		if ( input.length <= 0 ) {
			settings.aiDisplay = displayMaster.slice();
		} else {

			if ( invalidated ||
				 force ||
				 regex ||
				 prevSearch.length > input.length ||
				 input.indexOf( prevSearch ) !== 0 ||
				 settings.bSorted

			) {
				settings.aiDisplay = displayMaster.slice();
			}

			display = settings.aiDisplay;

			for ( i = 0; i < display.length; i++ ) {
				if ( rpSearch.test( settings.aoData[ display[i] ]._sFilterRow ) ) {
					filtered.push( display[i] );
				}
			}

			settings.aiDisplay = filtered;
		}
	}

	function _fnFilterCreateSearch( search, regex, smart, caseInsensitive )
	{
		search = regex ?
			search :
			_fnEscapeRegex( search );

		if ( smart ) {

			var a = $.map(
				search.match( /"[^"]+"|[^ ]+/g ) || [''],
				function ( word ) {
					if ( word.charAt( 0 ) === '"' ) {
						var m = word.match( /^"(.*)"$/ );
						word  = m ? m[1] : word;
					}

					return word.replace( '"', '' );
				}
			);

			search = '^(?=.*?' + a.join( ')(?=.*?' ) + ').*$';
		}

		return new RegExp( search, caseInsensitive ? 'i' : '' );
	}

	var _fnEscapeRegex = DataTable.util.escapeRegex;

	var __filter_div             = $( '<div>' )[0];
	var __filter_div_textContent = __filter_div.textContent !== undefined;

	function _fnFilterData ( settings )
	{
		var columns = settings.aoColumns;
		var column;
		var i, j, ien, jen, filterData, cellData, row;
		var fomatters      = DataTable.ext.type.search;
		var wasInvalidated = false;

		for ( i = 0, ien = settings.aoData.length; i < ien; i++ ) {
			row = settings.aoData[i];

			if ( ! row._aFilterData ) {
				filterData = [];

				for ( j = 0, jen = columns.length; j < jen; j++ ) {
					column = columns[j];

					if ( column.bSearchable ) {
						cellData = _fnGetCellData( settings, i, j, 'filter' );

						if ( fomatters[ column.sType ] ) {
							cellData = fomatters[ column.sType ]( cellData );
						}

						if ( cellData === null ) {
							cellData = '';
						}

						if ( typeof cellData !== 'string' && cellData.toString ) {
							cellData = cellData.toString();
						}
					} else {
						cellData = '';
					}

					if ( cellData.indexOf && cellData.indexOf( '&' ) !== -1 ) {
						__filter_div.innerHTML = cellData;
						cellData               = __filter_div_textContent ?
							__filter_div.textContent :
							__filter_div.innerText;
					}

					if ( cellData.replace ) {
						cellData = cellData.replace( /[\r\n\u2028]/g, '' );
					}

					filterData.push( cellData );
				}

				row._aFilterData = filterData;
				row._sFilterRow  = filterData.join( '  ' );
				wasInvalidated   = true;
			}
		}

		return wasInvalidated;
	}

	function _fnSearchToCamel ( obj )
	{
		return {
			search:          obj.sSearch,
			smart:           obj.bSmart,
			regex:           obj.bRegex,
			caseInsensitive: obj.bCaseInsensitive
		};
	}

	function _fnSearchToHung ( obj )
	{
		return {
			sSearch:          obj.search,
			bSmart:           obj.smart,
			bRegex:           obj.regex,
			bCaseInsensitive: obj.caseInsensitive
		};
	}

	function _fnFeatureHtmlInfo ( settings )
	{
		var
			tid   = settings.sTableId,
			nodes = settings.aanFeatures.i,
			n     = $(
				'<div/>',
				{
					'class': settings.oClasses.sInfo,
					'id': ! nodes ? tid + '_info' : null
				}
			);

		if ( ! nodes ) {

			settings.aoDrawCallback.push(
				{
					"fn": _fnUpdateInfo,
					"sName": "information"
				}
			);

			n
				.attr( 'role', 'status' )
				.attr( 'aria-live', 'polite' );

			$( settings.nTable ).attr( 'aria-describedby', tid + '_info' );
		}

		return n[0];
	}

	function _fnUpdateInfo ( settings )
	{

		var nodes = settings.aanFeatures.i;
		if ( nodes.length === 0 ) {
			return;
		}

		var
			lang  = settings.oLanguage,
			start = settings._iDisplayStart + 1,
			end   = settings.fnDisplayEnd(),
			max   = settings.fnRecordsTotal(),
			total = settings.fnRecordsDisplay(),
			out   = total ?
				lang.sInfo :
				lang.sInfoEmpty;

		if ( total !== max ) {

			out += ' ' + lang.sInfoFiltered;
		}

		out += lang.sInfoPostFix;
		out  = _fnInfoMacros( settings, out );

		var callback = lang.fnInfoCallback;
		if ( callback !== null ) {
			out = callback.call(
				settings.oInstance,
				settings,
				start,
				end,
				max,
				total,
				out
			);
		}

		$( nodes ).html( out );
	}

	function _fnInfoMacros ( settings, str )
	{

		var
			formatter = settings.fnFormatNumber,
			start     = settings._iDisplayStart + 1,
			len       = settings._iDisplayLength,
			vis       = settings.fnRecordsDisplay(),
			all       = len === -1;

		return str.
			replace( /_START_/g, formatter.call( settings, start ) ).
			replace( /_END_/g,   formatter.call( settings, settings.fnDisplayEnd() ) ).
			replace( /_MAX_/g,   formatter.call( settings, settings.fnRecordsTotal() ) ).
			replace( /_TOTAL_/g, formatter.call( settings, vis ) ).
			replace( /_PAGE_/g,  formatter.call( settings, all ? 1 : Math.ceil( start / len ) ) ).
			replace( /_PAGES_/g, formatter.call( settings, all ? 1 : Math.ceil( vis / len ) ) );
	}

	function _fnInitialise ( settings )
	{
		var i, iLen, iAjaxStart = settings.iInitDisplayStart;
		var columns             = settings.aoColumns, column;
		var features            = settings.oFeatures;
		var deferLoading        = settings.bDeferLoading;

		if ( ! settings.bInitialised ) {
			setTimeout( function(){ _fnInitialise( settings ); }, 200 );
			return;
		}

		_fnAddOptionsHtml( settings );

		_fnBuildHead( settings );
		_fnDrawHead( settings, settings.aoHeader );
		_fnDrawHead( settings, settings.aoFooter );

		_fnProcessingDisplay( settings, true );

		if ( features.bAutoWidth ) {
			_fnCalculateColumnWidths( settings );
		}

		for ( i = 0, iLen = columns.length; i < iLen; i++ ) {
			column = columns[i];

			if ( column.sWidth ) {
				column.nTh.style.width = _fnStringToCss( column.sWidth );
			}
		}

		_fnCallbackFire( settings, null, 'preInit', [settings] );

		_fnReDraw( settings );

		var dataSrc = _fnDataSource( settings );
		if ( dataSrc != 'ssp' || deferLoading ) {

			if ( dataSrc == 'ajax' ) {
				_fnBuildAjax(
					settings,
					[],
					function(json) {
						var aData = _fnAjaxDataSrc( settings, json );

						for ( i = 0; i < aData.length; i++ ) {
							_fnAddData( settings, aData[i] );
						}

						settings.iInitDisplayStart = iAjaxStart;

						_fnReDraw( settings );

						_fnProcessingDisplay( settings, false );
						_fnInitComplete( settings, json );
					},
					settings
				);
			} else {
				_fnProcessingDisplay( settings, false );
				_fnInitComplete( settings );
			}
		}
	}

	function _fnInitComplete ( settings, json )
	{
		settings._bInitComplete = true;

		if ( json || settings.oInit.aaData ) {
			_fnAdjustColumnSizing( settings );
		}

		_fnCallbackFire( settings, null, 'plugin-init', [settings, json] );
		_fnCallbackFire( settings, 'aoInitComplete', 'init', [settings, json] );
	}

	function _fnLengthChange ( settings, val )
	{
		var len                  = parseInt( val, 10 );
		settings._iDisplayLength = len;

		_fnLengthOverflow( settings );

		_fnCallbackFire( settings, null, 'length', [settings, len] );
	}

	function _fnFeatureHtmlLength ( settings )
	{
		var
			classes  = settings.oClasses,
			tableId  = settings.sTableId,
			menu     = settings.aLengthMenu,
			d2       = $.isArray( menu[0] ),
			lengths  = d2 ? menu[0] : menu,
			language = d2 ? menu[1] : menu;

		var select = $(
			'<select/>',
			{
				'name':          tableId + '_length',
				'aria-controls': tableId,
				'class':         classes.sLengthSelect
			}
		);

		for ( var i = 0, ien = lengths.length; i < ien; i++ ) {
			select[0][ i ] = new Option(
				typeof language[i] === 'number' ?
					settings.fnFormatNumber( language[i] ) :
					language[i],
				lengths[i]
			);
		}

		var div = $( '<div><label/></div>' ).addClass( classes.sLength );
		if ( ! settings.aanFeatures.l ) {
			div[0].id = tableId + '_length';
		}

		div.children().append(
			settings.oLanguage.sLengthMenu.replace( '_MENU_', select[0].outerHTML )
		);

		$( 'select', div )
			.val( settings._iDisplayLength )
			.on(
				'change.DT',
				function(e) {
					_fnLengthChange( settings, $( this ).val() );
					_fnDraw( settings );
				}
			);

		$( settings.nTable ).on(
			'length.dt.DT',
			function (e, s, len) {
				if ( settings === s ) {
					$( 'select', div ).val( len );
				}
			}
		);

		return div[0];
	}

	function _fnFeatureHtmlPaginate ( settings )
	{
		var
			type     = settings.sPaginationType,
			plugin   = DataTable.ext.pager[ type ],
			modern   = typeof plugin === 'function',
			redraw   = function( settings ) {
				_fnDraw( settings );
			},
			node     = $( '<div/>' ).addClass( settings.oClasses.sPaging + type )[0],
			features = settings.aanFeatures;

		if ( ! modern ) {
			plugin.fnInit( settings, node, redraw );
		}

		if ( ! features.p ) {
			node.id = settings.sTableId + '_paginate';

			settings.aoDrawCallback.push(
				{
					"fn": function( settings ) {
						if ( modern ) {
							var
								start      = settings._iDisplayStart,
								len        = settings._iDisplayLength,
								visRecords = settings.fnRecordsDisplay(),
								all        = len === -1,
								page       = all ? 0 : Math.ceil( start / len ),
								pages      = all ? 1 : Math.ceil( visRecords / len ),
								buttons    = plugin( page, pages ),
								i, ien;

							for ( i = 0, ien = features.p.length; i < ien; i++ ) {
								_fnRenderer( settings, 'pageButton' )(
									settings,
									features.p[i],
									i,
									buttons,
									page,
									pages
								);
							}
						} else {
							plugin.fnUpdate( settings, redraw );
						}
					},
					"sName": "pagination"
				}
			);
		}

		return node;
	}

	function _fnPageChange ( settings, action, redraw )
	{
		var
			start   = settings._iDisplayStart,
			len     = settings._iDisplayLength,
			records = settings.fnRecordsDisplay();

		if ( records === 0 || len === -1 ) {
			start = 0;
		} else if ( typeof action === "number" ) {
			start = action * len;

			if ( start > records ) {
				start = 0;
			}
		} else if ( action == "first" ) {
			start = 0;
		} else if ( action == "previous" ) {
			start = len >= 0 ?
				start - len :
				0;

			if ( start < 0 ) {
				start = 0;
			}
		} else if ( action == "next" ) {
			if ( start + len < records ) {
				start += len;
			}
		} else if ( action == "last" ) {
			start = Math.floor( (records - 1) / len ) * len;
		} else {
			_fnLog( settings, 0, "Unknown paging action: " + action, 5 );
		}

		var changed             = settings._iDisplayStart !== start;
		settings._iDisplayStart = start;

		if ( changed ) {
			_fnCallbackFire( settings, null, 'page', [settings] );

			if ( redraw ) {
				_fnDraw( settings );
			}
		}

		return changed;
	}

	function _fnFeatureHtmlProcessing ( settings )
	{
		return $(
			'<div/>',
			{
				'id': ! settings.aanFeatures.r ? settings.sTableId + '_processing' : null,
				'class': settings.oClasses.sProcessing
			}
		)
			.html( settings.oLanguage.sProcessing )
			.insertBefore( settings.nTable )[0];
	}

	function _fnProcessingDisplay ( settings, show )
	{
		if ( settings.oFeatures.bProcessing ) {
			$( settings.aanFeatures.r ).css( 'display', show ? 'block' : 'none' );
		}

		_fnCallbackFire( settings, null, 'processing', [settings, show] );
	}

	function _fnFeatureHtmlTable ( settings )
	{
		var table = $( settings.nTable );

		table.attr( 'role', 'grid' );

		var scroll = settings.oScroll;

		if ( scroll.sX === '' && scroll.sY === '' ) {
			return settings.nTable;
		}

		var scrollX     = scroll.sX;
		var scrollY     = scroll.sY;
		var classes     = settings.oClasses;
		var caption     = table.children( 'caption' );
		var captionSide = caption.length ? caption[0]._captionSide : null;
		var headerClone = $( table[0].cloneNode( false ) );
		var footerClone = $( table[0].cloneNode( false ) );
		var footer      = table.children( 'tfoot' );
		var _div        = '<div/>';
		var size        = function ( s ) {
			return ! s ? null : _fnStringToCss( s );
		};

		if ( ! footer.length ) {
			footer = null;
		}

		var scroller = $( _div, { 'class': classes.sScrollWrapper } )
			.append(
				$( _div, { 'class': classes.sScrollHead } )
					.css(
						{
							overflow: 'hidden',
							position: 'relative',
							border: 0,
							width: scrollX ? size( scrollX ) : '100%'
						}
					)
					.append(
						$( _div, { 'class': classes.sScrollHeadInner } )
							.css(
								{
									'box-sizing': 'content-box',
									width: scroll.sXInner || '100%'
								}
							)
							.append(
								headerClone
									.removeAttr( 'id' )
									.css( 'margin-left', 0 )
									.append( captionSide === 'top' ? caption : null )
									.append(
										table.children( 'thead' )
									)
							)
					)
			)
			.append(
				$( _div, { 'class': classes.sScrollBody } )
					.css(
						{
							position: 'relative',
							overflow: 'auto',
							width: size( scrollX )
						}
					)
					.append( table )
			);

		if ( footer ) {
			scroller.append(
				$( _div, { 'class': classes.sScrollFoot } )
					.css(
						{
							overflow: 'hidden',
							border: 0,
							width: scrollX ? size( scrollX ) : '100%'
						}
					)
					.append(
						$( _div, { 'class': classes.sScrollFootInner } )
							.append(
								footerClone
									.removeAttr( 'id' )
									.css( 'margin-left', 0 )
									.append( captionSide === 'bottom' ? caption : null )
									.append(
										table.children( 'tfoot' )
									)
							)
					)
			);
		}

		var children   = scroller.children();
		var scrollHead = children[0];
		var scrollBody = children[1];
		var scrollFoot = footer ? children[2] : null;

		if ( scrollX ) {
			$( scrollBody ).on(
				'scroll.DT',
				function (e) {
					var scrollLeft = this.scrollLeft;

					scrollHead.scrollLeft = scrollLeft;

					if ( footer ) {
						scrollFoot.scrollLeft = scrollLeft;
					}
				}
			);
		}

		$( scrollBody ).css( 'max-height', scrollY );
		if ( ! scroll.bCollapse) {
			$( scrollBody ).css( 'height', scrollY );
		}

		settings.nScrollHead = scrollHead;
		settings.nScrollBody = scrollBody;
		settings.nScrollFoot = scrollFoot;

		settings.aoDrawCallback.push(
			{
				"fn": _fnScrollDraw,
				"sName": "scrolling"
			}
		);

		return scroller[0];
	}

	function _fnScrollDraw ( settings )
	{

		var
			scroll              = settings.oScroll,
			scrollX             = scroll.sX,
			scrollXInner        = scroll.sXInner,
			scrollY             = scroll.sY,
			barWidth            = scroll.iBarWidth,
			divHeader           = $( settings.nScrollHead ),
			divHeaderStyle      = divHeader[0].style,
			divHeaderInner      = divHeader.children( 'div' ),
			divHeaderInnerStyle = divHeaderInner[0].style,
			divHeaderTable      = divHeaderInner.children( 'table' ),
			divBodyEl           = settings.nScrollBody,
			divBody             = $( divBodyEl ),
			divBodyStyle        = divBodyEl.style,
			divFooter           = $( settings.nScrollFoot ),
			divFooterInner      = divFooter.children( 'div' ),
			divFooterTable      = divFooterInner.children( 'table' ),
			header              = $( settings.nTHead ),
			table               = $( settings.nTable ),
			tableEl             = table[0],
			tableStyle          = tableEl.style,
			footer              = settings.nTFoot ? $( settings.nTFoot ) : null,
			browser             = settings.oBrowser,
			ie67                = browser.bScrollOversize,
			dtHeaderCells       = _pluck( settings.aoColumns, 'nTh' ),
			headerTrgEls, footerTrgEls,
			headerSrcEls, footerSrcEls,
			headerCopy, footerCopy,
			headerWidths        = [], footerWidths = [],
			headerContent       = [], footerContent = [],
			idx, correction, sanityWidth,
			zeroOut             = function(nSizer) {
				var style               = nSizer.style;
				style.paddingTop        = "0";
				style.paddingBottom     = "0";
				style.borderTopWidth    = "0";
				style.borderBottomWidth = "0";
				style.height            = 0;
			};

		var scrollBarVis = divBodyEl.scrollHeight > divBodyEl.clientHeight;

		if ( settings.scrollBarVis !== scrollBarVis && settings.scrollBarVis !== undefined ) {
			settings.scrollBarVis = scrollBarVis;
			_fnAdjustColumnSizing( settings );
			return;
		} else {
			settings.scrollBarVis = scrollBarVis;
		}

		table.children( 'thead, tfoot' ).remove();

		if ( footer ) {
			footerCopy   = footer.clone().prependTo( table );
			footerTrgEls = footer.find( 'tr' );
			footerSrcEls = footerCopy.find( 'tr' );
		}

		headerCopy   = header.clone().prependTo( table );
		headerTrgEls = header.find( 'tr' );
		headerSrcEls = headerCopy.find( 'tr' );
		headerCopy.find( 'th, td' ).removeAttr( 'tabindex' );

		if ( ! scrollX ) {
			divBodyStyle.width       = '100%';
			divHeader[0].style.width = '100%';
		}

		$.each(
			_fnGetUniqueThs( settings, headerCopy ),
			function ( i, el ) {
				idx            = _fnVisibleToColumnIndex( settings, i );
				el.style.width = settings.aoColumns[idx].sWidth;
			}
		);

		if ( footer ) {
			_fnApplyToChildren(
				function(n) {
					n.style.width = "";
				},
				footerSrcEls
			);
		}

		sanityWidth = table.outerWidth();
		if ( scrollX === "" ) {

			tableStyle.width = "100%";

			if ( ie67 && (table.find( 'tbody' ).height() > divBodyEl.offsetHeight ||
				divBody.css( 'overflow-y' ) == "scroll")
			) {
				tableStyle.width = _fnStringToCss( table.outerWidth() - barWidth );
			}

			sanityWidth = table.outerWidth();
		} else if ( scrollXInner !== "" ) {

			tableStyle.width = _fnStringToCss( scrollXInner );

			sanityWidth = table.outerWidth();
		}

		_fnApplyToChildren( zeroOut, headerSrcEls );

		_fnApplyToChildren(
			function(nSizer) {
				headerContent.push( nSizer.innerHTML );
				headerWidths.push( _fnStringToCss( $( nSizer ).css( 'width' ) ) );
			},
			headerSrcEls
		);

		_fnApplyToChildren(
			function(nToSize, i) {

				if ( $.inArray( nToSize, dtHeaderCells ) !== -1 ) {
					  nToSize.style.width = headerWidths[i];
				}
			},
			headerTrgEls
		);

		$( headerSrcEls ).height( 0 );

		if ( footer ) {
			_fnApplyToChildren( zeroOut, footerSrcEls );

			_fnApplyToChildren(
				function(nSizer) {
					footerContent.push( nSizer.innerHTML );
					footerWidths.push( _fnStringToCss( $( nSizer ).css( 'width' ) ) );
				},
				footerSrcEls
			);

			_fnApplyToChildren(
				function(nToSize, i) {
					nToSize.style.width = footerWidths[i];
				},
				footerTrgEls
			);

			$( footerSrcEls ).height( 0 );
		}

		_fnApplyToChildren(
			function(nSizer, i) {
				nSizer.innerHTML                    = '<div class="dataTables_sizing">' + headerContent[i] + '</div>';
				nSizer.childNodes[0].style.height   = "0";
				nSizer.childNodes[0].style.overflow = "hidden";
				nSizer.style.width                  = headerWidths[i];
			},
			headerSrcEls
		);

		if ( footer ) {
			_fnApplyToChildren(
				function(nSizer, i) {
					nSizer.innerHTML                    = '<div class="dataTables_sizing">' + footerContent[i] + '</div>';
					nSizer.childNodes[0].style.height   = "0";
					nSizer.childNodes[0].style.overflow = "hidden";
					nSizer.style.width                  = footerWidths[i];
				},
				footerSrcEls
			);
		}

		if ( table.outerWidth() < sanityWidth ) {

			correction = ((divBodyEl.scrollHeight > divBodyEl.offsetHeight ||
				divBody.css( 'overflow-y' ) == "scroll")) ?
					sanityWidth + barWidth :
					sanityWidth;

			if ( ie67 && (divBodyEl.scrollHeight >
				divBodyEl.offsetHeight || divBody.css( 'overflow-y' ) == "scroll")
			) {
				tableStyle.width = _fnStringToCss( correction - barWidth );
			}

			if ( scrollX === "" || scrollXInner !== "" ) {
				_fnLog( settings, 1, 'Possible column misalignment', 6 );
			}
		} else {
			correction = '100%';
		}

		divBodyStyle.width   = _fnStringToCss( correction );
		divHeaderStyle.width = _fnStringToCss( correction );

		if ( footer ) {
			settings.nScrollFoot.style.width = _fnStringToCss( correction );
		}

		if ( ! scrollY ) {

			if ( ie67 ) {
				divBodyStyle.height = _fnStringToCss( tableEl.offsetHeight + barWidth );
			}
		}

		var iOuterWidth               = table.outerWidth();
		divHeaderTable[0].style.width = _fnStringToCss( iOuterWidth );
		divHeaderInnerStyle.width     = _fnStringToCss( iOuterWidth );

		var bScrolling                 = table.height() > divBodyEl.clientHeight || divBody.css( 'overflow-y' ) == "scroll";
		var padding                    = 'padding' + (browser.bScrollbarLeft ? 'Left' : 'Right' );
		divHeaderInnerStyle[ padding ] = bScrolling ? barWidth + "px" : "0px";

		if ( footer ) {
			divFooterTable[0].style.width    = _fnStringToCss( iOuterWidth );
			divFooterInner[0].style.width    = _fnStringToCss( iOuterWidth );
			divFooterInner[0].style[padding] = bScrolling ? barWidth + "px" : "0px";
		}

		table.children( 'colgroup' ).insertBefore( table.children( 'thead' ) );

		divBody.trigger( 'scroll' );

		if ( (settings.bSorted || settings.bFiltered) && ! settings._drawHold ) {
			divBodyEl.scrollTop = 0;
		}
	}

	function _fnApplyToChildren( fn, an1, an2 )
	{
		var index = 0, i = 0, iLen = an1.length;
		var nNode1, nNode2;

		while ( i < iLen ) {
			nNode1 = an1[i].firstChild;
			nNode2 = an2 ? an2[i].firstChild : null;

			while ( nNode1 ) {
				if ( nNode1.nodeType === 1 ) {
					if ( an2 ) {
						fn( nNode1, nNode2, index );
					} else {
						fn( nNode1, index );
					}

					index++;
				}

				nNode1 = nNode1.nextSibling;
				nNode2 = an2 ? nNode2.nextSibling : null;
			}

			i++;
		}
	}

	var __re_html_remove = /<.*?>/g;

	function _fnCalculateColumnWidths ( oSettings )
	{
		var
			table          = oSettings.nTable,
			columns        = oSettings.aoColumns,
			scroll         = oSettings.oScroll,
			scrollY        = scroll.sY,
			scrollX        = scroll.sX,
			scrollXInner   = scroll.sXInner,
			columnCount    = columns.length,
			visibleColumns = _fnGetColumns( oSettings, 'bVisible' ),
			headerCells    = $( 'th', oSettings.nTHead ),
			tableWidthAttr = table.getAttribute( 'width' ),
			tableContainer = table.parentNode,
			userInputs     = false,
			i, column, columnIdx, width, outerWidth,
			browser        = oSettings.oBrowser,
			ie67           = browser.bScrollOversize;

		var styleWidth = table.style.width;
		if ( styleWidth && styleWidth.indexOf( '%' ) !== -1 ) {
			tableWidthAttr = styleWidth;
		}

		for ( i = 0; i < visibleColumns.length; i++ ) {
			column = columns[ visibleColumns[i] ];

			if ( column.sWidth !== null ) {
				column.sWidth = _fnConvertToWidth( column.sWidthOrig, tableContainer );

				userInputs = true;
			}
		}

		if ( ie67 || ! userInputs && ! scrollX && ! scrollY &&
			 columnCount == _fnVisbleColumns( oSettings ) &&
			 columnCount == headerCells.length
		) {
			for ( i = 0; i < columnCount; i++ ) {
				var colIdx = _fnVisibleToColumnIndex( oSettings, i );

				if ( colIdx !== null ) {
					columns[ colIdx ].sWidth = _fnStringToCss( headerCells.eq( i ).width() );
				}
			}
		} else {

			var tmpTable = $( table ).clone()
				.css( 'visibility', 'hidden' )
				.removeAttr( 'id' );

			tmpTable.find( 'tbody tr' ).remove();
			var tr = $( '<tr/>' ).appendTo( tmpTable.find( 'tbody' ) );

			tmpTable.find( 'thead, tfoot' ).remove();
			tmpTable
				.append( $( oSettings.nTHead ).clone() )
				.append( $( oSettings.nTFoot ).clone() );

			tmpTable.find( 'tfoot th, tfoot td' ).css( 'width', '' );

			headerCells = _fnGetUniqueThs( oSettings, tmpTable.find( 'thead' )[0] );

			for ( i = 0; i < visibleColumns.length; i++ ) {
				column = columns[ visibleColumns[i] ];

				headerCells[i].style.width = column.sWidthOrig !== null && column.sWidthOrig !== '' ?
					_fnStringToCss( column.sWidthOrig ) :
					'';

				if ( column.sWidthOrig && scrollX ) {
					$( headerCells[i] ).append(
						$( '<div/>' ).css(
							{
								width: column.sWidthOrig,
								margin: 0,
								padding: 0,
								border: 0,
								height: 1
							}
						)
					);
				}
			}

			if ( oSettings.aoData.length ) {
				for ( i = 0; i < visibleColumns.length; i++ ) {
					columnIdx = visibleColumns[i];
					column    = columns[ columnIdx ];

					$( _fnGetWidestNode( oSettings, columnIdx ) )
						.clone( false )
						.append( column.sContentPadding )
						.appendTo( tr );
				}
			}

			$( '[name]', tmpTable ).removeAttr( 'name' );

			var holder = $( '<div/>' ).css(
				scrollX || scrollY ?
					{
						position: 'absolute',
						top: 0,
						left: 0,
						height: 1,
						right: 0,
						overflow: 'hidden'
					} :
					{}
			)
				.append( tmpTable )
				.appendTo( tableContainer );

			if ( scrollX && scrollXInner ) {
				tmpTable.width( scrollXInner );
			} else if ( scrollX ) {
				tmpTable.css( 'width', 'auto' );
				tmpTable.removeAttr( 'width' );

				if ( tmpTable.width() < tableContainer.clientWidth && tableWidthAttr ) {
					tmpTable.width( tableContainer.clientWidth );
				}
			} else if ( scrollY ) {
				tmpTable.width( tableContainer.clientWidth );
			} else if ( tableWidthAttr ) {
				tmpTable.width( tableWidthAttr );
			}

			var total = 0;
			for ( i = 0; i < visibleColumns.length; i++ ) {
				var cell   = $( headerCells[i] );
				var border = cell.outerWidth() - cell.width();

				var bounding = browser.bBounding ?
					Math.ceil( headerCells[i].getBoundingClientRect().width ) :
					cell.outerWidth();

				total += bounding;

				columns[ visibleColumns[i] ].sWidth = _fnStringToCss( bounding - border );
			}

			table.style.width = _fnStringToCss( total );

			holder.remove();
		}

		if ( tableWidthAttr ) {
			table.style.width = _fnStringToCss( tableWidthAttr );
		}

		if ( (tableWidthAttr || scrollX) && ! oSettings._reszEvt ) {
			var bindResize = function () {
				$( window ).on(
					'resize.DT-' + oSettings.sInstance,
					_fnThrottle(
						function () {
							_fnAdjustColumnSizing( oSettings );
						}
					)
				);
			};

			if ( ie67 ) {
				setTimeout( bindResize, 1000 );
			} else {
				bindResize();
			}

			oSettings._reszEvt = true;
		}
	}

	var _fnThrottle = DataTable.util.throttle;

	function _fnConvertToWidth ( width, parent )
	{
		if ( ! width ) {
			return 0;
		}

		var n = $( '<div/>' )
			.css( 'width', _fnStringToCss( width ) )
			.appendTo( parent || document.body );

		var val = n[0].offsetWidth;
		n.remove();

		return val;
	}

	function _fnGetWidestNode( settings, colIdx )
	{
		var idx = _fnGetMaxLenString( settings, colIdx );
		if ( idx < 0 ) {
			return null;
		}

		var data = settings.aoData[ idx ];
		return ! data.nTr ?
			$( '<td/>' ).html( _fnGetCellData( settings, idx, colIdx, 'display' ) )[0] :
			data.anCells[ colIdx ];
	}

	function _fnGetMaxLenString( settings, colIdx )
	{
		var s, max = -1, maxIdx = -1;

		for ( var i = 0, ien = settings.aoData.length; i < ien; i++ ) {
			s = _fnGetCellData( settings, i, colIdx, 'display' ) + '';
			s = s.replace( __re_html_remove, '' );
			s = s.replace( /&nbsp;/g, ' ' );

			if ( s.length > max ) {
				max    = s.length;
				maxIdx = i;
			}
		}

		return maxIdx;
	}

	function _fnStringToCss( s )
	{
		if ( s === null ) {
			return '0px';
		}

		if ( typeof s == 'number' ) {
			return s < 0 ?
				'0px' :
				s + 'px';
		}

		return s.match( /\d$/ ) ?
			s + 'px' :
			s;
	}

	function _fnSortFlatten ( settings )
	{
		var
			i, iLen, k, kLen,
			aSort      = [],
			aiOrig     = [],
			aoColumns  = settings.aoColumns,
			aDataSort, iCol, sType, srcCol,
			fixed      = settings.aaSortingFixed,
			fixedObj   = $.isPlainObject( fixed ),
			nestedSort = [],
			add        = function ( a ) {
				if ( a.length && ! $.isArray( a[0] ) ) {

					nestedSort.push( a );
				} else {

					$.merge( nestedSort, a );
				}
			};

		if ( $.isArray( fixed ) ) {
			add( fixed );
		}

		if ( fixedObj && fixed.pre ) {
			add( fixed.pre );
		}

		add( settings.aaSorting );

		if (fixedObj && fixed.post ) {
			add( fixed.post );
		}

		for ( i = 0; i < nestedSort.length; i++ ) {
			srcCol    = nestedSort[i][0];
			aDataSort = aoColumns[ srcCol ].aDataSort;

			for ( k = 0, kLen = aDataSort.length; k < kLen; k++ ) {
				iCol  = aDataSort[k];
				sType = aoColumns[ iCol ].sType || 'string';

				if ( nestedSort[i]._idx === undefined ) {
					nestedSort[i]._idx = $.inArray( nestedSort[i][1], aoColumns[iCol].asSorting );
				}

				aSort.push(
					{
						src:       srcCol,
						col:       iCol,
						dir:       nestedSort[i][1],
						index:     nestedSort[i]._idx,
						type:      sType,
						formatter: DataTable.ext.type.order[ sType + "-pre" ]
					}
				);
			}
		}

		return aSort;
	}

	function _fnSort ( oSettings )
	{
		var
			i, ien, iLen, j, jLen, k, kLen,
			sDataType, nTh,
			aiOrig        = [],
			oExtSort      = DataTable.ext.type.order,
			aoData        = oSettings.aoData,
			aoColumns     = oSettings.aoColumns,
			aDataSort, data, iCol, sType, oSort,
			formatters    = 0,
			sortCol,
			displayMaster = oSettings.aiDisplayMaster,
			aSort;

		_fnColumnTypes( oSettings );

		aSort = _fnSortFlatten( oSettings );

		for ( i = 0, ien = aSort.length; i < ien; i++ ) {
			sortCol = aSort[i];

			if ( sortCol.formatter ) {
				formatters++;
			}

			_fnSortData( oSettings, sortCol.col );
		}

		if ( _fnDataSource( oSettings ) != 'ssp' && aSort.length !== 0 ) {

			for ( i = 0, iLen = displayMaster.length; i < iLen; i++ ) {
				aiOrig[ displayMaster[i] ] = i;
			}

			if ( formatters === aSort.length ) {

				displayMaster.sort(
					function ( a, b ) {
						var
						x, y, k, test, sort,
						len   = aSort.length,
						dataA = aoData[a]._aSortData,
						dataB = aoData[b]._aSortData;

						for ( k = 0; k < len; k++ ) {
							  sort = aSort[k];

							  x = dataA[ sort.col ];
							  y = dataB[ sort.col ];

							  test = x < y ? -1 : x > y ? 1 : 0;
							if ( test !== 0 ) {
								return sort.dir === 'asc' ? test : -test;
							}
						}

						x = aiOrig[a];
						y = aiOrig[b];
						return x < y ? -1 : x > y ? 1 : 0;
					}
				);
			} else {

				displayMaster.sort(
					function ( a, b ) {
						var
						x, y, k, l, test, sort, fn,
						len   = aSort.length,
						dataA = aoData[a]._aSortData,
						dataB = aoData[b]._aSortData;

						for ( k = 0; k < len; k++ ) {
							  sort = aSort[k];

							  x = dataA[ sort.col ];
							  y = dataB[ sort.col ];

							  fn   = oExtSort[ sort.type + "-" + sort.dir ] || oExtSort[ "string-" + sort.dir ];
							  test = fn( x, y );
							if ( test !== 0 ) {
								return test;
							}
						}

						x = aiOrig[a];
						y = aiOrig[b];
						return x < y ? -1 : x > y ? 1 : 0;
					}
				);
			}
		}

		oSettings.bSorted = true;
	}

	function _fnSortAria ( settings )
	{
		var label;
		var nextSort;
		var columns = settings.aoColumns;
		var aSort   = _fnSortFlatten( settings );
		var oAria   = settings.oLanguage.oAria;

		for ( var i = 0, iLen = columns.length; i < iLen; i++ ) {
			var col       = columns[i];
			var asSorting = col.asSorting;
			var sTitle    = col.sTitle.replace( /<.*?>/g, "" );
			var th        = col.nTh;

			th.removeAttribute( 'aria-sort' );

			if ( col.bSortable ) {
				if ( aSort.length > 0 && aSort[0].col == i ) {
					th.setAttribute( 'aria-sort', aSort[0].dir == "asc" ? "ascending" : "descending" );
					nextSort = asSorting[ aSort[0].index + 1 ] || asSorting[0];
				} else {
					nextSort = asSorting[0];
				}

				label = sTitle + ( nextSort === "asc" ?
					oAria.sSortAscending :
					oAria.sSortDescending
				);
			} else {
				label = sTitle;
			}

			th.setAttribute( 'aria-label', label );
		}
	}

	function _fnSortListener ( settings, colIdx, append, callback )
	{
		var col       = settings.aoColumns[ colIdx ];
		var sorting   = settings.aaSorting;
		var asSorting = col.asSorting;
		var nextSortIdx;
		var next = function ( a, overflow ) {
			var idx = a._idx;
			if ( idx === undefined ) {
				idx = $.inArray( a[1], asSorting );
			}

			return idx + 1 < asSorting.length ?
				idx + 1 :
				overflow ?
					null :
					0;
		};

		if ( typeof sorting[0] === 'number' ) {
			sorting = settings.aaSorting = [ sorting ];
		}

		if ( append && settings.oFeatures.bSortMulti ) {

			var sortIdx = $.inArray( colIdx, _pluck( sorting, '0' ) );

			if ( sortIdx !== -1 ) {

				nextSortIdx = next( sorting[sortIdx], true );

				if ( nextSortIdx === null && sorting.length === 1 ) {
					nextSortIdx = 0;
				}

				if ( nextSortIdx === null ) {
					sorting.splice( sortIdx, 1 );
				} else {
					sorting[sortIdx][1]   = asSorting[ nextSortIdx ];
					sorting[sortIdx]._idx = nextSortIdx;
				}
			} else {

				sorting.push( [ colIdx, asSorting[0], 0 ] );
				sorting[sorting.length - 1]._idx = 0;
			}
		} else if ( sorting.length && sorting[0][0] == colIdx ) {

			nextSortIdx = next( sorting[0] );

			sorting.length  = 1;
			sorting[0][1]   = asSorting[ nextSortIdx ];
			sorting[0]._idx = nextSortIdx;
		} else {

			sorting.length = 0;
			sorting.push( [ colIdx, asSorting[0] ] );
			sorting[0]._idx = 0;
		}

		_fnReDraw( settings );

		if ( typeof callback == 'function' ) {
			callback( settings );
		}
	}

	function _fnSortAttachListener ( settings, attachTo, colIdx, callback )
	{
		var col = settings.aoColumns[ colIdx ];

		_fnBindAction(
			attachTo,
			{},
			function (e) {

				if ( col.bSortable === false ) {
					return;
				}

				if ( settings.oFeatures.bProcessing ) {
					_fnProcessingDisplay( settings, true );

					setTimeout(
						function() {
							_fnSortListener( settings, colIdx, e.shiftKey, callback );

							if ( _fnDataSource( settings ) !== 'ssp' ) {
								  _fnProcessingDisplay( settings, false );
							}
						},
						0
					);
				} else {
					_fnSortListener( settings, colIdx, e.shiftKey, callback );
				}
			}
		);
	}

	function _fnSortingClasses( settings )
	{
		var oldSort   = settings.aLastSort;
		var sortClass = settings.oClasses.sSortColumn;
		var sort      = _fnSortFlatten( settings );
		var features  = settings.oFeatures;
		var i, ien, colIdx;

		if ( features.bSort && features.bSortClasses ) {

			for ( i = 0, ien = oldSort.length; i < ien; i++ ) {
				colIdx = oldSort[i].src;

				$( _pluck( settings.aoData, 'anCells', colIdx ) )
					.removeClass( sortClass + (i < 2 ? i + 1 : 3) );
			}

			for ( i = 0, ien = sort.length; i < ien; i++ ) {
				colIdx = sort[i].src;

				$( _pluck( settings.aoData, 'anCells', colIdx ) )
					.addClass( sortClass + (i < 2 ? i + 1 : 3) );
			}
		}

		settings.aLastSort = sort;
	}

	function _fnSortData( settings, idx )
	{

		var column     = settings.aoColumns[ idx ];
		var customSort = DataTable.ext.order[ column.sSortDataType ];
		var customData;

		if ( customSort ) {
			customData = customSort.call(
				settings.oInstance,
				settings,
				idx,
				_fnColumnIndexToVisible( settings, idx )
			);
		}

		var row, cellData;
		var formatter = DataTable.ext.type.order[ column.sType + "-pre" ];

		for ( var i = 0, ien = settings.aoData.length; i < ien; i++ ) {
			row = settings.aoData[i];

			if ( ! row._aSortData ) {
				row._aSortData = [];
			}

			if ( ! row._aSortData[idx] || customSort ) {
				cellData = customSort ?
					customData[i] :
					_fnGetCellData( settings, i, idx, 'sort' );

				row._aSortData[ idx ] = formatter ?
					formatter( cellData ) :
					cellData;
			}
		}
	}

	function _fnSaveState ( settings )
	{
		if ( ! settings.oFeatures.bStateSave || settings.bDestroying ) {
			return;
		}

		var state = {
			time:    +new Date(),
			start:   settings._iDisplayStart,
			length:  settings._iDisplayLength,
			order:   $.extend( true, [], settings.aaSorting ),
			search:  _fnSearchToCamel( settings.oPreviousSearch ),
			columns: $.map(
				settings.aoColumns,
				function ( col, i ) {
					return {
						visible: col.bVisible,
						search: _fnSearchToCamel( settings.aoPreSearchCols[i] )
					};
				}
			)
		};

		_fnCallbackFire( settings, "aoStateSaveParams", 'stateSaveParams', [settings, state] );

		settings.oSavedState = state;
		settings.fnStateSaveCallback.call( settings.oInstance, settings, state );
	}

	function _fnLoadState ( settings, oInit, callback )
	{
		var i, ien;
		var columns = settings.aoColumns;
		var loaded  = function ( s ) {
			if ( ! s || ! s.time ) {
				callback();
				return;
			}

			var abStateLoad = _fnCallbackFire( settings, 'aoStateLoadParams', 'stateLoadParams', [settings, s] );
			if ( $.inArray( false, abStateLoad ) !== -1 ) {
				callback();
				return;
			}

			var duration = settings.iStateDuration;
			if ( duration > 0 && s.time < +new Date() - (duration * 1000) ) {
				callback();
				return;
			}

			if ( s.columns && columns.length !== s.columns.length ) {
				callback();
				return;
			}

			settings.oLoadedState = $.extend( true, {}, s );

			if ( s.start !== undefined ) {
				settings._iDisplayStart    = s.start;
				settings.iInitDisplayStart = s.start;
			}
			if ( s.length !== undefined ) {
				settings._iDisplayLength = s.length;
			}

			if ( s.order !== undefined ) {
				settings.aaSorting = [];
				$.each(
					s.order,
					function ( i, col ) {
						settings.aaSorting.push(
							col[0] >= columns.length ?
							[ 0, col[1] ] :
							col
						);
					}
				);
			}

			if ( s.search !== undefined ) {
				$.extend( settings.oPreviousSearch, _fnSearchToHung( s.search ) );
			}

			if ( s.columns ) {
				for ( i = 0, ien = s.columns.length; i < ien; i++ ) {
					var col = s.columns[i];

					if ( col.visible !== undefined ) {
						columns[i].bVisible = col.visible;
					}

					if ( col.search !== undefined ) {
						$.extend( settings.aoPreSearchCols[i], _fnSearchToHung( col.search ) );
					}
				}
			}

			_fnCallbackFire( settings, 'aoStateLoaded', 'stateLoaded', [settings, s] );
			callback();
		};

		if ( ! settings.oFeatures.bStateSave ) {
			callback();
			return;
		}

		var state = settings.fnStateLoadCallback.call( settings.oInstance, settings, loaded );

		if ( state !== undefined ) {
			loaded( state );
		}

	}

	function _fnSettingsFromNode ( table )
	{
		var settings = DataTable.settings;
		var idx      = $.inArray( table, _pluck( settings, 'nTable' ) );

		return idx !== -1 ?
			settings[ idx ] :
			null;
	}

	function _fnLog( settings, level, msg, tn )
	{
		msg = 'DataTables warning: ' +
			(settings ? 'table id=' + settings.sTableId + ' - ' : '') + msg;

		if ( tn ) {
			msg += '. For more information about this error, please see ' +
			'http://datatables.net/tn/' + tn;
		}

		if ( ! level  ) {

			var ext  = DataTable.ext;
			var type = ext.sErrMode || ext.errMode;

			if ( settings ) {
				_fnCallbackFire( settings, null, 'error', [ settings, tn, msg ] );
			}

			if ( type == 'alert' ) {
				alert( msg );
			} else if ( type == 'throw' ) {
				throw new Error( msg );
			} else if ( typeof type == 'function' ) {
				type( settings, tn, msg );
			}
		} else if ( window.console && console.log ) {
			console.log( msg );
		}
	}

	function _fnMap( ret, src, name, mappedName )
	{
		if ( $.isArray( name ) ) {
			$.each(
				name,
				function (i, val) {
					if ( $.isArray( val ) ) {
						_fnMap( ret, src, val[0], val[1] );
					} else {
						_fnMap( ret, src, val );
					}
				}
			);

			return;
		}

		if ( mappedName === undefined ) {
			mappedName = name;
		}

		if ( src[name] !== undefined ) {
			ret[mappedName] = src[name];
		}
	}

	function _fnExtend( out, extender, breakRefs )
	{
		var val;

		for ( var prop in extender ) {
			if ( extender.hasOwnProperty( prop ) ) {
				val = extender[prop];

				if ( $.isPlainObject( val ) ) {
					if ( ! $.isPlainObject( out[prop] ) ) {
						out[prop] = {};
					}
					$.extend( true, out[prop], val );
				} else if ( breakRefs && prop !== 'data' && prop !== 'aaData' && $.isArray( val ) ) {
					out[prop] = val.slice();
				} else {
					out[prop] = val;
				}
			}
		}

		return out;
	}

	function _fnBindAction( n, oData, fn )
	{
		$( n )
			.on(
				'click.DT',
				oData,
				function (e) {
					$( n ).trigger( 'blur' );
					fn( e );
				}
			)
			.on(
				'keypress.DT',
				oData,
				function (e){
					if ( e.which === 13 ) {
						e.preventDefault();
						fn( e );
					}
				}
			)
			.on(
				'selectstart.DT',
				function () {

					return false;
				}
			);
	}

	function _fnCallbackReg( oSettings, sStore, fn, sName )
	{
		if ( fn ) {
			oSettings[sStore].push(
				{
					"fn": fn,
					"sName": sName
				}
			);
		}
	}

	function _fnCallbackFire( settings, callbackArr, eventName, args )
	{
		var ret = [];

		if ( callbackArr ) {
			ret = $.map(
				settings[callbackArr].slice().reverse(),
				function (val, i) {
					return val.fn.apply( settings.oInstance, args );
				}
			);
		}

		if ( eventName !== null ) {
			var e = $.Event( eventName + '.dt' );

			$( settings.nTable ).trigger( e, args );

			ret.push( e.result );
		}

		return ret;
	}

	function _fnLengthOverflow ( settings )
	{
		var
			start = settings._iDisplayStart,
			end   = settings.fnDisplayEnd(),
			len   = settings._iDisplayLength;

		if ( start >= end ) {
			start = end - len;
		}

		start -= (start % len);

		if ( len === -1 || start < 0 ) {
			start = 0;
		}

		settings._iDisplayStart = start;
	}

	function _fnRenderer( settings, type )
	{
		var renderer = settings.renderer;
		var host     = DataTable.ext.renderer[type];

		if ( $.isPlainObject( renderer ) && renderer[type] ) {

			return host[renderer[type]] || host._;
		} else if ( typeof renderer === 'string' ) {

			return host[renderer] || host._;
		}

		return host._;
	}

	function _fnDataSource ( settings )
	{
		if ( settings.oFeatures.bServerSide ) {
			return 'ssp';
		} else if ( settings.ajax || settings.sAjaxSource ) {
			return 'ajax';
		}
		return 'dom';
	}

	var __apiStruct = [];

	var __arrayProto = Array.prototype;

	var _toSettings = function ( mixed )
	{
		var idx, jq;
		var settings = DataTable.settings;
		var tables   = $.map(
			settings,
			function (el, i) {
				return el.nTable;
			}
		);

		if ( ! mixed ) {
			return [];
		} else if ( mixed.nTable && mixed.oApi ) {

			return [ mixed ];
		} else if ( mixed.nodeName && mixed.nodeName.toLowerCase() === 'table' ) {

			idx = $.inArray( mixed, tables );
			return idx !== -1 ? [ settings[idx] ] : null;
		} else if ( mixed && typeof mixed.settings === 'function' ) {
			return mixed.settings().toArray();
		} else if ( typeof mixed === 'string' ) {

			jq = $( mixed );
		} else if ( mixed instanceof $ ) {

			jq = mixed;
		}

		if ( jq ) {
			return jq.map(
				function(i) {
					idx = $.inArray( this, tables );
					return idx !== -1 ? settings[idx] : null;
				}
			).toArray();
		}
	};

	_Api = function ( context, data )
	{
		if ( ! (this instanceof _Api) ) {
			return new _Api( context, data );
		}

		var settings    = [];
		var ctxSettings = function ( o ) {
			var a = _toSettings( o );
			if ( a ) {
				settings.push.apply( settings, a );
			}
		};

		if ( $.isArray( context ) ) {
			for ( var i = 0, ien = context.length; i < ien; i++ ) {
				ctxSettings( context[i] );
			}
		} else {
			ctxSettings( context );
		}

		this.context = _unique( settings );

		if ( data ) {
			$.merge( this, data );
		}

		this.selector = {
			rows: null,
			cols: null,
			opts: null
		};

		_Api.extend( this, this, __apiStruct );
	};

	DataTable.Api = _Api;

	$.extend(
		_Api.prototype,
		{
			any: function ()
		{
				return this.count() !== 0;
			},

			concat:  __arrayProto.concat,

			context: [],

			count: function ()
		{
				return this.flatten().length;
			},

			each: function ( fn )
		{
				for ( var i = 0, ien = this.length; i < ien; i++ ) {
					fn.call( this, this[i], i, this );
				}

				return this;
			},

			eq: function ( idx )
		{
				var ctx = this.context;

				return ctx.length > idx ?
				new _Api( ctx[idx], this[idx] ) :
				null;
			},

			filter: function ( fn )
		{
				var a = [];

				if ( __arrayProto.filter ) {
					a = __arrayProto.filter.call( this, fn, this );
				} else {

					for ( var i = 0, ien = this.length; i < ien; i++ ) {
						if ( fn.call( this, this[i], i, this ) ) {
							a.push( this[i] );
						}
					}
				}

				return new _Api( this.context, a );
			},

			flatten: function ()
		{
				var a = [];
				return new _Api( this.context, a.concat.apply( a, this.toArray() ) );
			},

			join:    __arrayProto.join,

			indexOf: __arrayProto.indexOf || function (obj, start)
		{
				for ( var i = (start || 0), ien = this.length; i < ien; i++ ) {
					if ( this[i] === obj ) {
						return i;
					}
				}
				return -1;
			},

			iterator: function ( flatten, type, fn, alwaysNew ) {
				var
				a        = [], ret,
				i, ien, j, jen,
				context  = this.context,
				rows, items, item,
				selector = this.selector;

				if ( typeof flatten === 'string' ) {
					alwaysNew = fn;
					fn        = type;
					type      = flatten;
					flatten   = false;
				}

				for ( i = 0, ien = context.length; i < ien; i++ ) {
					var apiInst = new _Api( context[i] );

					if ( type === 'table' ) {
						ret = fn.call( apiInst, context[i], i );

						if ( ret !== undefined ) {
							a.push( ret );
						}
					} else if ( type === 'columns' || type === 'rows' ) {

						ret = fn.call( apiInst, context[i], this[i], i );

						if ( ret !== undefined ) {
							a.push( ret );
						}
					} else if ( type === 'column' || type === 'column-rows' || type === 'row' || type === 'cell' ) {

						items = this[i];

						if ( type === 'column-rows' ) {
							rows = _selector_row_indexes( context[i], selector.opts );
						}

						for ( j = 0, jen = items.length; j < jen; j++ ) {
							item = items[j];

							if ( type === 'cell' ) {
								ret = fn.call( apiInst, context[i], item.row, item.column, i, j );
							} else {
								ret = fn.call( apiInst, context[i], item, i, j, rows );
							}

							if ( ret !== undefined ) {
								a.push( ret );
							}
						}
					}
				}

				if ( a.length || alwaysNew ) {
					var api          = new _Api( context, flatten ? a.concat.apply( [], a ) : a );
					var apiSelector  = api.selector;
					apiSelector.rows = selector.rows;
					apiSelector.cols = selector.cols;
					apiSelector.opts = selector.opts;
					return api;
				}
				return this;
			},

			lastIndexOf: __arrayProto.lastIndexOf || function (obj, start)
		{

				return this.indexOf.apply( this.toArray.reverse(), arguments );
			},

			length:  0,

			map: function ( fn )
		{
				var a = [];

				if ( __arrayProto.map ) {
					a = __arrayProto.map.call( this, fn, this );
				} else {

					for ( var i = 0, ien = this.length; i < ien; i++ ) {
						a.push( fn.call( this, this[i], i ) );
					}
				}

				return new _Api( this.context, a );
			},

			pluck: function ( prop )
		{
				return this.map(
					function ( el ) {
						return el[ prop ];
					}
				);
			},

			pop:     __arrayProto.pop,

			push:    __arrayProto.push,

			reduce: __arrayProto.reduce || function ( fn, init )
		{
				return _fnReduce( this, fn, init, 0, this.length, 1 );
			},

			reduceRight: __arrayProto.reduceRight || function ( fn, init )
		{
				return _fnReduce( this, fn, init, this.length - 1, -1, -1 );
			},

			reverse: __arrayProto.reverse,

			selector: null,

			shift:   __arrayProto.shift,

			slice: function () {
				return new _Api( this.context, this );
			},

			sort:    __arrayProto.sort,

			splice:  __arrayProto.splice,

			toArray: function ()
		{
				return __arrayProto.slice.call( this );
			},

			to$: function ()
		{
				return $( this );
			},

			toJQuery: function ()
		{
				return $( this );
			},

			unique: function ()
		{
				return new _Api( this.context, _unique( this ) );
			},

			unshift: __arrayProto.unshift
		}
	);

	_Api.extend = function ( scope, obj, ext )
	{

		if ( ! ext.length || ! obj || ( ! (obj instanceof _Api) && ! obj.__dt_wrapper ) ) {
			return;
		}

		var
			i, ien,
			struct,
			methodScoping = function ( scope, fn, struc ) {
				return function () {
					var ret = fn.apply( scope, arguments );

					_Api.extend( ret, ret, struc.methodExt );
					return ret;
				};
			};

		for ( i = 0, ien = ext.length; i < ien; i++ ) {
			struct = ext[i];

			obj[ struct.name ] = struct.type === 'function' ?
				methodScoping( scope, struct.val, struct ) :
				struct.type === 'object' ?
					{} :
					struct.val;

			obj[ struct.name ].__dt_wrapper = true;

			_Api.extend( scope, obj[ struct.name ], struct.propExt );
		}
	};

	_Api.register = _api_register = function ( name, val )
	{
		if ( $.isArray( name ) ) {
			for ( var j = 0, jen = name.length; j < jen; j++ ) {
				_Api.register( name[j], val );
			}
			return;
		}

		var
			i, ien,
			heir   = name.split( '.' ),
			struct = __apiStruct,
			key, method;

		var find = function ( src, name ) {
			for ( var i = 0, ien = src.length; i < ien; i++ ) {
				if ( src[i].name === name ) {
					return src[i];
				}
			}
			return null;
		};

		for ( i = 0, ien = heir.length; i < ien; i++ ) {
			method = heir[i].indexOf( '()' ) !== -1;
			key    = method ?
				heir[i].replace( '()', '' ) :
				heir[i];

			var src = find( struct, key );
			if ( ! src ) {
				src = {
					name:      key,
					val:       {},
					methodExt: [],
					propExt:   [],
					type:      'object'
				};
				struct.push( src );
			}

			if ( i === ien - 1 ) {
				src.val  = val;
				src.type = typeof val === 'function' ?
					'function' :
					$.isPlainObject( val ) ?
						'object' :
						'other';
			} else {
				struct = method ?
					src.methodExt :
					src.propExt;
			}
		}
	};

	_Api.registerPlural = _api_registerPlural = function ( pluralName, singularName, val ) {
		_Api.register( pluralName, val );

		_Api.register(
			singularName,
			function () {
				var ret = val.apply( this, arguments );

				if ( ret === this ) {

					return this;
				} else if ( ret instanceof _Api ) {

					return ret.length ?
					$.isArray( ret[0] ) ?
						new _Api( ret.context, ret[0] ) :
						ret[0] :
					undefined;
				}

				return ret;
			}
		);
	};

	var __table_selector = function ( selector, a )
	{
		if ( $.isArray( selector ) ) {
			return $.map(
				selector,
				function (item) {
					return __table_selector( item, a );
				}
			);
		}

		if ( typeof selector === 'number' ) {
			return [ a[ selector ] ];
		}

		var nodes = $.map(
			a,
			function (el, i) {
				return el.nTable;
			}
		);

		return $( nodes )
			.filter( selector )
			.map(
				function (i) {

					var idx = $.inArray( this, nodes );
					return a[ idx ];
				}
			)
			.toArray();
	};

	_api_register(
		'tables()',
		function ( selector ) {

			return selector !== undefined && selector !== null ?
			new _Api( __table_selector( selector, this.context ) ) :
			this;
		}
	);

	_api_register(
		'table()',
		function ( selector ) {
			var tables = this.tables( selector );
			var ctx    = tables.context;

			return ctx.length ?
			new _Api( ctx[0] ) :
			tables;
		}
	);

	_api_registerPlural(
		'tables().nodes()',
		'table().node()' ,
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					return ctx.nTable;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'tables().body()',
		'table().body()' ,
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					return ctx.nTBody;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'tables().header()',
		'table().header()' ,
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					return ctx.nTHead;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'tables().footer()',
		'table().footer()' ,
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					return ctx.nTFoot;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'tables().containers()',
		'table().container()' ,
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					return ctx.nTableWrapper;
				},
				1
			);
		}
	);

	_api_register(
		'draw()',
		function ( paging ) {
			return this.iterator(
				'table',
				function ( settings ) {
					if ( paging === 'page' ) {
						_fnDraw( settings );
					} else {
						if ( typeof paging === 'string' ) {
							paging = paging === 'full-hold' ?
							false :
							true;
						}

						_fnReDraw( settings, paging === false );
					}
				}
			);
		}
	);

	_api_register(
		'page()',
		function ( action ) {
			if ( action === undefined ) {
				return this.page.info().page;
			}

			return this.iterator(
				'table',
				function ( settings ) {
					_fnPageChange( settings, action );
				}
			);
		}
	);

	_api_register(
		'page.info()',
		function ( action ) {
			if ( this.context.length === 0 ) {
				return undefined;
			}

			var
			settings   = this.context[0],
			start      = settings._iDisplayStart,
			len        = settings.oFeatures.bPaginate ? settings._iDisplayLength : -1,
			visRecords = settings.fnRecordsDisplay(),
			all        = len === -1;

			return {
				"page":           all ? 0 : Math.floor( start / len ),
				"pages":          all ? 1 : Math.ceil( visRecords / len ),
				"start":          start,
				"end":            settings.fnDisplayEnd(),
				"length":         len,
				"recordsTotal":   settings.fnRecordsTotal(),
				"recordsDisplay": visRecords,
				"serverSide":     _fnDataSource( settings ) === 'ssp'
			};
		}
	);

	_api_register(
		'page.len()',
		function ( len ) {

			if ( len === undefined ) {
				return this.context.length !== 0 ?
				this.context[0]._iDisplayLength :
				undefined;
			}

			return this.iterator(
				'table',
				function ( settings ) {
					_fnLengthChange( settings, len );
				}
			);
		}
	);

	var __reload = function ( settings, holdPosition, callback ) {

		if ( callback ) {
			var api = new _Api( settings );

			api.one(
				'draw',
				function () {
					callback( api.ajax.json() );
				}
			);
		}

		if ( _fnDataSource( settings ) == 'ssp' ) {
			_fnReDraw( settings, holdPosition );
		} else {
			_fnProcessingDisplay( settings, true );

			var xhr = settings.jqXHR;
			if ( xhr && xhr.readyState !== 4 ) {
				xhr.abort();
			}

			_fnBuildAjax(
				settings,
				[],
				function( json ) {
					_fnClearTable( settings );

					var data = _fnAjaxDataSrc( settings, json );
					for ( var i = 0, ien = data.length; i < ien; i++ ) {
						_fnAddData( settings, data[i] );
					}

					_fnReDraw( settings, holdPosition );
					_fnProcessingDisplay( settings, false );
				}
			);
		}
	};

	_api_register(
		'ajax.json()',
		function () {
			var ctx = this.context;

			if ( ctx.length > 0 ) {
				return ctx[0].json;
			}

		}
	);

	_api_register(
		'ajax.params()',
		function () {
			var ctx = this.context;

			if ( ctx.length > 0 ) {
				return ctx[0].oAjaxData;
			}

		}
	);

	_api_register(
		'ajax.reload()',
		function ( callback, resetPaging ) {
			return this.iterator(
				'table',
				function (settings) {
					__reload( settings, resetPaging === false, callback );
				}
			);
		}
	);

	_api_register(
		'ajax.url()',
		function ( url ) {
			var ctx = this.context;

			if ( url === undefined ) {

				if ( ctx.length === 0 ) {
					return undefined;
				}
				ctx = ctx[0];

				return ctx.ajax ?
				$.isPlainObject( ctx.ajax ) ?
					ctx.ajax.url :
					ctx.ajax :
				ctx.sAjaxSource;
			}

			return this.iterator(
				'table',
				function ( settings ) {
					if ( $.isPlainObject( settings.ajax ) ) {
						settings.ajax.url = url;
					} else {
						settings.ajax = url;
					}

				}
			);
		}
	);

	_api_register(
		'ajax.url().load()',
		function ( callback, resetPaging ) {

			return this.iterator(
				'table',
				function ( ctx ) {
					__reload( ctx, resetPaging === false, callback );
				}
			);
		}
	);

	var _selector_run = function ( type, selector, selectFn, settings, opts )
	{
		var
			out          = [], res,
			a, i, ien, j, jen,
			selectorType = typeof selector;

		if ( ! selector || selectorType === 'string' || selectorType === 'function' || selector.length === undefined ) {
			selector = [ selector ];
		}

		for ( i = 0, ien = selector.length; i < ien; i++ ) {

			a = selector[i] && selector[i].split && ! selector[i].match( /[\[\(:]/ ) ?
				selector[i].split( ',' ) :
				[ selector[i] ];

			for ( j = 0, jen = a.length; j < jen; j++ ) {
				res = selectFn( typeof a[j] === 'string' ? $.trim( a[j] ) : a[j] );

				if ( res && res.length ) {
					out = out.concat( res );
				}
			}
		}

		var ext = _ext.selector[ type ];
		if ( ext.length ) {
			for ( i = 0, ien = ext.length; i < ien; i++ ) {
				out = ext[i]( settings, opts, out );
			}
		}

		return _unique( out );
	};

	var _selector_opts = function ( opts )
	{
		if ( ! opts ) {
			opts = {};
		}

		if ( opts.filter && opts.search === undefined ) {
			opts.search = opts.filter;
		}

		return $.extend(
			{
				search: 'none',
				order: 'current',
				page: 'all'
			},
			opts
		);
	};

	var _selector_first = function ( inst )
	{

		for ( var i = 0, ien = inst.length; i < ien; i++ ) {
			if ( inst[i].length > 0 ) {

				inst[0]        = inst[i];
				inst[0].length = 1;
				inst.length    = 1;
				inst.context   = [ inst.context[i] ];

				return inst;
			}
		}

		inst.length = 0;
		return inst;
	};

	var _selector_row_indexes = function ( settings, opts )
	{
		var
			i, ien, tmp, a  = [],
			displayFiltered = settings.aiDisplay,
			displayMaster   = settings.aiDisplayMaster;

		var
			search = opts.search,
			order  = opts.order,
			page   = opts.page;

		if ( _fnDataSource( settings ) == 'ssp' ) {

			return search === 'removed' ?
				[] :
				_range( 0, displayMaster.length );
		} else if ( page == 'current' ) {

			for ( i = settings._iDisplayStart, ien = settings.fnDisplayEnd(); i < ien; i++ ) {
				a.push( displayFiltered[i] );
			}
		} else if ( order == 'current' || order == 'applied' ) {
			if ( search == 'none') {
				a = displayMaster.slice();
			} else if ( search == 'applied' ) {
				a = displayFiltered.slice();
			} else if ( search == 'removed' ) {

				var displayFilteredMap = {};

				for ( var i = 0, ien = displayFiltered.length; i < ien; i++ ) {
					displayFilteredMap[displayFiltered[i]] = null;
				}

				a = $.map(
					displayMaster,
					function (el) {
						return ! displayFilteredMap.hasOwnProperty( el ) ?
						el :
						null;
					}
				);
			}
		} else if ( order == 'index' || order == 'original' ) {
			for ( i = 0, ien = settings.aoData.length; i < ien; i++ ) {
				if ( search == 'none' ) {
					a.push( i );
				} else {
					tmp = $.inArray( i, displayFiltered );

					if ((tmp === -1 && search == 'removed') ||
						(tmp >= 0 && search == 'applied') ) {
						a.push( i );
					}
				}
			}
		}

		return a;
	};

	var __row_selector = function ( settings, selector, opts )
	{
		var rows;
		var run = function ( sel ) {
			var selInt = _intVal( sel );
			var i, ien;
			var aoData = settings.aoData;

			if ( selInt !== null && ! opts ) {
				return [ selInt ];
			}

			if ( ! rows ) {
				rows = _selector_row_indexes( settings, opts );
			}

			if ( selInt !== null && $.inArray( selInt, rows ) !== -1 ) {

				return [ selInt ];
			} else if ( sel === null || sel === undefined || sel === '' ) {

				return rows;
			}

			if ( typeof sel === 'function' ) {
				return $.map(
					rows,
					function (idx) {
						var row = aoData[ idx ];
						return sel( idx, row._aData, row.nTr ) ? idx : null;
					}
				);
			}

			if ( sel.nodeName ) {
				var rowIdx  = sel._DT_RowIndex;
				var cellIdx = sel._DT_CellIndex;

				if ( rowIdx !== undefined ) {

					return aoData[ rowIdx ] && aoData[ rowIdx ].nTr === sel ?
						[ rowIdx ] :
						[];
				} else if ( cellIdx ) {
					return aoData[ cellIdx.row ] && aoData[ cellIdx.row ].nTr === sel.parentNode ?
						[ cellIdx.row ] :
						[];
				} else {
					var host = $( sel ).closest( '*[data-dt-row]' );
					return host.length ?
						[ host.data( 'dt-row' ) ] :
						[];
				}
			}

			if ( typeof sel === 'string' && sel.charAt( 0 ) === '#' ) {

				var rowObj = settings.aIds[ sel.replace( /^#/, '' ) ];
				if ( rowObj !== undefined ) {
					return [ rowObj.idx ];
				}

			}

			var nodes = _removeEmpty(
				_pluck_order( settings.aoData, rows, 'nTr' )
			);

			return $( nodes )
				.filter( sel )
				.map(
					function () {
						return this._DT_RowIndex;
					}
				)
				.toArray();
		};

		return _selector_run( 'row', selector, run, settings, opts );
	};

	_api_register(
		'rows()',
		function ( selector, opts ) {

			if ( selector === undefined ) {
				selector = '';
			} else if ( $.isPlainObject( selector ) ) {
				opts     = selector;
				selector = '';
			}

			opts = _selector_opts( opts );

			var inst = this.iterator(
				'table',
				function ( settings ) {
					return __row_selector( settings, selector, opts );
				},
				1
			);

			inst.selector.rows = selector;
			inst.selector.opts = opts;

			return inst;
		}
	);

	_api_register(
		'rows().nodes()',
		function () {
			return this.iterator(
				'row',
				function ( settings, row ) {
					return settings.aoData[ row ].nTr || undefined;
				},
				1
			);
		}
	);

	_api_register(
		'rows().data()',
		function () {
			return this.iterator(
				true,
				'rows',
				function ( settings, rows ) {
					return _pluck_order( settings.aoData, rows, '_aData' );
				},
				1
			);
		}
	);

	_api_registerPlural(
		'rows().cache()',
		'row().cache()',
		function ( type ) {
			return this.iterator(
				'row',
				function ( settings, row ) {
					var r = settings.aoData[ row ];
					return type === 'search' ? r._aFilterData : r._aSortData;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'rows().invalidate()',
		'row().invalidate()',
		function ( src ) {
			return this.iterator(
				'row',
				function ( settings, row ) {
					_fnInvalidate( settings, row, src );
				}
			);
		}
	);

	_api_registerPlural(
		'rows().indexes()',
		'row().index()',
		function () {
			return this.iterator(
				'row',
				function ( settings, row ) {
					return row;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'rows().ids()',
		'row().id()',
		function ( hash ) {
			var a       = [];
			var context = this.context;

			for ( var i = 0, ien = context.length; i < ien; i++ ) {
				for ( var j = 0, jen = this[i].length; j < jen; j++ ) {
					var id = context[i].rowIdFn( context[i].aoData[ this[i][j] ]._aData );
					a.push( (hash === true ? '#' : '' ) + id );
				}
			}

			return new _Api( context, a );
		}
	);

	_api_registerPlural(
		'rows().remove()',
		'row().remove()',
		function () {
			var that = this;

			this.iterator(
				'row',
				function ( settings, row, thatIdx ) {
					var data    = settings.aoData;
					var rowData = data[ row ];
					var i, ien, j, jen;
					var loopRow, loopCells;

					data.splice( row, 1 );

					for ( i = 0, ien = data.length; i < ien; i++ ) {
						loopRow   = data[i];
						loopCells = loopRow.anCells;

						if ( loopRow.nTr !== null ) {
							loopRow.nTr._DT_RowIndex = i;
						}

						if ( loopCells !== null ) {
							for ( j = 0, jen = loopCells.length; j < jen; j++ ) {
								loopCells[j]._DT_CellIndex.row = i;
							}
						}
					}

					_fnDeleteIndex( settings.aiDisplayMaster, row );
					_fnDeleteIndex( settings.aiDisplay, row );
					_fnDeleteIndex( that[ thatIdx ], row, false );

					if ( settings._iRecordsDisplay > 0 ) {
						settings._iRecordsDisplay--;
					}

					_fnLengthOverflow( settings );

					var id = settings.rowIdFn( rowData._aData );
					if ( id !== undefined ) {
						delete settings.aIds[ id ];
					}
				}
			);

			this.iterator(
				'table',
				function ( settings ) {
					for ( var i = 0, ien = settings.aoData.length; i < ien; i++ ) {
						settings.aoData[i].idx = i;
					}
				}
			);

			return this;
		}
	);

	_api_register(
		'rows.add()',
		function ( rows ) {
			var newRows = this.iterator(
				'table',
				function ( settings ) {
					var row, i, ien;
					var out = [];

					for ( i = 0, ien = rows.length; i < ien; i++ ) {
						row = rows[i];

						if ( row.nodeName && row.nodeName.toUpperCase() === 'TR' ) {
							out.push( _fnAddTr( settings, row )[0] );
						} else {
							out.push( _fnAddData( settings, row ) );
						}
					}

					return out;
				},
				1
			);

			var modRows = this.rows( -1 );
			modRows.pop();
			$.merge( modRows, newRows );

			return modRows;
		}
	);

	_api_register(
		'row()',
		function ( selector, opts ) {
			return _selector_first( this.rows( selector, opts ) );
		}
	);

	_api_register(
		'row().data()',
		function ( data ) {
			var ctx = this.context;

			if ( data === undefined ) {

				return ctx.length && this.length ?
				ctx[0].aoData[ this[0] ]._aData :
				undefined;
			}

			var row    = ctx[0].aoData[ this[0] ];
			row._aData = data;

			if ( $.isArray( data ) && row.nTr && row.nTr.id ) {
				_fnSetObjectDataFn( ctx[0].rowId )( data, row.nTr.id );
			}

			_fnInvalidate( ctx[0], this[0], 'data' );

			return this;
		}
	);

	_api_register(
		'row().node()',
		function () {
			var ctx = this.context;

			return ctx.length && this.length ?
			ctx[0].aoData[ this[0] ].nTr || null :
			null;
		}
	);

	_api_register(
		'row.add()',
		function ( row ) {

			if ( row instanceof $ && row.length ) {
				row = row[0];
			}

			var rows = this.iterator(
				'table',
				function ( settings ) {
					if ( row.nodeName && row.nodeName.toUpperCase() === 'TR' ) {
						return _fnAddTr( settings, row )[0];
					}
					return _fnAddData( settings, row );
				}
			);

			return this.row( rows[0] );
		}
	);

	var __details_add = function ( ctx, row, data, klass )
	{

		var rows   = [];
		var addRow = function ( r, k ) {

			if ( $.isArray( r ) || r instanceof $ ) {
				for ( var i = 0, ien = r.length; i < ien; i++ ) {
					addRow( r[i], k );
				}
				return;
			}

			if ( r.nodeName && r.nodeName.toLowerCase() === 'tr' ) {
				rows.push( r );
			} else {

				var created     = $( '<tr><td/></tr>' ).addClass( k );
				$( 'td', created )
					.addClass( k )
					.html( r )
					[0].colSpan = _fnVisbleColumns( ctx );

				rows.push( created[0] );
			}
		};

		addRow( data, klass );

		if ( row._details ) {
			row._details.detach();
		}

		row._details = $( rows );

		if ( row._detailsShow ) {
			row._details.insertAfter( row.nTr );
		}
	};

	var __details_remove = function ( api, idx )
	{
		var ctx = api.context;

		if ( ctx.length ) {
			var row = ctx[0].aoData[ idx !== undefined ? idx : api[0] ];

			if ( row && row._details ) {
				row._details.remove();

				row._detailsShow = undefined;
				row._details     = undefined;
			}
		}
	};

	var __details_display = function ( api, show ) {
		var ctx = api.context;

		if ( ctx.length && api.length ) {
			var row = ctx[0].aoData[ api[0] ];

			if ( row._details ) {
				row._detailsShow = show;

				if ( show ) {
					row._details.insertAfter( row.nTr );
				} else {
					row._details.detach();
				}

				__details_events( ctx[0] );
			}
		}
	};

	var __details_events = function ( settings )
	{
		var api          = new _Api( settings );
		var namespace    = '.dt.DT_details';
		var drawEvent    = 'draw' + namespace;
		var colvisEvent  = 'column-visibility' + namespace;
		var destroyEvent = 'destroy' + namespace;
		var data         = settings.aoData;

		api.off( drawEvent + ' ' + colvisEvent + ' ' + destroyEvent );

		if ( _pluck( data, '_details' ).length > 0 ) {

			api.on(
				drawEvent,
				function ( e, ctx ) {
					if ( settings !== ctx ) {
						return;
					}

					api.rows( {page:'current'} ).eq( 0 ).each(
						function (idx) {

							var row = data[ idx ];

							if ( row._detailsShow ) {
								  row._details.insertAfter( row.nTr );
							}
						}
					);
				}
			);

			api.on(
				colvisEvent,
				function ( e, ctx, idx, vis ) {
					if ( settings !== ctx ) {
						return;
					}

					var row, visible = _fnVisbleColumns( ctx );

					for ( var i = 0, ien = data.length; i < ien; i++ ) {
						row = data[i];

						if ( row._details ) {
							row._details.children( 'td[colspan]' ).attr( 'colspan', visible );
						}
					}
				}
			);

			api.on(
				destroyEvent,
				function ( e, ctx ) {
					if ( settings !== ctx ) {
						return;
					}

					for ( var i = 0, ien = data.length; i < ien; i++ ) {
						if ( data[i]._details ) {
							__details_remove( api, i );
						}
					}
				}
			);
		}
	};

	var _emp       = '';
	var _child_obj = _emp + 'row().child';
	var _child_mth = _child_obj + '()';

	_api_register(
		_child_mth,
		function ( data, klass ) {
			var ctx = this.context;

			if ( data === undefined ) {

				return ctx.length && this.length ?
				ctx[0].aoData[ this[0] ]._details :
				undefined;
			} else if ( data === true ) {

				this.child.show();
			} else if ( data === false ) {

				__details_remove( this );
			} else if ( ctx.length && this.length ) {

				__details_add( ctx[0], ctx[0].aoData[ this[0] ], data, klass );
			}

			return this;
		}
	);

	_api_register(
		[
		_child_obj + '.show()',
		_child_mth + '.show()'
		],
		function ( show ) {
			__details_display( this, true );
			return this;
		}
	);

	_api_register(
		[
		_child_obj + '.hide()',
		_child_mth + '.hide()'
		],
		function () {
			__details_display( this, false );
			return this;
		}
	);

	_api_register(
		[
		_child_obj + '.remove()',
		_child_mth + '.remove()'
		],
		function () {
			__details_remove( this );
			return this;
		}
	);

	_api_register(
		_child_obj + '.isShown()',
		function () {
			var ctx = this.context;

			if ( ctx.length && this.length ) {

				return ctx[0].aoData[ this[0] ]._detailsShow || false;
			}
			return false;
		}
	);

	var __re_column_selector = /^([^:]+):(name|visIdx|visible)$/;

	var __columnData = function ( settings, column, r1, r2, rows ) {
		var a = [];
		for ( var row = 0, ien = rows.length; row < ien; row++ ) {
			a.push( _fnGetCellData( settings, rows[row], column ) );
		}
		return a;
	};

	var __column_selector = function ( settings, selector, opts )
	{
		var
			columns = settings.aoColumns,
			names   = _pluck( columns, 'sName' ),
			nodes   = _pluck( columns, 'nTh' );

		var run = function ( s ) {
			var selInt = _intVal( s );

			if ( s === '' ) {
				return _range( columns.length );
			}

			if ( selInt !== null ) {
				return [ selInt >= 0 ?
					selInt :
					columns.length + selInt
				];
			}

			if ( typeof s === 'function' ) {
				var rows = _selector_row_indexes( settings, opts );

				return $.map(
					columns,
					function (col, idx) {
						return s(
							idx,
							__columnData( settings, idx, 0, 0, rows ),
							nodes[ idx ]
						) ? idx : null;
					}
				);
			}

			var match = typeof s === 'string' ?
				s.match( __re_column_selector ) :
				'';

			if ( match ) {
				switch ( match[2] ) {
					case 'visIdx':
					case 'visible':
						var idx = parseInt( match[1], 10 );

						if ( idx < 0 ) {

							var visColumns = $.map(
								columns,
								function (col,i) {
									return col.bVisible ? i : null;
								}
							);
							return [ visColumns[ visColumns.length + idx ] ];
						}

						return [ _fnVisibleToColumnIndex( settings, idx ) ];

					case 'name':

						return $.map(
							names,
							function (name, i) {
								return name === match[1] ? i : null;
							}
						);

					default:
						return [];
				}
			}

			if ( s.nodeName && s._DT_CellIndex ) {
				return [ s._DT_CellIndex.column ];
			}

			var jqResult = $( nodes )
				.filter( s )
				.map(
					function () {
						return $.inArray( this, nodes );
					}
				)
				.toArray();

			if ( jqResult.length || ! s.nodeName ) {
				return jqResult;
			}

			var host = $( s ).closest( '*[data-dt-column]' );
			return host.length ?
				[ host.data( 'dt-column' ) ] :
				[];
		};

		return _selector_run( 'column', selector, run, settings, opts );
	};

	var __setColumnVis = function ( settings, column, vis ) {
		var
			cols = settings.aoColumns,
			col  = cols[ column ],
			data = settings.aoData,
			row, cells, i, ien, tr;

		if ( vis === undefined ) {
			return col.bVisible;
		}

		if ( col.bVisible === vis ) {
			return;
		}

		if ( vis ) {

			var insertBefore = $.inArray( true, _pluck( cols, 'bVisible' ), column + 1 );

			for ( i = 0, ien = data.length; i < ien; i++ ) {
				tr    = data[i].nTr;
				cells = data[i].anCells;

				if ( tr ) {

					tr.insertBefore( cells[ column ], cells[ insertBefore ] || null );
				}
			}
		} else {

			$( _pluck( settings.aoData, 'anCells', column ) ).detach();
		}

		col.bVisible = vis;
	};

	_api_register(
		'columns()',
		function ( selector, opts ) {

			if ( selector === undefined ) {
				selector = '';
			} else if ( $.isPlainObject( selector ) ) {
				opts     = selector;
				selector = '';
			}

			opts = _selector_opts( opts );

			var inst = this.iterator(
				'table',
				function ( settings ) {
					return __column_selector( settings, selector, opts );
				},
				1
			);

			inst.selector.cols = selector;
			inst.selector.opts = opts;

			return inst;
		}
	);

	_api_registerPlural(
		'columns().header()',
		'column().header()',
		function ( selector, opts ) {
			return this.iterator(
				'column',
				function ( settings, column ) {
					return settings.aoColumns[column].nTh;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'columns().footer()',
		'column().footer()',
		function ( selector, opts ) {
			return this.iterator(
				'column',
				function ( settings, column ) {
					return settings.aoColumns[column].nTf;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'columns().data()',
		'column().data()',
		function () {
			return this.iterator( 'column-rows', __columnData, 1 );
		}
	);

	_api_registerPlural(
		'columns().dataSrc()',
		'column().dataSrc()',
		function () {
			return this.iterator(
				'column',
				function ( settings, column ) {
					return settings.aoColumns[column].mData;
				},
				1
			);
		}
	);

	_api_registerPlural(
		'columns().cache()',
		'column().cache()',
		function ( type ) {
			return this.iterator(
				'column-rows',
				function ( settings, column, i, j, rows ) {
					return _pluck_order(
						settings.aoData,
						rows,
						type === 'search' ? '_aFilterData' : '_aSortData',
						column
					);
				},
				1
			);
		}
	);

	_api_registerPlural(
		'columns().nodes()',
		'column().nodes()',
		function () {
			return this.iterator(
				'column-rows',
				function ( settings, column, i, j, rows ) {
					return _pluck_order( settings.aoData, rows, 'anCells', column );
				},
				1
			);
		}
	);

	_api_registerPlural(
		'columns().visible()',
		'column().visible()',
		function ( vis, calc ) {
			var that = this;
			var ret  = this.iterator(
				'column',
				function ( settings, column ) {
					if ( vis === undefined ) {
						return settings.aoColumns[ column ].bVisible;
					}
					__setColumnVis( settings, column, vis );
				}
			);

			if ( vis !== undefined ) {
				this.iterator(
					'table',
					function ( settings ) {

						_fnDrawHead( settings, settings.aoHeader );
						_fnDrawHead( settings, settings.aoFooter );

						if ( ! settings.aiDisplay.length ) {
							$( settings.nTBody ).find( 'td[colspan]' ).attr( 'colspan', _fnVisbleColumns( settings ) );
						}

						_fnSaveState( settings );

						that.iterator(
							'column',
							function ( settings, column ) {
								_fnCallbackFire( settings, null, 'column-visibility', [settings, column, vis, calc] );
							}
						);

						if ( calc === undefined || calc ) {
							that.columns.adjust();
						}
					}
				);
			}

			return ret;
		}
	);

	_api_registerPlural(
		'columns().indexes()',
		'column().index()',
		function ( type ) {
			return this.iterator(
				'column',
				function ( settings, column ) {
					return type === 'visible' ?
					_fnColumnIndexToVisible( settings, column ) :
					column;
				},
				1
			);
		}
	);

	_api_register(
		'columns.adjust()',
		function () {
			return this.iterator(
				'table',
				function ( settings ) {
					_fnAdjustColumnSizing( settings );
				},
				1
			);
		}
	);

	_api_register(
		'column.index()',
		function ( type, idx ) {
			if ( this.context.length !== 0 ) {
				var ctx = this.context[0];

				if ( type === 'fromVisible' || type === 'toData' ) {
					return _fnVisibleToColumnIndex( ctx, idx );
				} else if ( type === 'fromData' || type === 'toVisible' ) {
					return _fnColumnIndexToVisible( ctx, idx );
				}
			}
		}
	);

	_api_register(
		'column()',
		function ( selector, opts ) {
			return _selector_first( this.columns( selector, opts ) );
		}
	);

	var __cell_selector = function ( settings, selector, opts )
	{
		var data     = settings.aoData;
		var rows     = _selector_row_indexes( settings, opts );
		var cells    = _removeEmpty( _pluck_order( data, rows, 'anCells' ) );
		var allCells = $( [].concat.apply( [], cells ) );
		var row;
		var columns = settings.aoColumns.length;
		var a, i, ien, j, o, host;

		var run = function ( s ) {
			var fnSelector = typeof s === 'function';

			if ( s === null || s === undefined || fnSelector ) {

				a = [];

				for ( i = 0, ien = rows.length; i < ien; i++ ) {
					row = rows[i];

					for ( j = 0; j < columns; j++ ) {
						o = {
							row: row,
							column: j
						};

						if ( fnSelector ) {

							host = data[ row ];

							if ( s( o, _fnGetCellData( settings, row, j ), host.anCells ? host.anCells[j] : null ) ) {
								a.push( o );
							}
						} else {

							a.push( o );
						}
					}
				}

				return a;
			}

			if ( $.isPlainObject( s ) ) {

				return s.column !== undefined && s.row !== undefined && $.inArray( s.row, rows ) !== -1 ?
					[s] :
					[];
			}

			var jqResult = allCells
				.filter( s )
				.map(
					function (i, el) {
						return {
							row:    el._DT_CellIndex.row,
							column: el._DT_CellIndex.column
						};
					}
				)
				.toArray();

			if ( jqResult.length || ! s.nodeName ) {
				return jqResult;
			}

			host = $( s ).closest( '*[data-dt-row]' );
			return host.length ?
				[ {
					row: host.data( 'dt-row' ),
					column: host.data( 'dt-column' )
			} ] :
				[];
		};

		return _selector_run( 'cell', selector, run, settings, opts );
	};

	_api_register(
		'cells()',
		function ( rowSelector, columnSelector, opts ) {

			if ( $.isPlainObject( rowSelector ) ) {

				if ( rowSelector.row === undefined ) {

					opts        = rowSelector;
					rowSelector = null;
				} else {

					opts           = columnSelector;
					columnSelector = null;
				}
			}
			if ( $.isPlainObject( columnSelector ) ) {
				opts           = columnSelector;
				columnSelector = null;
			}

			if ( columnSelector === null || columnSelector === undefined ) {
				return this.iterator(
					'table',
					function ( settings ) {
						return __cell_selector( settings, rowSelector, _selector_opts( opts ) );
					}
				);
			}

			var internalOpts = opts ? {
				page: opts.page,
				order: opts.order,
				search: opts.search
			} : {};

			var columns = this.columns( columnSelector, internalOpts );
			var rows    = this.rows( rowSelector, internalOpts );
			var i, ien, j, jen;

			var cellsNoOpts = this.iterator(
				'table',
				function ( settings, idx ) {
					var a = [];

					for ( i = 0, ien = rows[idx].length; i < ien; i++ ) {
						for ( j = 0, jen = columns[idx].length; j < jen; j++ ) {
							a.push(
								{
									row:    rows[idx][i],
									column: columns[idx][j]
								}
							);
						}
					}

					return a;
				},
				1
			);

			var cells = opts && opts.selected ?
			this.cells( cellsNoOpts, opts ) :
			cellsNoOpts;

			$.extend(
				cells.selector,
				{
					cols: columnSelector,
					rows: rowSelector,
					opts: opts
				}
			);

			return cells;
		}
	);

	_api_registerPlural(
		'cells().nodes()',
		'cell().node()',
		function () {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					var data = settings.aoData[ row ];

					return data && data.anCells ?
					data.anCells[ column ] :
					undefined;
				},
				1
			);
		}
	);

	_api_register(
		'cells().data()',
		function () {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					return _fnGetCellData( settings, row, column );
				},
				1
			);
		}
	);

	_api_registerPlural(
		'cells().cache()',
		'cell().cache()',
		function ( type ) {
			type = type === 'search' ? '_aFilterData' : '_aSortData';

			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					return settings.aoData[ row ][ type ][ column ];
				},
				1
			);
		}
	);

	_api_registerPlural(
		'cells().render()',
		'cell().render()',
		function ( type ) {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					return _fnGetCellData( settings, row, column, type );
				},
				1
			);
		}
	);

	_api_registerPlural(
		'cells().indexes()',
		'cell().index()',
		function () {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					return {
						row: row,
						column: column,
						columnVisible: _fnColumnIndexToVisible( settings, column )
					};
				},
				1
			);
		}
	);

	_api_registerPlural(
		'cells().invalidate()',
		'cell().invalidate()',
		function ( src ) {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					_fnInvalidate( settings, row, src, column );
				}
			);
		}
	);

	_api_register(
		'cell()',
		function ( rowSelector, columnSelector, opts ) {
			return _selector_first( this.cells( rowSelector, columnSelector, opts ) );
		}
	);

	_api_register(
		'cell().data()',
		function ( data ) {
			var ctx  = this.context;
			var cell = this[0];

			if ( data === undefined ) {

				return ctx.length && cell.length ?
				_fnGetCellData( ctx[0], cell[0].row, cell[0].column ) :
				undefined;
			}

			_fnSetCellData( ctx[0], cell[0].row, cell[0].column, data );
			_fnInvalidate( ctx[0], cell[0].row, 'data', cell[0].column );

			return this;
		}
	);

	_api_register(
		'order()',
		function ( order, dir ) {
			var ctx = this.context;

			if ( order === undefined ) {

				return ctx.length !== 0 ?
				ctx[0].aaSorting :
				undefined;
			}

			if ( typeof order === 'number' ) {

				order = [ [ order, dir ] ];
			} else if ( order.length && ! $.isArray( order[0] ) ) {

				order = Array.prototype.slice.call( arguments );
			}

			return this.iterator(
				'table',
				function ( settings ) {
					settings.aaSorting = order.slice();
				}
			);
		}
	);

	_api_register(
		'order.listener()',
		function ( node, column, callback ) {
			return this.iterator(
				'table',
				function ( settings ) {
					_fnSortAttachListener( settings, node, column, callback );
				}
			);
		}
	);

	_api_register(
		'order.fixed()',
		function ( set ) {
			if ( ! set ) {
				var ctx   = this.context;
				var fixed = ctx.length ?
				ctx[0].aaSortingFixed :
				undefined;

				return $.isArray( fixed ) ?
				{ pre: fixed } :
				fixed;
			}

			return this.iterator(
				'table',
				function ( settings ) {
					settings.aaSortingFixed = $.extend( true, {}, set );
				}
			);
		}
	);

	_api_register(
		[
		'columns().order()',
		'column().order()'
		],
		function ( dir ) {
			var that = this;

			return this.iterator(
				'table',
				function ( settings, i ) {
					var sort = [];

					$.each(
						that[i],
						function (j, col) {
							sort.push( [ col, dir ] );
						}
					);

					settings.aaSorting = sort;
				}
			);
		}
	);

	_api_register(
		'search()',
		function ( input, regex, smart, caseInsen ) {
			var ctx = this.context;

			if ( input === undefined ) {

				return ctx.length !== 0 ?
				ctx[0].oPreviousSearch.sSearch :
				undefined;
			}

			return this.iterator(
				'table',
				function ( settings ) {
					if ( ! settings.oFeatures.bFilter ) {
						return;
					}

					_fnFilterComplete(
						settings,
						$.extend(
							{},
							settings.oPreviousSearch,
							{
								"sSearch": input + "",
								"bRegex":  regex === null ? false : regex,
								"bSmart":  smart === null ? true : smart,
								"bCaseInsensitive": caseInsen === null ? true : caseInsen
							}
						),
						1
					);
				}
			);
		}
	);

	_api_registerPlural(
		'columns().search()',
		'column().search()',
		function ( input, regex, smart, caseInsen ) {
			return this.iterator(
				'column',
				function ( settings, column ) {
					var preSearch = settings.aoPreSearchCols;

					if ( input === undefined ) {

						return preSearch[ column ].sSearch;
					}

					if ( ! settings.oFeatures.bFilter ) {
						return;
					}

					$.extend(
						preSearch[ column ],
						{
							"sSearch": input + "",
							"bRegex":  regex === null ? false : regex,
							"bSmart":  smart === null ? true : smart,
							"bCaseInsensitive": caseInsen === null ? true : caseInsen
						}
					);

					_fnFilterComplete( settings, settings.oPreviousSearch, 1 );
				}
			);
		}
	);

	_api_register(
		'state()',
		function () {
			return this.context.length ?
			this.context[0].oSavedState :
			null;
		}
	);

	_api_register(
		'state.clear()',
		function () {
			return this.iterator(
				'table',
				function ( settings ) {

					settings.fnStateSaveCallback.call( settings.oInstance, settings, {} );
				}
			);
		}
	);

	_api_register(
		'state.loaded()',
		function () {
			return this.context.length ?
			this.context[0].oLoadedState :
			null;
		}
	);

	_api_register(
		'state.save()',
		function () {
			return this.iterator(
				'table',
				function ( settings ) {
					_fnSaveState( settings );
				}
			);
		}
	);

	DataTable.versionCheck = DataTable.fnVersionCheck = function( version )
	{
		var aThis = DataTable.version.split( '.' );
		var aThat = version.split( '.' );
		var iThis, iThat;

		for ( var i = 0, iLen = aThat.length; i < iLen; i++ ) {
			iThis = parseInt( aThis[i], 10 ) || 0;
			iThat = parseInt( aThat[i], 10 ) || 0;

			if (iThis === iThat) {
				continue;
			}

			return iThis > iThat;
		}

		return true;
	};

	DataTable.isDataTable = DataTable.fnIsDataTable = function ( table )
	{
		var t  = $( table ).get( 0 );
		var is = false;

		if ( table instanceof DataTable.Api ) {
			return true;
		}

		$.each(
			DataTable.settings,
			function (i, o) {
				var head = o.nScrollHead ? $( 'table', o.nScrollHead )[0] : null;
				var foot = o.nScrollFoot ? $( 'table', o.nScrollFoot )[0] : null;

				if ( o.nTable === t || head === t || foot === t ) {
					is = true;
				}
			}
		);

		return is;
	};

	DataTable.tables = DataTable.fnTables = function ( visible )
	{
		var api = false;

		if ( $.isPlainObject( visible ) ) {
			api     = visible.api;
			visible = visible.visible;
		}

		var a = $.map(
			DataTable.settings,
			function (o) {
				if ( ! visible || (visible && $( o.nTable ).is( ':visible' )) ) {
					return o.nTable;
				}
			}
		);

		return api ?
			new _Api( a ) :
			a;
	};

	DataTable.camelToHungarian = _fnCamelToHungarian;

	_api_register(
		'$()',
		function ( selector, opts ) {
			var
			rows   = this.rows( opts ).nodes(),
			jqRows = $( rows );

			return $(
				[].concat(
					jqRows.filter( selector ).toArray(),
					jqRows.find( selector ).toArray()
				)
			);
		}
	);

	$.each(
		[ 'on', 'one', 'off' ],
		function (i, key) {
			_api_register(
				key + '()',
				function (  ) {
					var args = Array.prototype.slice.call( arguments );

					args[0] = $.map(
						args[0].split( /\s/ ),
						function ( e ) {
							return ! e.match( /\.dt\b/ ) ?
							e + '.dt' :
							e;
						}
					).join( ' ' );

					var inst = $( this.tables().nodes() );
					inst[key].apply( inst, args );
					return this;
				}
			);
		}
	);

	_api_register(
		'clear()',
		function () {
			return this.iterator(
				'table',
				function ( settings ) {
					_fnClearTable( settings );
				}
			);
		}
	);

	_api_register(
		'settings()',
		function () {
			return new _Api( this.context, this.context );
		}
	);

	_api_register(
		'init()',
		function () {
			var ctx = this.context;
			return ctx.length ? ctx[0].oInit : null;
		}
	);

	_api_register(
		'data()',
		function () {
			return this.iterator(
				'table',
				function ( settings ) {
					return _pluck( settings.aoData, '_aData' );
				}
			).flatten();
		}
	);

	_api_register(
		'destroy()',
		function ( remove ) {
			remove = remove || false;

			return this.iterator(
				'table',
				function ( settings ) {
					var orig      = settings.nTableWrapper.parentNode;
					var classes   = settings.oClasses;
					var table     = settings.nTable;
					var tbody     = settings.nTBody;
					var thead     = settings.nTHead;
					var tfoot     = settings.nTFoot;
					var jqTable   = $( table );
					var jqTbody   = $( tbody );
					var jqWrapper = $( settings.nTableWrapper );
					var rows      = $.map( settings.aoData, function (r) { return r.nTr; } );
					var i, ien;

					settings.bDestroying = true;

					_fnCallbackFire( settings, "aoDestroyCallback", "destroy", [settings] );

					if ( ! remove ) {
						new _Api( settings ).columns().visible( true );
					}

					jqWrapper.off( '.DT' ).find( ':not(tbody *)' ).off( '.DT' );
					$( window ).off( '.DT-' + settings.sInstance );

					if ( table != thead.parentNode ) {
						jqTable.children( 'thead' ).detach();
						jqTable.append( thead );
					}

					if ( tfoot && table != tfoot.parentNode ) {
						jqTable.children( 'tfoot' ).detach();
						jqTable.append( tfoot );
					}

					settings.aaSorting      = [];
					settings.aaSortingFixed = [];
					_fnSortingClasses( settings );

					$( rows ).removeClass( settings.asStripeClasses.join( ' ' ) );

					$( 'th, td', thead ).removeClass(
						classes.sSortable + ' ' +
						classes.sSortableAsc + ' ' + classes.sSortableDesc + ' ' + classes.sSortableNone
					);

					jqTbody.children().detach();
					jqTbody.append( rows );

					var removedMethod = remove ? 'remove' : 'detach';
					jqTable[ removedMethod ]();
					jqWrapper[ removedMethod ]();

					if ( ! remove && orig ) {

						orig.insertBefore( table, settings.nTableReinsertBefore );

						jqTable
						.css( 'width', settings.sDestroyWidth )
						.removeClass( classes.sTable );

						ien = settings.asDestroyStripes.length;

						if ( ien ) {
							jqTbody.children().each(
								function (i) {
									$( this ).addClass( settings.asDestroyStripes[i % ien] );
								}
							);
						}
					}

					var idx = $.inArray( settings, DataTable.settings );
					if ( idx !== -1 ) {
						DataTable.settings.splice( idx, 1 );
					}
				}
			);
		}
	);

	$.each(
		[ 'column', 'row', 'cell' ],
		function ( i, type ) {
			_api_register(
				type + 's().every()',
				function ( fn ) {
					var opts = this.selector.opts;
					var api  = this;

					return this.iterator(
						type,
						function ( settings, arg1, arg2, arg3, arg4 ) {

							fn.call(
								api[ type ](
									arg1,
									type === 'cell' ? arg2 : opts,
									type === 'cell' ? opts : undefined
								),
								arg1,
								arg2,
								arg3,
								arg4
							);
						}
					);
				}
			);
		}
	);

	_api_register(
		'i18n()',
		function ( token, def, plural ) {
			var ctx      = this.context[0];
			var resolved = _fnGetObjectDataFn( token )( ctx.oLanguage );

			if ( resolved === undefined ) {
				resolved = def;
			}

			if ( plural !== undefined && $.isPlainObject( resolved ) ) {
				resolved = resolved[ plural ] !== undefined ?
				resolved[ plural ] :
				resolved._;
			}

			return resolved.replace( '%d', plural );
		}
	);

	DataTable.version = "1.10.21";

	DataTable.settings = [];

	DataTable.models = {};

	DataTable.models.oSearch = {

		"bCaseInsensitive": true,

		"sSearch": "",

		"bRegex": false,

		"bSmart": true
	};

	DataTable.models.oRow = {

		"nTr": null,

		"anCells": null,

		"_aData": [],

		"_aSortData": null,

		"_aFilterData": null,

		"_sFilterRow": null,

		"_sRowStripe": "",

		"src": null,

		"idx": -1
	};

	DataTable.models.oColumn = {

		"idx": null,

		"aDataSort": null,

		"asSorting": null,

		"bSearchable": null,

		"bSortable": null,

		"bVisible": null,

		"_sManualType": null,

		"_bAttrSrc": false,

		"fnCreatedCell": null,

		"fnGetData": null,

		"fnSetData": null,

		"mData": null,

		"mRender": null,

		"nTh": null,

		"nTf": null,

		"sClass": null,

		"sContentPadding": null,

		"sDefaultContent": null,

		"sName": null,

		"sSortDataType": 'std',

		"sSortingClass": null,

		"sSortingClassJUI": null,

		"sTitle": null,

		"sType": null,

		"sWidth": null,

		"sWidthOrig": null
	};

	DataTable.defaults = {

		"aaData": null,

		"aaSorting": [[0,'asc']],

		"aaSortingFixed": [],

		"ajax": null,

		"aLengthMenu": [ 10, 25, 50, 100 ],

		"aoColumns": null,

		"aoColumnDefs": null,

		"aoSearchCols": [],

		"asStripeClasses": null,

		"bAutoWidth": true,

		"bDeferRender": false,

		"bDestroy": false,

		"bFilter": true,

		"bInfo": true,

		"bLengthChange": true,

		"bPaginate": true,

		"bProcessing": false,

		"bRetrieve": false,

		"bScrollCollapse": false,

		"bServerSide": false,

		"bSort": true,

		"bSortMulti": true,

		"bSortCellsTop": false,

		"bSortClasses": true,

		"bStateSave": false,

		"fnCreatedRow": null,

		"fnDrawCallback": null,

		"fnFooterCallback": null,

		"fnFormatNumber": function ( toFormat ) {
			return toFormat.toString().replace(
				/\B(?=(\d{3})+(?!\d))/g,
				this.oLanguage.sThousands
			);
		},

		"fnHeaderCallback": null,

		"fnInfoCallback": null,

		"fnInitComplete": null,

		"fnPreDrawCallback": null,

		"fnRowCallback": null,

		"fnServerData": null,

		"fnServerParams": null,

		"fnStateLoadCallback": function ( settings ) {
			try {
				return JSON.parse(
					(settings.iStateDuration === -1 ? sessionStorage : localStorage).getItem(
						'DataTables_' + settings.sInstance + '_' + location.pathname
					)
				);
			} catch (e) {
				return {};
			}
		},

		"fnStateLoadParams": null,

		"fnStateLoaded": null,

		"fnStateSaveCallback": function ( settings, data ) {
			try {
				(settings.iStateDuration === -1 ? sessionStorage : localStorage).setItem(
					'DataTables_' + settings.sInstance + '_' + location.pathname,
					JSON.stringify( data )
				);
			} catch (e) {
			}
		},

		"fnStateSaveParams": null,

		"iStateDuration": 7200,

		"iDeferLoading": null,

		"iDisplayLength": 10,

		"iDisplayStart": 0,

		"iTabIndex": 0,

		"oClasses": {},

		"oLanguage": {

			"oAria": {

				"sSortAscending": ": activate to sort column ascending",

				"sSortDescending": ": activate to sort column descending"
			},

			"oPaginate": {

				"sFirst": "First",

				"sLast": "Last",

				"sNext": "Next",

				"sPrevious": "Previous"
			},

			"sEmptyTable": "No data available in table",

			"sInfo": "Showing _START_ to _END_ of _TOTAL_ entries",

			"sInfoEmpty": "Showing 0 to 0 of 0 entries",

			"sInfoFiltered": "(filtered from _MAX_ total entries)",

			"sInfoPostFix": "",

			"sDecimal": "",

			"sThousands": ",",

			"sLengthMenu": "Show _MENU_ entries",

			"sLoadingRecords": "Loading...",

			"sProcessing": "Processing...",

			"sSearch": "Search:",

			"sSearchPlaceholder": "",

			"sUrl": "",

			"sZeroRecords": "No matching records found"
		},

		"oSearch": $.extend( {}, DataTable.models.oSearch ),

		"sAjaxDataProp": "data",

		"sAjaxSource": null,

		"sDom": "lfrtip",

		"searchDelay": null,

		"sPaginationType": "simple_numbers",

		"sScrollX": "",

		"sScrollXInner": "",

		"sScrollY": "",

		"sServerMethod": "GET",

		"renderer": null,

		"rowId": "DT_RowId"
	};

	_fnHungarianMap( DataTable.defaults );

	DataTable.defaults.column = {

		"aDataSort": null,
		"iDataSort": -1,

		"asSorting": [ 'asc', 'desc' ],

		"bSearchable": true,

		"bSortable": true,

		"bVisible": true,

		"fnCreatedCell": null,

		"mData": null,

		"mRender": null,

		"sCellType": "td",

		"sClass": "",

		"sContentPadding": "",

		"sDefaultContent": null,

		"sName": "",

		"sSortDataType": "std",

		"sTitle": null,

		"sType": null,

		"sWidth": null
	};

	_fnHungarianMap( DataTable.defaults.column );

	DataTable.models.oSettings = {

		"oFeatures": {

			"bAutoWidth": null,

			"bDeferRender": null,

			"bFilter": null,

			"bInfo": null,

			"bLengthChange": null,

			"bPaginate": null,

			"bProcessing": null,

			"bServerSide": null,

			"bSort": null,

			"bSortMulti": null,

			"bSortClasses": null,

			"bStateSave": null
		},

		"oScroll": {

			"bCollapse": null,

			"iBarWidth": 0,

			"sX": null,

			"sXInner": null,

			"sY": null
		},

		"oLanguage": {

			"fnInfoCallback": null
		},

		"oBrowser": {

			"bScrollOversize": false,

			"bScrollbarLeft": false,

			"bBounding": false,

			"barWidth": 0
		},

		"ajax": null,

		"aanFeatures": [],

		"aoData": [],

		"aiDisplay": [],

		"aiDisplayMaster": [],

		"aIds": {},

		"aoColumns": [],

		"aoHeader": [],

		"aoFooter": [],

		"oPreviousSearch": {},

		"aoPreSearchCols": [],

		"aaSorting": null,

		"aaSortingFixed": [],

		"asStripeClasses": null,

		"asDestroyStripes": [],

		"sDestroyWidth": 0,

		"aoRowCallback": [],

		"aoHeaderCallback": [],

		"aoFooterCallback": [],

		"aoDrawCallback": [],

		"aoRowCreatedCallback": [],

		"aoPreDrawCallback": [],

		"aoInitComplete": [],

		"aoStateSaveParams": [],

		"aoStateLoadParams": [],

		"aoStateLoaded": [],

		"sTableId": "",

		"nTable": null,

		"nTHead": null,

		"nTFoot": null,

		"nTBody": null,

		"nTableWrapper": null,

		"bDeferLoading": false,

		"bInitialised": false,

		"aoOpenRows": [],

		"sDom": null,

		"searchDelay": null,

		"sPaginationType": "two_button",

		"iStateDuration": 0,

		"aoStateSave": [],

		"aoStateLoad": [],

		"oSavedState": null,

		"oLoadedState": null,

		"sAjaxSource": null,

		"sAjaxDataProp": null,

		"bAjaxDataGet": true,

		"jqXHR": null,

		"json": undefined,

		"oAjaxData": undefined,

		"fnServerData": null,

		"aoServerParams": [],

		"sServerMethod": null,

		"fnFormatNumber": null,

		"aLengthMenu": null,

		"iDraw": 0,

		"bDrawing": false,

		"iDrawError": -1,

		"_iDisplayLength": 10,

		"_iDisplayStart": 0,

		"_iRecordsTotal": 0,

		"_iRecordsDisplay": 0,

		"oClasses": {},

		"bFiltered": false,

		"bSorted": false,

		"bSortCellsTop": null,

		"oInit": null,

		"aoDestroyCallback": [],

		"fnRecordsTotal": function ()
		{
			return _fnDataSource( this ) == 'ssp' ?
				this._iRecordsTotal * 1 :
				this.aiDisplayMaster.length;
		},

		"fnRecordsDisplay": function ()
		{
			return _fnDataSource( this ) == 'ssp' ?
				this._iRecordsDisplay * 1 :
				this.aiDisplay.length;
		},

		"fnDisplayEnd": function ()
		{
			var
				len      = this._iDisplayLength,
				start    = this._iDisplayStart,
				calc     = start + len,
				records  = this.aiDisplay.length,
				features = this.oFeatures,
				paginate = features.bPaginate;

			if ( features.bServerSide ) {
				return paginate === false || len === -1 ?
					start + records :
					Math.min( start + len, this._iRecordsDisplay );
			} else {
				return ! paginate || calc > records || len === -1 ?
				records :
					calc;
			}
		},

		"oInstance": null,

		"sInstance": null,

		"iTabIndex": 0,

		"nScrollHead": null,

		"nScrollFoot": null,

		"aLastSort": [],

		"oPlugins": {},

		"rowIdFn": null,

		"rowId": null
	};

	DataTable.ext = _ext = {

		buttons: {},

		classes: {},

		build:"dt/dt-1.10.21/b-1.6.2/cr-1.5.2/fc-3.3.1/fh-3.1.7/r-2.2.5",

		errMode: "alert",

		feature: [],

		search: [],

		selector: {
			cell: [],
			column: [],
			row: []
		},

		internal: {},

		legacy: {

			ajax: null
		},

		pager: {},

		renderer: {
			pageButton: {},
			header: {}
		},

		order: {},

		type: {

			detect: [],

			search: {},

			order: {}
		},

		_unique: 0,

		fnVersionCheck: DataTable.fnVersionCheck,

		iApiIndex: 0,

		oJUIClasses: {},

		sVersion: DataTable.version
	};

	$.extend(
		_ext,
		{
			afnFiltering: _ext.search,
			aTypes:       _ext.type.detect,
			ofnSearch:    _ext.type.search,
			oSort:        _ext.type.order,
			afnSortData:  _ext.order,
			aoFeatures:   _ext.feature,
			oApi:         _ext.internal,
			oStdClasses:  _ext.classes,
			oPagination:  _ext.pager
		}
	);

	$.extend(
		DataTable.ext.classes,
		{
			"sTable": "dataTable",
			"sNoFooter": "no-footer",

			"sPageButton": "paginate_button",
			"sPageButtonActive": "current",
			"sPageButtonDisabled": "disabled",

			"sStripeOdd": "odd",
			"sStripeEven": "even",

			"sRowEmpty": "dataTables_empty",

			"sWrapper": "dataTables_wrapper",
			"sFilter": "dataTables_filter",
			"sInfo": "dataTables_info",
			"sPaging": "dataTables_paginate paging_",
			"sLength": "dataTables_length",
			"sProcessing": "dataTables_processing",

			"sSortAsc": "sorting_asc",
			"sSortDesc": "sorting_desc",
			"sSortable": "sorting",
			"sSortableAsc": "sorting_asc_disabled",
			"sSortableDesc": "sorting_desc_disabled",
			"sSortableNone": "sorting_disabled",
			"sSortColumn": "sorting_",

			"sFilterInput": "",

			"sLengthSelect": "",

			"sScrollWrapper": "dataTables_scroll",
			"sScrollHead": "dataTables_scrollHead",
			"sScrollHeadInner": "dataTables_scrollHeadInner",
			"sScrollBody": "dataTables_scrollBody",
			"sScrollFoot": "dataTables_scrollFoot",
			"sScrollFootInner": "dataTables_scrollFootInner",

			"sHeaderTH": "",
			"sFooterTH": "",

			"sSortJUIAsc": "",
			"sSortJUIDesc": "",
			"sSortJUI": "",
			"sSortJUIAscAllowed": "",
			"sSortJUIDescAllowed": "",
			"sSortJUIWrapper": "",
			"sSortIcon": "",
			"sJUIHeader": "",
			"sJUIFooter": ""
		}
	);

	var extPagination = DataTable.ext.pager;

	function _numbers ( page, pages ) {
		var
			numbers = [],
			buttons = extPagination.numbers_length,
			half    = Math.floor( buttons / 2 ),
			i       = 1;

		if ( pages <= buttons ) {
			numbers = _range( 0, pages );
		} else if ( page <= half ) {
			numbers = _range( 0, buttons - 2 );
			numbers.push( 'ellipsis' );
			numbers.push( pages - 1 );
		} else if ( page >= pages - 1 - half ) {
			numbers = _range( pages - (buttons - 2), pages );
			numbers.splice( 0, 0, 'ellipsis' );
			numbers.splice( 0, 0, 0 );
		} else {
			numbers = _range( page - half + 2, page + half - 1 );
			numbers.push( 'ellipsis' );
			numbers.push( pages - 1 );
			numbers.splice( 0, 0, 'ellipsis' );
			numbers.splice( 0, 0, 0 );
		}

		numbers.DT_el = 'span';
		return numbers;
	}

	$.extend(
		extPagination,
		{
			simple: function ( page, pages ) {
				return [ 'previous', 'next' ];
			},

			full: function ( page, pages ) {
				return [  'first', 'previous', 'next', 'last' ];
			},

			numbers: function ( page, pages ) {
				return [ _numbers( page, pages ) ];
			},

			simple_numbers: function ( page, pages ) {
				return [ 'previous', _numbers( page, pages ), 'next' ];
			},

			full_numbers: function ( page, pages ) {
				return [ 'first', 'previous', _numbers( page, pages ), 'next', 'last' ];
			},

			first_last_numbers: function (page, pages) {
				return ['first', _numbers( page, pages ), 'last'];
			},

			_numbers: _numbers,

			numbers_length: 7
		}
	);

	$.extend(
		true,
		DataTable.ext.renderer,
		{
			pageButton: {
				_: function ( settings, host, idx, buttons, page, pages ) {
					var classes                       = settings.oClasses;
					var lang                          = settings.oLanguage.oPaginate;
					var aria                          = settings.oLanguage.oAria.paginate || {};
					var btnDisplay, btnClass, counter = 0;

					var attach = function( container, buttons ) {
						var i, ien, node, button, tabIndex;
						var disabledClass = classes.sPageButtonDisabled;
						var clickHandler  = function ( e ) {
							_fnPageChange( settings, e.data.action, true );
						};

						for ( i = 0, ien = buttons.length; i < ien; i++ ) {
							button = buttons[i];

							if ( $.isArray( button ) ) {
								var inner = $( '<' + (button.DT_el || 'div') + '/>' )
								.appendTo( container );
								attach( inner, button );
							} else {
								btnDisplay = null;
								btnClass   = button;
								tabIndex   = settings.iTabIndex;

								switch ( button ) {
									case 'ellipsis':
										container.append( '<span class="ellipsis">&#x2026;</span>' );
										break;

									case 'first':
										btnDisplay = lang.sFirst;

										if ( page === 0 ) {
											tabIndex  = -1;
											btnClass += ' ' + disabledClass;
										}
										break;

									case 'previous':
										btnDisplay = lang.sPrevious;

										if ( page === 0 ) {
											tabIndex  = -1;
											btnClass += ' ' + disabledClass;
										}
										break;

									case 'next':
										btnDisplay = lang.sNext;

										if ( pages === 0 || page === pages - 1 ) {
											tabIndex  = -1;
											btnClass += ' ' + disabledClass;
										}
										break;

									case 'last':
										btnDisplay = lang.sLast;

										if ( page === pages - 1 ) {
											tabIndex  = -1;
											btnClass += ' ' + disabledClass;
										}
										break;

									default:
										btnDisplay = button + 1;
										btnClass   = page === button ?
										classes.sPageButtonActive : '';
										break;
								}

								if ( btnDisplay !== null ) {
									node = $(
										'<a>',
										{
											'class': classes.sPageButton + ' ' + btnClass,
											'aria-controls': settings.sTableId,
											'aria-label': aria[ button ],
											'data-dt-idx': counter,
											'tabindex': tabIndex,
											'id': idx === 0 && typeof button === 'string' ?
											settings.sTableId + '_' + button :
											null
										}
									)
										.html( btnDisplay )
										.appendTo( container );

									_fnBindAction(
										node,
										{action: button},
										clickHandler
									);

									counter++;
								}
							}
						}
					};

					var activeEl;

					try {

						activeEl = $( host ).find( document.activeElement ).data( 'dt-idx' );
					} catch (e) {
					}

					attach( $( host ).empty(), buttons );

					if ( activeEl !== undefined ) {
						$( host ).find( '[data-dt-idx=' + activeEl + ']' ).trigger( 'focus' );
					}
				}
			}
		}
	);

	$.extend(
		DataTable.ext.type.detect,
		[

		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _isNumber( d, decimal ) ? 'num' + decimal : null;
		},

		function ( d, settings )
		{

			if ( d && ! (d instanceof Date) && ! _re_date.test( d ) ) {
				return null;
			}
			var parsed = Date.parse( d );
			return (parsed !== null && ! isNaN( parsed )) || _empty( d ) ? 'date' : null;
		},

		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _isNumber( d, decimal, true ) ? 'num-fmt' + decimal : null;
		},

		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _htmlNumeric( d, decimal ) ? 'html-num' + decimal : null;
		},

		function ( d, settings )
		{
			var decimal = settings.oLanguage.sDecimal;
			return _htmlNumeric( d, decimal, true ) ? 'html-num-fmt' + decimal : null;
		},

		function ( d, settings )
		{
			return _empty( d ) || (typeof d === 'string' && d.indexOf( '<' ) !== -1) ?
				'html' : null;
		}
		]
	);

	$.extend(
		DataTable.ext.type.search,
		{
			html: function ( data ) {
				return _empty( data ) ?
				data :
				typeof data === 'string' ?
					data
						.replace( _re_new_lines, " " )
						.replace( _re_html, "" ) :
					'';
			},

			string: function ( data ) {
				return _empty( data ) ?
				data :
				typeof data === 'string' ?
					data.replace( _re_new_lines, " " ) :
					data;
			}
		}
	);

	var __numericReplace = function ( d, decimalPlace, re1, re2 ) {
		if ( d !== 0 && ( ! d || d === '-') ) {
			return -Infinity;
		}

		if ( decimalPlace ) {
			d = _numToDecimal( d, decimalPlace );
		}

		if ( d.replace ) {
			if ( re1 ) {
				d = d.replace( re1, '' );
			}

			if ( re2 ) {
				d = d.replace( re2, '' );
			}
		}

		return d * 1;
	};

	function _addNumericSort ( decimalPlace ) {
		$.each(
			{

				"num": function ( d ) {
					return __numericReplace( d, decimalPlace );
				},

				"num-fmt": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_formatted_numeric );
				},

				"html-num": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_html );
				},

				"html-num-fmt": function ( d ) {
					return __numericReplace( d, decimalPlace, _re_html, _re_formatted_numeric );
				}
			},
			function ( key, fn ) {

				_ext.type.order[ key + decimalPlace + '-pre' ] = fn;

				if ( key.match( /^html\-/ ) ) {
					_ext.type.search[ key + decimalPlace ] = _ext.type.search.html;
				}
			}
		);
	}

	$.extend(
		_ext.type.order,
		{

			"date-pre": function ( d ) {
				var ts = Date.parse( d );
				return isNaN( ts ) ? -Infinity : ts;
			},

			"html-pre": function ( a ) {
				return _empty( a ) ?
				'' :
				a.replace ?
					a.replace( /<.*?>/g, "" ).toLowerCase() :
					a + '';
			},

			"string-pre": function ( a ) {

				return _empty( a ) ?
				'' :
				typeof a === 'string' ?
					a.toLowerCase() :
					! a.toString ?
				'' :
						a.toString();
			},

			"string-asc": function ( x, y ) {
				return ((x < y) ? -1 : ((x > y) ? 1 : 0));
			},

			"string-desc": function ( x, y ) {
				return ((x < y) ? 1 : ((x > y) ? -1 : 0));
			}
		}
	);

	_addNumericSort( '' );

	$.extend(
		true,
		DataTable.ext.renderer,
		{
			header: {
				_: function ( settings, cell, column, classes ) {

					$( settings.nTable ).on(
						'order.dt.DT',
						function ( e, ctx, sorting, columns ) {
							if ( settings !== ctx ) {
								return;
							}

							var colIdx = column.idx;

							cell
							.removeClass(
								column.sSortingClass + ' ' +
								classes.sSortAsc + ' ' +
								classes.sSortDesc
							)
							.addClass(
								columns[ colIdx ] == 'asc' ?
								classes.sSortAsc : columns[ colIdx ] == 'desc' ?
									classes.sSortDesc :
									column.sSortingClass
							);
						}
					);
				},

				jqueryui: function ( settings, cell, column, classes ) {
					$( '<div/>' )
					.addClass( classes.sSortJUIWrapper )
					.append( cell.contents() )
					.append(
						$( '<span/>' )
						.addClass( classes.sSortIcon + ' ' + column.sSortingClassJUI )
					)
					.appendTo( cell );

					$( settings.nTable ).on(
						'order.dt.DT',
						function ( e, ctx, sorting, columns ) {
							if ( settings !== ctx ) {
								return;
							}

							var colIdx = column.idx;

							cell
							.removeClass( classes.sSortAsc + " " + classes.sSortDesc )
							.addClass(
								columns[ colIdx ] == 'asc' ?
								classes.sSortAsc : columns[ colIdx ] == 'desc' ?
									classes.sSortDesc :
									column.sSortingClass
							);

							cell
							.find( 'span.' + classes.sSortIcon )
							.removeClass(
								classes.sSortJUIAsc + " " +
								classes.sSortJUIDesc + " " +
								classes.sSortJUI + " " +
								classes.sSortJUIAscAllowed + " " +
								classes.sSortJUIDescAllowed
							)
							.addClass(
								columns[ colIdx ] == 'asc' ?
								classes.sSortJUIAsc : columns[ colIdx ] == 'desc' ?
									classes.sSortJUIDesc :
									column.sSortingClassJUI
							);
						}
					);
				}
			}
		}
	);

	var __htmlEscapeEntities = function ( d ) {
		return typeof d === 'string' ?
			d
				.replace( /&/g, '&amp;' )
				.replace( /</g, '&lt;' )
				.replace( />/g, '&gt;' )
				.replace( /"/g, '&quot;' ) :
			d;
	};

	DataTable.render = {
		number: function ( thousands, decimal, precision, prefix, postfix ) {
			return {
				display: function ( d ) {
					if ( typeof d !== 'number' && typeof d !== 'string' ) {
						return d;
					}

					var negative = d < 0 ? '-' : '';
					var flo      = parseFloat( d );

					if ( isNaN( flo ) ) {
						return __htmlEscapeEntities( d );
					}

					flo = flo.toFixed( precision );
					d   = Math.abs( flo );

					var intPart   = parseInt( d, 10 );
					var floatPart = precision ?
						decimal + (d - intPart).toFixed( precision ).substring( 2 ):
						'';

					return negative + (prefix || '') +
						intPart.toString().replace(
							/\B(?=(\d{3})+(?!\d))/g,
							thousands
						) +
						floatPart +
						(postfix || '');
				}
			};
		},

		text: function () {
			return {
				display: __htmlEscapeEntities,
				filter: __htmlEscapeEntities
			};
		}
	};

	function _fnExternApiFunc (fn)
	{
		return function() {
			var args = [_fnSettingsFromNode( this[DataTable.ext.iApiIndex] )].concat(
				Array.prototype.slice.call( arguments )
			);
			return DataTable.ext.internal[fn].apply( this, args );
		};
	}

	$.extend(
		DataTable.ext.internal,
		{
			_fnExternApiFunc: _fnExternApiFunc,
			_fnBuildAjax: _fnBuildAjax,
			_fnAjaxUpdate: _fnAjaxUpdate,
			_fnAjaxParameters: _fnAjaxParameters,
			_fnAjaxUpdateDraw: _fnAjaxUpdateDraw,
			_fnAjaxDataSrc: _fnAjaxDataSrc,
			_fnAddColumn: _fnAddColumn,
			_fnColumnOptions: _fnColumnOptions,
			_fnAdjustColumnSizing: _fnAdjustColumnSizing,
			_fnVisibleToColumnIndex: _fnVisibleToColumnIndex,
			_fnColumnIndexToVisible: _fnColumnIndexToVisible,
			_fnVisbleColumns: _fnVisbleColumns,
			_fnGetColumns: _fnGetColumns,
			_fnColumnTypes: _fnColumnTypes,
			_fnApplyColumnDefs: _fnApplyColumnDefs,
			_fnHungarianMap: _fnHungarianMap,
			_fnCamelToHungarian: _fnCamelToHungarian,
			_fnLanguageCompat: _fnLanguageCompat,
			_fnBrowserDetect: _fnBrowserDetect,
			_fnAddData: _fnAddData,
			_fnAddTr: _fnAddTr,
			_fnNodeToDataIndex: _fnNodeToDataIndex,
			_fnNodeToColumnIndex: _fnNodeToColumnIndex,
			_fnGetCellData: _fnGetCellData,
			_fnSetCellData: _fnSetCellData,
			_fnSplitObjNotation: _fnSplitObjNotation,
			_fnGetObjectDataFn: _fnGetObjectDataFn,
			_fnSetObjectDataFn: _fnSetObjectDataFn,
			_fnGetDataMaster: _fnGetDataMaster,
			_fnClearTable: _fnClearTable,
			_fnDeleteIndex: _fnDeleteIndex,
			_fnInvalidate: _fnInvalidate,
			_fnGetRowElements: _fnGetRowElements,
			_fnCreateTr: _fnCreateTr,
			_fnBuildHead: _fnBuildHead,
			_fnDrawHead: _fnDrawHead,
			_fnDraw: _fnDraw,
			_fnReDraw: _fnReDraw,
			_fnAddOptionsHtml: _fnAddOptionsHtml,
			_fnDetectHeader: _fnDetectHeader,
			_fnGetUniqueThs: _fnGetUniqueThs,
			_fnFeatureHtmlFilter: _fnFeatureHtmlFilter,
			_fnFilterComplete: _fnFilterComplete,
			_fnFilterCustom: _fnFilterCustom,
			_fnFilterColumn: _fnFilterColumn,
			_fnFilter: _fnFilter,
			_fnFilterCreateSearch: _fnFilterCreateSearch,
			_fnEscapeRegex: _fnEscapeRegex,
			_fnFilterData: _fnFilterData,
			_fnFeatureHtmlInfo: _fnFeatureHtmlInfo,
			_fnUpdateInfo: _fnUpdateInfo,
			_fnInfoMacros: _fnInfoMacros,
			_fnInitialise: _fnInitialise,
			_fnInitComplete: _fnInitComplete,
			_fnLengthChange: _fnLengthChange,
			_fnFeatureHtmlLength: _fnFeatureHtmlLength,
			_fnFeatureHtmlPaginate: _fnFeatureHtmlPaginate,
			_fnPageChange: _fnPageChange,
			_fnFeatureHtmlProcessing: _fnFeatureHtmlProcessing,
			_fnProcessingDisplay: _fnProcessingDisplay,
			_fnFeatureHtmlTable: _fnFeatureHtmlTable,
			_fnScrollDraw: _fnScrollDraw,
			_fnApplyToChildren: _fnApplyToChildren,
			_fnCalculateColumnWidths: _fnCalculateColumnWidths,
			_fnThrottle: _fnThrottle,
			_fnConvertToWidth: _fnConvertToWidth,
			_fnGetWidestNode: _fnGetWidestNode,
			_fnGetMaxLenString: _fnGetMaxLenString,
			_fnStringToCss: _fnStringToCss,
			_fnSortFlatten: _fnSortFlatten,
			_fnSort: _fnSort,
			_fnSortAria: _fnSortAria,
			_fnSortListener: _fnSortListener,
			_fnSortAttachListener: _fnSortAttachListener,
			_fnSortingClasses: _fnSortingClasses,
			_fnSortData: _fnSortData,
			_fnSaveState: _fnSaveState,
			_fnLoadState: _fnLoadState,
			_fnSettingsFromNode: _fnSettingsFromNode,
			_fnLog: _fnLog,
			_fnMap: _fnMap,
			_fnBindAction: _fnBindAction,
			_fnCallbackReg: _fnCallbackReg,
			_fnCallbackFire: _fnCallbackFire,
			_fnLengthOverflow: _fnLengthOverflow,
			_fnRenderer: _fnRenderer,
			_fnDataSource: _fnDataSource,
			_fnRowAttributes: _fnRowAttributes,
			_fnExtend: _fnExtend,
			_fnCalculateEnd: function () {}

		}
	);

	$.fn.dataTable = DataTable;

	DataTable.$ = $;

	$.fn.dataTableSettings = DataTable.settings;
	$.fn.dataTableExt      = DataTable.ext;

	$.fn.DataTable = function ( opts ) {
		return $( this ).dataTable( opts ).api();
	};

	$.each(
		DataTable,
		function ( prop, val ) {
			$.fn.DataTable[ prop ] = val;
		}
	);

	return $.fn.dataTable;
}));

(function( factory ){
	if ( typeof define === 'function' && define.amd ) {

		define(
			['jquery', 'datatables.net'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {

		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}

			if ( ! $ || ! $.fn.dataTable ) {
				$ = require( 'datatables.net' )( root, $ ).$;
			}

			return factory( $, root, root.document );
		};
	} else {

		factory( jQuery, window, document );
	}
}(function( $, window, document, undefined ) {
	'use strict';
	var DataTable = $.fn.dataTable;

	var _instCounter = 0;

	var _buttonCounter = 0;

	var _dtButtons = DataTable.ext.buttons;

	function _fadeIn(el, duration, fn) {
		if ($.fn.animate) {
			el
			.stop()
			.fadeIn( duration, fn );
		} else {
			el.css( 'display', 'block' );

			if (fn) {
				fn.call( el );
			}
		}
	}

	function _fadeOut(el, duration, fn) {
		if ($.fn.animate) {
			el
			.stop()
			.fadeOut( duration, fn );
		} else {
			el.css( 'display', 'none' );

			if (fn) {
				fn.call( el );
			}
		}
	}

	var Buttons = function( dt, config )
{

		if ( ! (this instanceof Buttons) ) {
			return function (settings) {
				return new Buttons( settings, dt ).container();
			};
		}

		if ( typeof( config ) === 'undefined' ) {
			config = {};
		}

		if ( config === true ) {
			config = {};
		}

		if ( $.isArray( config ) ) {
			config = { buttons: config };
		}

		this.c = $.extend( true, {}, Buttons.defaults, config );

		if ( config.buttons ) {
			this.c.buttons = config.buttons;
		}

		this.s = {
			dt: new DataTable.Api( dt ),
			buttons: [],
			listenKeys: '',
			namespace: 'dtb' + (_instCounter++)
		};

		this.dom = {
			container: $( '<' + this.c.dom.container.tag + '/>' )
			.addClass( this.c.dom.container.className )
		};

		this._constructor();
	};

	$.extend(
		Buttons.prototype,
		{

			action: function ( node, action )
		{
				var button = this._nodeToButton( node );

				if ( action === undefined ) {
					return button.conf.action;
				}

				button.conf.action = action;

				return this;
			},

			active: function ( node, flag ) {
				var button = this._nodeToButton( node );
				var klass  = this.c.dom.button.active;
				var jqNode = $( button.node );

				if ( flag === undefined ) {
					return jqNode.hasClass( klass );
				}

				jqNode.toggleClass( klass, flag === undefined ? true : flag );

				return this;
			},

			add: function ( config, idx )
		{
				var buttons = this.s.buttons;

				if ( typeof idx === 'string' ) {
					var split = idx.split( '-' );
					var base  = this.s;

					for ( var i = 0, ien = split.length - 1; i < ien; i++ ) {
						base = base.buttons[ split[i] * 1 ];
					}

					buttons = base.buttons;
					idx     = split[ split.length - 1 ] * 1;
				}

				this._expandButton( buttons, config, base !== undefined, idx );
				this._draw();

				return this;
			},

			container: function ()
		{
				return this.dom.container;
			},

			disable: function ( node ) {
				var button = this._nodeToButton( node );

				$( button.node )
				.addClass( this.c.dom.button.disabled )
				.attr( 'disabled', true );

				return this;
			},

			destroy: function ()
		{

				$( 'body' ).off( 'keyup.' + this.s.namespace );

				var buttons = this.s.buttons.slice();
				var i, ien;

				for ( i = 0, ien = buttons.length; i < ien; i++ ) {
					this.remove( buttons[i].node );
				}

				this.dom.container.remove();

				var buttonInsts = this.s.dt.settings()[0];

				for ( i = 0, ien = buttonInsts.length; i < ien; i++ ) {
					if ( buttonInsts.inst === this ) {
						buttonInsts.splice( i, 1 );
						break;
					}
				}

				return this;
			},

			enable: function ( node, flag )
		{
				if ( flag === false ) {
					return this.disable( node );
				}

				var button = this._nodeToButton( node );
				$( button.node )
				.removeClass( this.c.dom.button.disabled )
				.removeAttr( 'disabled' );

				return this;
			},

			name: function ()
		{
				return this.c.name;
			},

			node: function ( node )
		{
				if ( ! node ) {
					return this.dom.container;
				}

				var button = this._nodeToButton( node );
				return $( button.node );
			},

			processing: function ( node, flag )
		{
				var dt     = this.s.dt;
				var button = this._nodeToButton( node );

				if ( flag === undefined ) {
					return $( button.node ).hasClass( 'processing' );
				}

				$( button.node ).toggleClass( 'processing', flag );

				$( dt.table().node() ).triggerHandler(
					'buttons-processing.dt',
					[
					flag, dt.button( node ), dt, $( node ), button.conf
					]
				);

				return this;
			},

			remove: function ( node )
		{
				var button = this._nodeToButton( node );
				var host   = this._nodeToHost( node );
				var dt     = this.s.dt;

				if ( button.buttons.length ) {
					for ( var i = button.buttons.length - 1; i >= 0; i-- ) {
						this.remove( button.buttons[i].node );
					}
				}

				if ( button.conf.destroy ) {
					button.conf.destroy.call( dt.button( node ), dt, $( node ), button.conf );
				}

				this._removeKey( button.conf );

				$( button.node ).remove();

				var idx = $.inArray( button, host );
				host.splice( idx, 1 );

				return this;
			},

			text: function ( node, label )
		{
				var button      = this._nodeToButton( node );
				var buttonLiner = this.c.dom.collection.buttonLiner;
				var linerTag    = button.inCollection && buttonLiner && buttonLiner.tag ?
				buttonLiner.tag :
				this.c.dom.buttonLiner.tag;
				var dt          = this.s.dt;
				var jqNode      = $( button.node );
				var text        = function ( opt ) {
					return typeof opt === 'function' ?
					opt( dt, jqNode, button.conf ) :
					opt;
				};

				if ( label === undefined ) {
					return text( button.conf.text );
				}

				button.conf.text = label;

				if ( linerTag ) {
					jqNode.children( linerTag ).html( text( label ) );
				} else {
					jqNode.html( text( label ) );
				}

				return this;
			},

			_constructor: function ()
		{
				var that       = this;
				var dt         = this.s.dt;
				var dtSettings = dt.settings()[0];
				var buttons    = this.c.buttons;

				if ( ! dtSettings._buttons ) {
					dtSettings._buttons = [];
				}

				dtSettings._buttons.push(
					{
						inst: this,
						name: this.c.name
						}
				);

				for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
					this.add( buttons[i] );
				}

				dt.on(
					'destroy',
					function ( e, settings ) {
						if ( settings === dtSettings ) {
							that.destroy();
						}
					}
				);

				$( 'body' ).on(
					'keyup.' + this.s.namespace,
					function ( e ) {
						if ( ! document.activeElement || document.activeElement === document.body ) {

							var character = String.fromCharCode( e.keyCode ).toLowerCase();

							if ( that.s.listenKeys.toLowerCase().indexOf( character ) !== -1 ) {
								that._keypress( character, e );
							}
						}
					}
				);
			},

			_addKey: function ( conf )
		{
				if ( conf.key ) {
					this.s.listenKeys += $.isPlainObject( conf.key ) ?
					conf.key.key :
					conf.key;
				}
			},

			_draw: function ( container, buttons )
		{
				if ( ! container ) {
					container = this.dom.container;
					buttons   = this.s.buttons;
				}

				container.children().detach();

				for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
					container.append( buttons[i].inserter );
					container.append( ' ' );

					if ( buttons[i].buttons && buttons[i].buttons.length ) {
						this._draw( buttons[i].collection, buttons[i].buttons );
					}
				}
			},

			_expandButton: function ( attachTo, button, inCollection, attachPoint )
		{
				var dt            = this.s.dt;
				var buttonCounter = 0;
				var buttons       = ! $.isArray( button ) ?
				[ button ] :
				button;

				for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
					var conf = this._resolveExtends( buttons[i] );

					if ( ! conf ) {
						continue;
					}

					if ( $.isArray( conf ) ) {
						this._expandButton( attachTo, conf, inCollection, attachPoint );
						continue;
					}

					var built = this._buildButton( conf, inCollection );
					if ( ! built ) {
						continue;
					}

					if ( attachPoint !== undefined && attachPoint !== null ) {
						attachTo.splice( attachPoint, 0, built );
						attachPoint++;
					} else {
						attachTo.push( built );
					}

					if ( built.conf.buttons ) {
						built.collection = $( '<' + this.c.dom.collection.tag + '/>' );

						built.conf._collection = built.collection;

						this._expandButton( built.buttons, built.conf.buttons, true, attachPoint );
					}

					if ( conf.init ) {
						conf.init.call( dt.button( built.node ), dt, $( built.node ), conf );
					}

					buttonCounter++;
				}
			},

			_buildButton: function ( config, inCollection )
		{
				var buttonDom     = this.c.dom.button;
				var linerDom      = this.c.dom.buttonLiner;
				var collectionDom = this.c.dom.collection;
				var dt            = this.s.dt;

				var text = function ( opt ) {
					return typeof opt === 'function' ?
					opt( dt, button, config ) :
					opt;
				};

				if ( inCollection && collectionDom.button ) {
					buttonDom = collectionDom.button;
				}

				if ( inCollection && collectionDom.buttonLiner ) {
					linerDom = collectionDom.buttonLiner;
				}

				if ( config.available && ! config.available( dt, config ) ) {
					return false;
				}

				var action     = function ( e, dt, button, config ) {
					config.action.call( dt.button( button ), e, dt, button, config );

					$( dt.table().node() ).triggerHandler(
						'buttons-action.dt',
						[
						dt.button( button ), dt, button, config
						]
					);
				};
				var tag        = config.tag || buttonDom.tag;
				var clickBlurs = config.clickBlurs === undefined ? true : config.clickBlurs
				var button     = $( '<' + tag + '/>' )
				.addClass( buttonDom.className )
				.attr( 'tabindex', this.s.dt.settings()[0].iTabIndex )
				.attr( 'aria-controls', this.s.dt.table().node().id )
				.on(
					'click.dtb',
					function (e) {
						e.preventDefault();

						if ( ! button.hasClass( buttonDom.disabled ) && config.action ) {
							action( e, dt, button, config );
						}
						if ( clickBlurs ) {
							button.trigger( 'blur' );
						}
					}
				)
				.on(
					'keyup.dtb',
					function (e) {
						if ( e.keyCode === 13 ) {
							if ( ! button.hasClass( buttonDom.disabled ) && config.action ) {
								action( e, dt, button, config );
							}
						}
					}
				);

				if ( tag.toLowerCase() === 'a' ) {
					button.attr( 'href', '#' );
				}

				if ( tag.toLowerCase() === 'button' ) {
					button.attr( 'type', 'button' );
				}

				if ( linerDom.tag ) {
					var liner = $( '<' + linerDom.tag + '/>' )
					.html( text( config.text ) )
					.addClass( linerDom.className );

					if ( linerDom.tag.toLowerCase() === 'a' ) {
						liner.attr( 'href', '#' );
					}

					button.append( liner );
				} else {
					button.html( text( config.text ) );
				}

				if ( config.enabled === false ) {
					button.addClass( buttonDom.disabled );
				}

				if ( config.className ) {
					button.addClass( config.className );
				}

				if ( config.titleAttr ) {
					button.attr( 'title', text( config.titleAttr ) );
				}

				if ( config.attr ) {
					button.attr( config.attr );
				}

				if ( ! config.namespace ) {
					config.namespace = '.dt-button-' + (_buttonCounter++);
				}

				var buttonContainer = this.c.dom.buttonContainer;
				var inserter;
				if ( buttonContainer && buttonContainer.tag ) {
					inserter = $( '<' + buttonContainer.tag + '/>' )
					.addClass( buttonContainer.className )
					.append( button );
				} else {
					inserter = button;
				}

				this._addKey( config );

				if ( this.c.buttonCreated ) {
					inserter = this.c.buttonCreated( config, inserter );
				}

				return {
					conf:         config,
					node:         button.get( 0 ),
					inserter:     inserter,
					buttons:      [],
					inCollection: inCollection,
					collection:   null
				};
			},

			_nodeToButton: function ( node, buttons )
		{
				if ( ! buttons ) {
					buttons = this.s.buttons;
				}

				for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
					if ( buttons[i].node === node ) {
						return buttons[i];
					}

					if ( buttons[i].buttons.length ) {
						var ret = this._nodeToButton( node, buttons[i].buttons );

						if ( ret ) {
							return ret;
						}
					}
				}
			},

			_nodeToHost: function ( node, buttons )
		{
				if ( ! buttons ) {
					buttons = this.s.buttons;
				}

				for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
					if ( buttons[i].node === node ) {
						return buttons;
					}

					if ( buttons[i].buttons.length ) {
						var ret = this._nodeToHost( node, buttons[i].buttons );

						if ( ret ) {
							return ret;
						}
					}
				}
			},

			_keypress: function ( character, e )
		{

				if ( e._buttonsHandled ) {
					return;
				}

				var run = function ( conf, node ) {
					if ( ! conf.key ) {
						return;
					}

					if ( conf.key === character ) {
						e._buttonsHandled = true;
						$( node ).click();
					} else if ( $.isPlainObject( conf.key ) ) {
						if ( conf.key.key !== character ) {
							return;
						}

						if ( conf.key.shiftKey && ! e.shiftKey ) {
							return;
						}

						if ( conf.key.altKey && ! e.altKey ) {
							return;
						}

						if ( conf.key.ctrlKey && ! e.ctrlKey ) {
							return;
						}

						if ( conf.key.metaKey && ! e.metaKey ) {
							return;
						}

						e._buttonsHandled = true;
						$( node ).click();
					}
				};

				var recurse = function ( a ) {
					for ( var i = 0, ien = a.length; i < ien; i++ ) {
						run( a[i].conf, a[i].node );

						if ( a[i].buttons.length ) {
							recurse( a[i].buttons );
						}
					}
				};

				recurse( this.s.buttons );
			},

			_removeKey: function ( conf )
		{
				if ( conf.key ) {
					var character = $.isPlainObject( conf.key ) ?
					conf.key.key :
					conf.key;

					var a   = this.s.listenKeys.split( '' );
					var idx = $.inArray( character, a );
					a.splice( idx, 1 );
					this.s.listenKeys = a.join( '' );
				}
			},

			_resolveExtends: function ( conf )
		{
				var dt = this.s.dt;
				var i, ien;
				var toConfObject = function ( base ) {
					var loop = 0;

					while ( ! $.isPlainObject( base ) && ! $.isArray( base ) ) {
						if ( base === undefined ) {
							return;
						}

						if ( typeof base === 'function' ) {
							base = base( dt, conf );

							if ( ! base ) {
								return false;
							}
						} else if ( typeof base === 'string' ) {
							if ( ! _dtButtons[ base ] ) {
								throw 'Unknown button type: ' + base;
							}

							base = _dtButtons[ base ];
						}

						loop++;
						if ( loop > 30 ) {

							throw 'Buttons: Too many iterations';
						}
					}

					return $.isArray( base ) ?
					base :
					$.extend( {}, base );
				};

				conf = toConfObject( conf );

				while ( conf && conf.extend ) {

					if ( ! _dtButtons[ conf.extend ] ) {
						throw 'Cannot extend unknown button type: ' + conf.extend;
					}

					var objArray = toConfObject( _dtButtons[ conf.extend ] );
					if ( $.isArray( objArray ) ) {
						return objArray;
					} else if ( ! objArray ) {

						return false;
					}

					var originalClassName = objArray.className;

					conf = $.extend( {}, objArray, conf );

					if ( originalClassName && conf.className !== originalClassName ) {
						conf.className = originalClassName + ' ' + conf.className;
					}

					var postfixButtons = conf.postfixButtons;
					if ( postfixButtons ) {
						if ( ! conf.buttons ) {
							conf.buttons = [];
						}

						for ( i = 0, ien = postfixButtons.length; i < ien; i++ ) {
							conf.buttons.push( postfixButtons[i] );
						}

						conf.postfixButtons = null;
					}

					var prefixButtons = conf.prefixButtons;
					if ( prefixButtons ) {
						if ( ! conf.buttons ) {
							conf.buttons = [];
						}

						for ( i = 0, ien = prefixButtons.length; i < ien; i++ ) {
							conf.buttons.splice( i, 0, prefixButtons[i] );
						}

						conf.prefixButtons = null;
					}

					conf.extend = objArray.extend;
				}

				return conf;
			},

			_popover: function ( content, hostButton, inOpts ) {
				var dt              = hostButton;
				var buttonsSettings = this.c;
				var options         = $.extend(
					{
						align: 'button-left',
						autoClose: false,
						background: true,
						backgroundClassName: 'dt-button-background',
						contentClassName: buttonsSettings.dom.collection.className,
						collectionLayout: '',
						collectionTitle: '',
						dropup: false,
						fade: 400,
						rightAlignClassName: 'dt-button-right',
						tag: buttonsSettings.dom.collection.tag
					},
					inOpts
				);
				var hostNode        = hostButton.node();

				var close = function () {
					_fadeOut(
						$( '.dt-button-collection' ),
						options.fade,
						function () {
							$( this ).detach();
						}
					);

					$( dt.buttons( '[aria-haspopup="true"][aria-expanded="true"]' ).nodes() )
					.attr( 'aria-expanded', 'false' );

					$( 'div.dt-button-background' ).off( 'click.dtb-collection' );
					Buttons.background( false, options.backgroundClassName, options.fade, hostNode );

					$( 'body' ).off( '.dtb-collection' );
					dt.off( 'buttons-action.b-internal' );
				};

				if (content === false) {
					close();
				}

				var existingExpanded = $( dt.buttons( '[aria-haspopup="true"][aria-expanded="true"]' ).nodes() );
				if ( existingExpanded.length ) {
					hostNode = existingExpanded.eq( 0 );

					close();
				}

				var display = $( '<div/>' )
				.addClass( 'dt-button-collection ColVis_collection TableTools_collection ui-buttonset ui-buttonset-multi' )
				.addClass( options.collectionLayout )
				.css( 'display', 'none' );

				content = $( content )
				.addClass( options.contentClassName )
				.attr( 'role', 'menu' )
				.appendTo( display );

				hostNode.attr( 'aria-expanded', 'true' );

				if ( hostNode.parents( 'body' )[0] !== document.body ) {
					hostNode = document.body.lastChild;
				}

				if ( options.collectionTitle ) {
					display.prepend( '<div class="dt-button-collection-title">' + options.collectionTitle + '</div>' );
				}

				_fadeIn( display.insertAfter( hostNode ) );

				var tableContainer = $( hostButton.table().container() );
				var position       = display.css( 'position' );

				if ( options.align === 'dt-container' ) {
					hostNode = hostNode.parent();
					display.css( 'width', tableContainer.width() );
				}

				if ( position === 'absolute' ) {

					var hostPosition = hostNode.position();

					display.css(
						{
							top: hostPosition.top + hostNode.outerHeight(),
							left: hostPosition.left
						}
					);

					var collectionHeight = display.outerHeight();
					var collectionWidth  = display.outerWidth();
					var tableBottom      = tableContainer.offset().top + tableContainer.height();
					var listBottom       = hostPosition.top + hostNode.outerHeight() + collectionHeight;
					var bottomOverflow   = listBottom - tableBottom;

					var listTop     = hostPosition.top - collectionHeight;
					var tableTop    = tableContainer.offset().top;
					var topOverflow = tableTop - listTop;

					var moveTop = hostPosition.top - collectionHeight - 5;
					if ( (bottomOverflow > topOverflow || options.dropup) && -moveTop < tableTop ) {
						display.css( 'top', moveTop );
					}

					var tableLeft  = tableContainer.offset().left;
					var tableWidth = tableContainer.width();
					var tableRight = tableLeft + tableWidth;

					var popoverLeft  = display.offset().left;
					var popoverWidth = display.width();
					var popoverRight = popoverLeft + popoverWidth;

					var buttonsLeft  = hostNode.offset().left;
					var buttonsWidth = hostNode.outerWidth()
					var buttonsRight = buttonsLeft + buttonsWidth;

					var popoverShuffle = 0;
					if ( display.hasClass( options.rightAlignClassName ) || options.align === 'button-right' ) {
						popoverShuffle = buttonsRight - popoverRight;
						if (tableLeft > (popoverLeft + popoverShuffle)) {
							var leftGap  = tableLeft - (popoverLeft + popoverShuffle);
							var rightGap = tableRight - (popoverRight + popoverShuffle);

							if (leftGap > rightGap) {
								popoverShuffle += rightGap;
							} else {
								popoverShuffle += leftGap;
							}
						}
					} else {
						popoverShuffle = tableLeft - popoverLeft;

						if (tableRight < (popoverRight + popoverShuffle)) {
							var leftGap  = tableLeft - (popoverLeft + popoverShuffle);
							var rightGap = tableRight - (popoverRight + popoverShuffle);

							if (leftGap > rightGap ) {
								popoverShuffle += rightGap;
							} else {
								popoverShuffle += leftGap;
							}

						}
					}

					display.css( 'left', display.position().left + popoverShuffle );

				} else {

					var top = display.height() / 2;
					if ( top > $( window ).height() / 2 ) {
						top = $( window ).height() / 2;
					}

					display.css( 'marginTop', top * -1 );
				}

				if ( options.background ) {
					Buttons.background( true, options.backgroundClassName, options.fade, hostNode );
				}

				$( 'div.dt-button-background' ).on( 'click.dtb-collection', function () {} );

				$( 'body' )
				.on(
					'click.dtb-collection',
					function (e) {

						var back = $.fn.addBack ? 'addBack' : 'andSelf';

						if ( ! $( e.target ).parents()[back]().filter( content ).length ) {
							close();
						}
					}
				)
				.on(
					'keyup.dtb-collection',
					function (e) {
						if ( e.keyCode === 27 ) {
							close();
						}
					}
				);

				if ( options.autoClose ) {
					setTimeout(
						function () {
							dt.on(
								'buttons-action.b-internal',
								function (e, btn, dt, node) {
									if ( node[0] === hostNode[0] ) {
										 return;
									}
									close();
								}
							);
						},
						0
					);
				}

				$( display ).trigger( 'buttons-popover.dt' );
			}
		}
	);

	Buttons.background = function ( show, className, fade, insertPoint ) {
		if ( fade === undefined ) {
			fade = 400;
		}
		if ( ! insertPoint ) {
			insertPoint = document.body;
		}

		if ( show ) {
			_fadeIn(
				$( '<div/>' )
				.addClass( className )
				.css( 'display', 'none' )
				.insertAfter( insertPoint ),
				fade
			);
		} else {
			_fadeOut(
				$( 'div.' + className ),
				fade,
				function () {
					$( this )
						.removeClass( className )
						.remove();
				}
			);
		}
	};

	Buttons.instanceSelector = function ( group, buttons )
{
		if ( group === undefined || group === null ) {
			return $.map(
				buttons,
				function ( v ) {
					return v.inst;
				}
			);
		}

		var ret   = [];
		var names = $.map(
			buttons,
			function ( v ) {
				return v.name;
			}
		);

		var process = function ( input ) {
			if ( $.isArray( input ) ) {
				for ( var i = 0, ien = input.length; i < ien; i++ ) {
					process( input[i] );
				}
				return;
			}

			if ( typeof input === 'string' ) {
				if ( input.indexOf( ',' ) !== -1 ) {

					process( input.split( ',' ) );
				} else {

					var idx = $.inArray( $.trim( input ), names );

					if ( idx !== -1 ) {
						ret.push( buttons[ idx ].inst );
					}
				}
			} else if ( typeof input === 'number' ) {

				ret.push( buttons[ input ].inst );
			}
		};

		process( group );

		return ret;
	};

	Buttons.buttonSelector = function ( insts, selector )
{
		var ret         = [];
		var nodeBuilder = function ( a, buttons, baseIdx ) {
			var button;
			var idx;

			for ( var i = 0, ien = buttons.length; i < ien; i++ ) {
				button = buttons[i];

				if ( button ) {
					idx = baseIdx !== undefined ?
					baseIdx + i :
					i + '';

					a.push(
						{
							node: button.node,
							name: button.conf.name,
							idx:  idx
						}
					);

					if ( button.buttons ) {
						nodeBuilder( a, button.buttons, idx + '-' );
					}
				}
			}
		};

		var run = function ( selector, inst ) {
			var i, ien;
			var buttons = [];
			nodeBuilder( buttons, inst.s.buttons );

			var nodes = $.map(
				buttons,
				function (v) {
					return v.node;
				}
			);

			if ( $.isArray( selector ) || selector instanceof $ ) {
				for ( i = 0, ien = selector.length; i < ien; i++ ) {
					run( selector[i], inst );
				}
				return;
			}

			if ( selector === null || selector === undefined || selector === '*' ) {

				for ( i = 0, ien = buttons.length; i < ien; i++ ) {
					ret.push(
						{
							inst: inst,
							node: buttons[i].node
						}
					);
				}
			} else if ( typeof selector === 'number' ) {

				ret.push(
					{
						inst: inst,
						node: inst.s.buttons[ selector ].node
					}
				);
			} else if ( typeof selector === 'string' ) {
				if ( selector.indexOf( ',' ) !== -1 ) {

					var a = selector.split( ',' );

					for ( i = 0, ien = a.length; i < ien; i++ ) {
						run( $.trim( a[i] ), inst );
					}
				} else if ( selector.match( /^\d+(\-\d+)*$/ ) ) {

					var indexes = $.map(
						buttons,
						function (v) {
							return v.idx;
						}
					);

					ret.push(
						{
							inst: inst,
							node: buttons[ $.inArray( selector, indexes ) ].node
						}
					);
				} else if ( selector.indexOf( ':name' ) !== -1 ) {

					var name = selector.replace( ':name', '' );

					for ( i = 0, ien = buttons.length; i < ien; i++ ) {
						if ( buttons[i].name === name ) {
							ret.push(
								{
									inst: inst,
									node: buttons[i].node
								}
							);
						}
					}
				} else {

					$( nodes ).filter( selector ).each(
						function () {
							ret.push(
								{
									inst: inst,
									node: this
								}
							);
						}
					);
				}
			} else if ( typeof selector === 'object' && selector.nodeName ) {

				var idx = $.inArray( selector, nodes );

				if ( idx !== -1 ) {
					ret.push(
						{
							inst: inst,
							node: nodes[ idx ]
						}
					);
				}
			}
		};

		for ( var i = 0, ien = insts.length; i < ien; i++ ) {
			var inst = insts[i];

			run( selector, inst );
		}

		return ret;
	};

	Buttons.defaults = {
		buttons: [ 'copy', 'excel', 'csv', 'pdf', 'print' ],
		name: 'main',
		tabIndex: 0,
		dom: {
			container: {
				tag: 'div',
				className: 'dt-buttons'
			},
			collection: {
				tag: 'div',
				className: ''
			},
			button: {

				tag: 'ActiveXObject' in window ?
				'a' :
				'button',
				className: '',
				active: 'active',
				disabled: 'disabled'
			},
			buttonLiner: {
				tag: 'span',
				className: ''
			}
		}
	};

	Buttons.version = '1.6.2';

	$.extend(
		_dtButtons,
		{
			collection: {
				text: function ( dt ) {
					return dt.i18n( 'buttons.collection', 'Collection' );
				},
				className: 'buttons-collection',
				init: function ( dt, button, config ) {
					button.attr( 'aria-expanded', false );
				},
				action: function ( e, dt, button, config ) {
					e.stopPropagation();

					if ( config._collection.parents( 'body' ).length ) {
						this.popover( false, config );
					} else {
						this.popover( config._collection, config );
						jQuery( config._collection ).find( 'button.ColVis_Button.active' ).each(
							function(){
								jQuery( this ).find( ':checkbox' ).attr( 'checked','checked' );
							}
						);
					}
				},
				attr: {
					'aria-haspopup': true
				}

			},
			copy: function ( dt, conf ) {
				if ( _dtButtons.copyHtml5 ) {
					return 'copyHtml5';
				}
				if ( _dtButtons.copyFlash && _dtButtons.copyFlash.available( dt, conf ) ) {
					return 'copyFlash';
				}
			},
			csv: function ( dt, conf ) {

				if ( _dtButtons.csvHtml5 && _dtButtons.csvHtml5.available( dt, conf ) ) {
					return 'csvHtml5';
				}
				if ( _dtButtons.csvFlash && _dtButtons.csvFlash.available( dt, conf ) ) {
					return 'csvFlash';
				}
			},
			excel: function ( dt, conf ) {

				if ( _dtButtons.excelHtml5 && _dtButtons.excelHtml5.available( dt, conf ) ) {
					return 'excelHtml5';
				}
				if ( _dtButtons.excelFlash && _dtButtons.excelFlash.available( dt, conf ) ) {
					return 'excelFlash';
				}
			},
			pdf: function ( dt, conf ) {

				if ( _dtButtons.pdfHtml5 && _dtButtons.pdfHtml5.available( dt, conf ) ) {
					return 'pdfHtml5';
				}
				if ( _dtButtons.pdfFlash && _dtButtons.pdfFlash.available( dt, conf ) ) {
					return 'pdfFlash';
				}
			},
			pageLength: function ( dt ) {
				var lengthMenu = dt.settings()[0].aLengthMenu;
				var vals       = $.isArray( lengthMenu[0] ) ? lengthMenu[0] : lengthMenu;
				var lang       = $.isArray( lengthMenu[0] ) ? lengthMenu[1] : lengthMenu;
				var text       = function ( dt ) {
					return dt.i18n(
						'buttons.pageLength',
						{
							"-1": 'Show all rows',
							_:    'Show %d rows'
						},
						dt.page.len()
					);
				};

				return {
					extend: 'collection',
					text: text,
					className: 'buttons-page-length',
					autoClose: true,
					buttons: $.map(
						vals,
						function ( val, i ) {
							return {
								text: lang[i],
								className: 'button-page-length',
								action: function ( e, dt ) {
									dt.page.len( val ).draw();
								},
								init: function ( dt, node, conf ) {
									var that = this;
									var fn   = function () {
										that.active( dt.page.len() === val );
									};

									dt.on( 'length.dt' + conf.namespace, fn );
									fn();
								},
								destroy: function ( dt, node, conf ) {
									dt.off( 'length.dt' + conf.namespace );
								}
							};
						}
					),
				init: function ( dt, node, conf ) {
					var that = this;
					dt.on(
						'length.dt' + conf.namespace,
						function () {
							that.text( conf.text );
						}
					);
				},
					destroy: function ( dt, node, conf ) {
						dt.off( 'length.dt' + conf.namespace );
					}
				};
			}
		}
	);

	DataTable.Api.register(
		'buttons()',
		function ( group, selector ) {

			if ( selector === undefined ) {
				selector = group;
				group    = undefined;
			}

			this.selector.buttonGroup = group;

			var res = this.iterator(
				true,
				'table',
				function ( ctx ) {
					if ( ctx._buttons ) {
						return Buttons.buttonSelector(
							Buttons.instanceSelector( group, ctx._buttons ),
							selector
						);
					}
				},
				true
			);

			res._groupSelector = group;
			return res;
		}
	);

	DataTable.Api.register(
		'button()',
		function ( group, selector ) {

			var buttons = this.buttons( group, selector );

			if ( buttons.length > 1 ) {
				buttons.splice( 1, buttons.length );
			}

			return buttons;
		}
	);

	DataTable.Api.registerPlural(
		'buttons().active()',
		'button().active()',
		function ( flag ) {
			if ( flag === undefined ) {
				return this.map(
					function ( set ) {
						return set.inst.active( set.node );
					}
				);
			}

			return this.each(
				function ( set ) {
					set.inst.active( set.node, flag );
				}
			);
		}
	);

	DataTable.Api.registerPlural(
		'buttons().action()',
		'button().action()',
		function ( action ) {
			if ( action === undefined ) {
				return this.map(
					function ( set ) {
						return set.inst.action( set.node );
					}
				);
			}

			return this.each(
				function ( set ) {
					set.inst.action( set.node, action );
				}
			);
		}
	);

	DataTable.Api.register(
		['buttons().enable()', 'button().enable()'],
		function ( flag ) {
			return this.each(
				function ( set ) {
					set.inst.enable( set.node, flag );
				}
			);
		}
	);

	DataTable.Api.register(
		['buttons().disable()', 'button().disable()'],
		function () {
			return this.each(
				function ( set ) {
					set.inst.disable( set.node );
				}
			);
		}
	);

	DataTable.Api.registerPlural(
		'buttons().nodes()',
		'button().node()',
		function () {
			var jq = $();

			$(
				this.each(
					function ( set ) {
						jq = jq.add( set.inst.node( set.node ) );
					}
				)
			);

			return jq;
		}
	);

	DataTable.Api.registerPlural(
		'buttons().processing()',
		'button().processing()',
		function ( flag ) {
			if ( flag === undefined ) {
				return this.map(
					function ( set ) {
						return set.inst.processing( set.node );
					}
				);
			}

			return this.each(
				function ( set ) {
					set.inst.processing( set.node, flag );
				}
			);
		}
	);

	DataTable.Api.registerPlural(
		'buttons().text()',
		'button().text()',
		function ( label ) {
			if ( label === undefined ) {
				return this.map(
					function ( set ) {
						return set.inst.text( set.node );
					}
				);
			}

			return this.each(
				function ( set ) {
					set.inst.text( set.node, label );
				}
			);
		}
	);

	DataTable.Api.registerPlural(
		'buttons().trigger()',
		'button().trigger()',
		function () {
			return this.each(
				function ( set ) {
					set.inst.node( set.node ).trigger( 'click' );
				}
			);
		}
	);

	DataTable.Api.register(
		'button().popover()',
		function (content, options) {
			return this.map(
				function ( set ) {
					return set.inst._popover( content, this.button( this[0].node ), options );
				}
			);
		}
	);

	DataTable.Api.register(
		'buttons().containers()',
		function () {
			var jq            = $();
			var groupSelector = this._groupSelector;

			this.iterator(
				true,
				'table',
				function ( ctx ) {
					if ( ctx._buttons ) {
						var insts = Buttons.instanceSelector( groupSelector, ctx._buttons );

						for ( var i = 0, ien = insts.length; i < ien; i++ ) {
							jq = jq.add( insts[i].container() );
						}
					}
				}
			);

			return jq;
		}
	);

	DataTable.Api.register(
		'buttons().container()',
		function () {

			return this.containers().eq( 0 );
		}
	);

	DataTable.Api.register(
		'button().add()',
		function ( idx, conf ) {
			var ctx = this.context;

			if ( ctx.length ) {
				var inst = Buttons.instanceSelector( this._groupSelector, ctx[0]._buttons );

				if ( inst.length ) {
					inst[0].add( conf, idx );
				}
			}

			return this.button( this._groupSelector, idx );
		}
	);

	DataTable.Api.register(
		'buttons().destroy()',
		function () {
			this.pluck( 'inst' ).unique().each(
				function ( inst ) {
					inst.destroy();
				}
			);

			return this;
		}
	);

	DataTable.Api.registerPlural(
		'buttons().remove()',
		'buttons().remove()',
		function () {
			this.each(
				function ( set ) {
					set.inst.remove( set.node );
				}
			);

			return this;
		}
	);

	var _infoTimer;
	DataTable.Api.register(
		'buttons.info()',
		function ( title, message, time ) {
			var that = this;

			if ( title === false ) {
				this.off( 'destroy.btn-info' );
				_fadeOut(
					$( '#datatables_buttons_info' ),
					400,
					function () {
						$( this ).remove();
					}
				);
				clearTimeout( _infoTimer );
				_infoTimer = null;

				return this;
			}

			if ( _infoTimer ) {
				clearTimeout( _infoTimer );
			}

			if ( $( '#datatables_buttons_info' ).length ) {
				$( '#datatables_buttons_info' ).remove();
			}

			title = title ? '<h2>' + title + '</h2>' : '';

			_fadeIn(
				$( '<div id="datatables_buttons_info" class="dt-button-info"/>' )
				.html( title )
				.append( $( '<div/>' )[ typeof message === 'string' ? 'html' : 'append' ]( message ) )
				.css( 'display', 'none' )
				.appendTo( 'body' )
			);

			if ( time !== undefined && time !== 0 ) {
				_infoTimer = setTimeout(
					function () {
						that.buttons.info( false );
					},
					time
				);
			}

			this.on(
				'destroy.btn-info',
				function () {
					that.buttons.info( false );
				}
			);

			return this;
		}
	);

	DataTable.Api.register(
		'buttons.exportData()',
		function ( options ) {
			if ( this.context.length ) {
				return _exportData( new DataTable.Api( this.context[0] ), options );
			}
		}
	);

	DataTable.Api.register(
		'buttons.exportInfo()',
		function ( conf ) {
			if ( ! conf ) {
				conf = {};
			}

			return {
				filename: _filename( conf ),
				title: _title( conf ),
				messageTop: _message( this, conf.message || conf.messageTop, 'top' ),
				messageBottom: _message( this, conf.messageBottom, 'bottom' )
			};
		}
	);

	var _filename = function ( config )
{

		var filename = config.filename === '*' && config.title !== '*' && config.title !== undefined && config.title !== null && config.title !== '' ?
		config.title :
		config.filename;

		if ( typeof filename === 'function' ) {
			filename = filename();
		}

		if ( filename === undefined || filename === null ) {
			return null;
		}

		if ( filename.indexOf( '*' ) !== -1 ) {
			filename = $.trim( filename.replace( '*', $( 'head > title' ).text() ) );
		}

		filename = filename.replace( /[^a-zA-Z0-9_\u00A1-\uFFFF\.,\-_ !\(\)]/g, "" );

		var extension = _stringOrFunction( config.extension );
		if ( ! extension ) {
			extension = '';
		}

		return filename + extension;
	};

	var _stringOrFunction = function ( option )
{
		if ( option === null || option === undefined ) {
			return null;
		} else if ( typeof option === 'function' ) {
			return option();
		}
		return option;
	};

	var _title = function ( config )
{
		var title = _stringOrFunction( config.title );

		return title === null ?
		null : title.indexOf( '*' ) !== -1 ?
			title.replace( '*', $( 'head > title' ).text() || 'Exported data' ) :
			title;
	};

	var _message = function ( dt, option, position )
{
		var message = _stringOrFunction( option );
		if ( message === null ) {
			return null;
		}

		var caption = $( 'caption', dt.table().container() ).eq( 0 );
		if ( message === '*' ) {
			var side = caption.css( 'caption-side' );
			if ( side !== position ) {
				return null;
			}

			return caption.length ?
			caption.text() :
			'';
		}

		return message;
	};

	var _exportTextarea = $( '<textarea/>' )[0];
	var _exportData     = function ( dt, inOpts )
{
		var config = $.extend(
			true,
			{},
			{
				rows:           null,
				columns:        '',
				modifier:       {
					search: 'applied',
					order:  'applied'
				},
				orthogonal:     'display',
				stripHtml:      true,
				stripNewlines:  true,
				decodeEntities: true,
				trim:           true,
				format:         {
					header: function ( d ) {
						return strip( d );
					},
					footer: function ( d ) {
						return strip( d );
					},
					body: function ( d ) {
						return strip( d );
					}
				},
				customizeData: null
			},
			inOpts
		);

		var strip = function ( str ) {
			if ( typeof str !== 'string' ) {
				return str;
			}

			str = str.replace( /<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi, '' );

			str = str.replace( /<!\-\-.*?\-\->/g, '' );

			if ( config.stripHtml ) {
				str = str.replace( /<[^>]*>/g, '' );
			}

			if ( config.trim ) {
				str = str.replace( /^\s+|\s+$/g, '' );
			}

			if ( config.stripNewlines ) {
				str = str.replace( /\n/g, ' ' );
			}

			if ( config.decodeEntities ) {
				_exportTextarea.innerHTML = str;
				str                       = _exportTextarea.value;
			}

			return str;
		};

		var header = dt.columns( config.columns ).indexes().map(
			function (idx) {
				var el = dt.column( idx ).header();
				return config.format.header( el.innerHTML, idx, el );
			}
		).toArray();

		var footer = dt.table().footer() ?
		dt.columns( config.columns ).indexes().map(
			function (idx) {
				var el = dt.column( idx ).footer();
				return config.format.footer( el ? el.innerHTML : '', idx, el );
			}
		).toArray() :
		null;

		var modifier = $.extend( {}, config.modifier );
		if ( dt.select && typeof dt.select.info === 'function' && modifier.selected === undefined ) {
			if ( dt.rows( config.rows, $.extend( { selected: true }, modifier ) ).any() ) {
				$.extend( modifier, { selected: true } )
			}
		}

		var rowIndexes    = dt.rows( config.rows, modifier ).indexes().toArray();
		var selectedCells = dt.cells( rowIndexes, config.columns );
		var cells         = selectedCells
		.render( config.orthogonal )
		.toArray();
		var cellNodes     = selectedCells
		.nodes()
		.toArray();

		var columns     = header.length;
		var rows        = columns > 0 ? cells.length / columns : 0;
		var body        = [];
		var cellCounter = 0;

		for ( var i = 0, ien = rows; i < ien; i++ ) {
			var row = [ columns ];

			for ( var j = 0; j < columns; j++ ) {
				row[j] = config.format.body( cells[ cellCounter ], i, j, cellNodes[ cellCounter ] );
				cellCounter++;
			}

			body[i] = row;
		}

		var data = {
			header: header,
			footer: footer,
			body:   body
		};

		if ( config.customizeData ) {
			config.customizeData( data );
		}

		return data;
	};

	$.fn.dataTable.Buttons = Buttons;
	$.fn.DataTable.Buttons = Buttons;

	$( document ).on(
		'init.dt plugin-init.dt',
		function (e, settings) {
			if ( e.namespace !== 'dt' ) {
				return;
			}

			var opts = settings.oInit.buttons || DataTable.defaults.buttons;

			if ( opts && ! settings._buttons ) {
				new Buttons( settings, opts ).container();
			}
		}
	);

	function _init ( settings, options ) {
		var api  = new DataTable.Api( settings );
		var opts = options
		? options
		: api.init().buttons || DataTable.defaults.buttons;

		return new Buttons( api, opts ).container();
	}

	DataTable.ext.feature.push(
		{
			fnInit: _init,
			cFeature: "B"
		}
	);

	if ( DataTable.ext.features ) {
		DataTable.ext.features.register( 'buttons', _init );
	}

	return Buttons;
}));

(function( factory ){
	if ( typeof define === 'function' && define.amd ) {

		define(
			['jquery', 'datatables.net'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {

		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}

			if ( ! $ || ! $.fn.dataTable ) {
				$ = require( 'datatables.net' )( root, $ ).$;
			}

			return factory( $, root, root.document );
		};
	} else {

		factory( jQuery, window, document );
	}
}(function( $, window, document, undefined ) {
	'use strict';
	var DataTable = $.fn.dataTable;

	function fnInvertKeyValues( aIn )
	{
		var aRet = [];
		for ( var i = 0, iLen = aIn.length; i < iLen; i++ ) {
			aRet[ aIn[i] ] = i;
		}
		return aRet;
	}

	function fnArraySwitch( aArray, iFrom, iTo )
	{
		var mStore = aArray.splice( iFrom, 1 )[0];
		aArray.splice( iTo, 0, mStore );
	}

	function fnDomSwitch( nParent, iFrom, iTo )
	{
		var anTags = [];
		for ( var i = 0, iLen = nParent.childNodes.length; i < iLen; i++ ) {
			if ( nParent.childNodes[i].nodeType == 1 ) {
				anTags.push( nParent.childNodes[i] );
			}
		}
		var nStore = anTags[ iFrom ];

		if ( iTo !== null ) {
			nParent.insertBefore( nStore, anTags[iTo] );
		} else {
			nParent.appendChild( nStore );
		}
	}

	$.fn.dataTableExt.oApi.fnColReorder = function ( oSettings, iFrom, iTo, drop, invalidateRows )
{
		var i, iLen, j, jLen, jen, iCols = oSettings.aoColumns.length, nTrs, oCol;
		var attrMap                      = function ( obj, prop, mapping ) {
			if ( ! obj[ prop ] || typeof obj[ prop ] === 'function' ) {
				return;
			}

			var a   = obj[ prop ].split( '.' );
			var num = a.shift();

			if ( isNaN( num * 1 ) ) {
				return;
			}

			obj[ prop ] = mapping[ num * 1 ] + '.' + a.join( '.' );
		};

		if ( iFrom == iTo ) {

			return;
		}

		if ( iFrom < 0 || iFrom >= iCols ) {
			this.oApi._fnLog( oSettings, 1, "ColReorder 'from' index is out of bounds: " + iFrom );
			return;
		}

		if ( iTo < 0 || iTo >= iCols ) {
			this.oApi._fnLog( oSettings, 1, "ColReorder 'to' index is out of bounds: " + iTo );
			return;
		}

		var aiMapping = [];
		for ( i = 0, iLen = iCols; i < iLen; i++ ) {
			aiMapping[i] = i;
		}
		fnArraySwitch( aiMapping, iFrom, iTo );
		var aiInvertMapping = fnInvertKeyValues( aiMapping );

		for ( i = 0, iLen = oSettings.aaSorting.length; i < iLen; i++ ) {
			oSettings.aaSorting[i][0] = aiInvertMapping[ oSettings.aaSorting[i][0] ];
		}

		if ( oSettings.aaSortingFixed !== null ) {
			for ( i = 0, iLen = oSettings.aaSortingFixed.length; i < iLen; i++ ) {
				oSettings.aaSortingFixed[i][0] = aiInvertMapping[ oSettings.aaSortingFixed[i][0] ];
			}
		}

		for ( i = 0, iLen = iCols; i < iLen; i++ ) {
			oCol = oSettings.aoColumns[i];
			for ( j = 0, jLen = oCol.aDataSort.length; j < jLen; j++ ) {
				oCol.aDataSort[j] = aiInvertMapping[ oCol.aDataSort[j] ];
			}

			oCol.idx = aiInvertMapping[ oCol.idx ];
		}

		$.each(
			oSettings.aLastSort,
			function (i, val) {
				oSettings.aLastSort[i].src = aiInvertMapping[ val.src ];
			}
		);

		for ( i = 0, iLen = iCols; i < iLen; i++ ) {
			oCol = oSettings.aoColumns[i];

			if ( typeof oCol.mData == 'number' ) {
				oCol.mData = aiInvertMapping[ oCol.mData ];
			} else if ( $.isPlainObject( oCol.mData ) ) {

				attrMap( oCol.mData, '_',      aiInvertMapping );
				attrMap( oCol.mData, 'filter', aiInvertMapping );
				attrMap( oCol.mData, 'sort',   aiInvertMapping );
				attrMap( oCol.mData, 'type',   aiInvertMapping );
			}
		}

		if ( oSettings.aoColumns[iFrom].bVisible ) {

			var iVisibleIndex      = this.oApi._fnColumnIndexToVisible( oSettings, iFrom );
			var iInsertBeforeIndex = null;

			i = iTo < iFrom ? iTo : iTo + 1;
			while ( iInsertBeforeIndex === null && i < iCols ) {
				iInsertBeforeIndex = this.oApi._fnColumnIndexToVisible( oSettings, i );
				i++;
			}

			nTrs = oSettings.nTHead.getElementsByTagName( 'tr' );
			for ( i = 0, iLen = nTrs.length; i < iLen; i++ ) {
				fnDomSwitch( nTrs[i], iVisibleIndex, iInsertBeforeIndex );
			}

			if ( oSettings.nTFoot !== null ) {
				nTrs = oSettings.nTFoot.getElementsByTagName( 'tr' );
				for ( i = 0, iLen = nTrs.length; i < iLen; i++ ) {
					fnDomSwitch( nTrs[i], iVisibleIndex, iInsertBeforeIndex );
				}
			}

			for ( i = 0, iLen = oSettings.aoData.length; i < iLen; i++ ) {
				if ( oSettings.aoData[i].nTr !== null ) {
					fnDomSwitch( oSettings.aoData[i].nTr, iVisibleIndex, iInsertBeforeIndex );
				}
			}
		}

		fnArraySwitch( oSettings.aoColumns, iFrom, iTo );

		for ( i = 0, iLen = iCols; i < iLen; i++ ) {
			oSettings.oApi._fnColumnOptions( oSettings, i, {} );
		}

		fnArraySwitch( oSettings.aoPreSearchCols, iFrom, iTo );

		for ( i = 0, iLen = oSettings.aoData.length; i < iLen; i++ ) {
			var data  = oSettings.aoData[i];
			var cells = data.anCells;

			if ( cells ) {
				fnArraySwitch( cells, iFrom, iTo );

				for ( j = 0, jen = cells.length; j < jen; j++ ) {
					if ( cells[j] && cells[j]._DT_CellIndex ) {
						cells[j]._DT_CellIndex.column = j;
					}
				}
			}

			if ( data.src !== 'dom' && $.isArray( data._aData ) ) {
				fnArraySwitch( data._aData, iFrom, iTo );
			}
		}

		for ( i = 0, iLen = oSettings.aoHeader.length; i < iLen; i++ ) {
			fnArraySwitch( oSettings.aoHeader[i], iFrom, iTo );
		}

		if ( oSettings.aoFooter !== null ) {
			for ( i = 0, iLen = oSettings.aoFooter.length; i < iLen; i++ ) {
				fnArraySwitch( oSettings.aoFooter[i], iFrom, iTo );
			}
		}

		if ( invalidateRows || invalidateRows === undefined ) {
			$.fn.dataTable.Api( oSettings ).rows().invalidate();
		}

		for ( i = 0, iLen = iCols; i < iLen; i++ ) {
			$( oSettings.aoColumns[i].nTh ).off( '.DT' );
			this.oApi._fnSortAttachListener( oSettings, oSettings.aoColumns[i].nTh, i );
		}

		$( oSettings.oInstance ).trigger(
			'column-reorder.dt',
			[ oSettings, {
				from: iFrom,
				to: iTo,
				mapping: aiInvertMapping,
				drop: drop,

				iFrom: iFrom,
				iTo: iTo,
				aiInvertMapping: aiInvertMapping
			} ]
		);
	};

	var ColReorder = function( dt, opts )
{
		var settings = new $.fn.dataTable.Api( dt ).settings()[0];

		if ( settings._colReorder ) {
			return settings._colReorder;
		}

		if ( opts === true ) {
			opts = {};
		}

		var camelToHungarian = $.fn.dataTable.camelToHungarian;
		if ( camelToHungarian ) {
			camelToHungarian( ColReorder.defaults, ColReorder.defaults, true );
			camelToHungarian( ColReorder.defaults, opts || {} );
		}

		this.s = {

			"dt": null,

			"enable": null,

			"init": $.extend( true, {}, ColReorder.defaults, opts ),

			"fixed": 0,

			"fixedRight": 0,

			"reorderCallback": null,

			"mouse": {
				"startX": -1,
				"startY": -1,
				"offsetX": -1,
				"offsetY": -1,
				"target": -1,
				"targetIndex": -1,
				"fromIndex": -1
			},

			"aoTargets": []
		};

		this.dom = {

			"drag": null,

			"pointer": null
		};

		this.s.enable         = this.s.init.bEnable;
		this.s.dt             = settings;
		this.s.dt._colReorder = this;
		this._fnConstruct();

		return this;
	};

	$.extend(
		ColReorder.prototype,
		{

			fnEnable: function ( flag )
		{
				if ( flag === false ) {
					return fnDisable();
				}

				this.s.enable = true;
			},

			fnDisable: function ()
		{
				this.s.enable = false;
			},

			"fnReset": function ()
		{
				this._fnOrderColumns( this.fnOrder() );

				return this;
			},

			"fnGetCurrentOrder": function ()
		{
				return this.fnOrder();
			},

			"fnOrder": function ( set, original )
		{
				var a       = [], i, ien, j, jen;
				var columns = this.s.dt.aoColumns;

				if ( set === undefined ) {
					for ( i = 0, ien = columns.length; i < ien; i++ ) {
						a.push( columns[i]._ColReorder_iOrigCol );
					}

					return a;
				}

				if ( original ) {
					var order = this.fnOrder();

					for ( i = 0, ien = set.length; i < ien; i++ ) {
						a.push( $.inArray( set[i], order ) );
					}

					set = a;
				}

				this._fnOrderColumns( fnInvertKeyValues( set ) );

				return this;
			},

			fnTranspose: function ( idx, dir )
		{
				if ( ! dir ) {
					dir = 'toCurrent';
				}

				var order   = this.fnOrder();
				var columns = this.s.dt.aoColumns;

				if ( dir === 'toCurrent' ) {

					return ! $.isArray( idx ) ?
					$.inArray( idx, order ) :
					$.map(
						idx,
						function ( index ) {
							return $.inArray( index, order );
						}
					);
				} else {

					return ! $.isArray( idx ) ?
					columns[idx]._ColReorder_iOrigCol :
					$.map(
						idx,
						function ( index ) {
							return columns[index]._ColReorder_iOrigCol;
						}
					);
				}
			},

			"_fnConstruct": function ()
		{
				var that  = this;
				var iLen  = this.s.dt.aoColumns.length;
				var table = this.s.dt.nTable;
				var i;

				if ( this.s.init.iFixedColumns ) {
					this.s.fixed = this.s.init.iFixedColumns;
				}

				if ( this.s.init.iFixedColumnsLeft ) {
					this.s.fixed = this.s.init.iFixedColumnsLeft;
				}

				this.s.fixedRight = this.s.init.iFixedColumnsRight ?
				this.s.init.iFixedColumnsRight :
				0;

				if ( this.s.init.fnReorderCallback ) {
					this.s.reorderCallback = this.s.init.fnReorderCallback;
				}

				for ( i = 0; i < iLen; i++ ) {
					if ( i > this.s.fixed - 1 && i < iLen - this.s.fixedRight ) {
						this._fnMouseListener( i, this.s.dt.aoColumns[i].nTh );
					}

					this.s.dt.aoColumns[i]._ColReorder_iOrigCol = i;
				}

				this.s.dt.oApi._fnCallbackReg(
					this.s.dt,
					'aoStateSaveParams',
					function (oS, oData) {
						that._fnStateSave.call( that, oData );
					},
					"ColReorder_State"
				);

				var aiOrder = null;
				if ( this.s.init.aiOrder ) {
					aiOrder = this.s.init.aiOrder.slice();
				}

				if ( this.s.dt.oLoadedState && typeof this.s.dt.oLoadedState.ColReorder != 'undefined' &&
				  this.s.dt.oLoadedState.ColReorder.length == this.s.dt.aoColumns.length ) {
					aiOrder = this.s.dt.oLoadedState.ColReorder;
				}

				if ( aiOrder ) {

					if ( ! that.s.dt._bInitComplete ) {
						var bDone = false;
						$( table ).on(
							'draw.dt.colReorder',
							function () {
								if ( ! that.s.dt._bInitComplete && ! bDone ) {
									bDone      = true;
									var resort = fnInvertKeyValues( aiOrder );
									that._fnOrderColumns.call( that, resort );
								}
							}
						);
					} else {
						var resort = fnInvertKeyValues( aiOrder );
						that._fnOrderColumns.call( that, resort );
					}
				} else {
					this._fnSetColumnIndexes();
				}

				$( table ).on(
					'destroy.dt.colReorder',
					function () {
						$( table ).off( 'destroy.dt.colReorder draw.dt.colReorder' );

						$.each(
							that.s.dt.aoColumns,
							function (i, column) {
								$( column.nTh ).off( '.ColReorder' );
								$( column.nTh ).removeAttr( 'data-column-index' );
							}
						);

						that.s.dt._colReorder = null;
						that.s                = null;
					}
				);
			},

			"_fnOrderColumns": function ( a )
		{
				var changed = false;

				if ( a.length != this.s.dt.aoColumns.length ) {
					this.s.dt.oInstance.oApi._fnLog(
						this.s.dt,
						1,
						"ColReorder - array reorder does not " +
						"match known number of columns. Skipping."
					);
					return;
				}

				for ( var i = 0, iLen = a.length; i < iLen; i++ ) {
					var currIndex = $.inArray( i, a );
					if ( i != currIndex ) {

						fnArraySwitch( a, currIndex, i );

						this.s.dt.oInstance.fnColReorder( currIndex, i, true, false );

						changed = true;
					}
				}

				this._fnSetColumnIndexes();

				if ( ! changed ) {
					return;
				}

				$.fn.dataTable.Api( this.s.dt ).rows().invalidate();

				if ( this.s.dt.oScroll.sX !== "" || this.s.dt.oScroll.sY !== "" ) {
					this.s.dt.oInstance.fnAdjustColumnSizing( false );
				}

				this.s.dt.oInstance.oApi._fnSaveState( this.s.dt );

				if ( this.s.reorderCallback !== null ) {
					this.s.reorderCallback.call( this );
				}
			},

			"_fnStateSave": function ( oState )
		{
				var i, iLen, aCopy, iOrigColumn;
				var oSettings = this.s.dt;
				var columns   = oSettings.aoColumns;

				oState.ColReorder = [];

				if ( oState.aaSorting ) {

					for ( i = 0; i < oState.aaSorting.length; i++ ) {
						oState.aaSorting[i][0] = columns[ oState.aaSorting[i][0] ]._ColReorder_iOrigCol;
					}

					var aSearchCopy = $.extend( true, [], oState.aoSearchCols );

					for ( i = 0, iLen = columns.length; i < iLen; i++ ) {
						iOrigColumn = columns[i]._ColReorder_iOrigCol;

						oState.aoSearchCols[ iOrigColumn ] = aSearchCopy[i];

						oState.abVisCols[ iOrigColumn ] = columns[i].bVisible;

						oState.ColReorder.push( iOrigColumn );
					}
				} else if ( oState.order ) {

					for ( i = 0; i < oState.order.length; i++ ) {
						oState.order[i][0] = columns[ oState.order[i][0] ]._ColReorder_iOrigCol;
					}

					var stateColumnsCopy = $.extend( true, [], oState.columns );

					for ( i = 0, iLen = columns.length; i < iLen; i++ ) {
						iOrigColumn = columns[i]._ColReorder_iOrigCol;

						oState.columns[ iOrigColumn ] = stateColumnsCopy[i];

						oState.ColReorder.push( iOrigColumn );
					}
				}
			},

			"_fnMouseListener": function ( i, nTh )
		{
				var that = this;
				$( nTh )
				.on(
					'mousedown.ColReorder',
					function (e) {
						if ( that.s.enable && e.which === 1 ) {
							that._fnMouseDown.call( that, e, nTh );
						}
					}
				)
				.on(
					'touchstart.ColReorder',
					function (e) {
						if ( that.s.enable ) {
							that._fnMouseDown.call( that, e, nTh );
						}
					}
				);
			},

			"_fnMouseDown": function ( e, nTh )
		{
				var that = this;

				var target = $( e.target ).closest( 'th, td' );
				var offset = target.offset();
				var idx    = parseInt( $( nTh ).attr( 'data-column-index' ), 10 );

				if ( idx === undefined ) {
					return;
				}

				this.s.mouse.startX      = this._fnCursorPosition( e, 'pageX' );
				this.s.mouse.startY      = this._fnCursorPosition( e, 'pageY' );
				this.s.mouse.offsetX     = this._fnCursorPosition( e, 'pageX' ) - offset.left;
				this.s.mouse.offsetY     = this._fnCursorPosition( e, 'pageY' ) - offset.top;
				this.s.mouse.target      = this.s.dt.aoColumns[ idx ].nTh;
				this.s.mouse.targetIndex = idx;
				this.s.mouse.fromIndex   = idx;

				this._fnRegions();

				$( document )
				.on(
					'mousemove.ColReorder touchmove.ColReorder',
					function (e) {
						that._fnMouseMove.call( that, e );
					}
				)
				.on(
					'mouseup.ColReorder touchend.ColReorder',
					function (e) {
						that._fnMouseUp.call( that, e );
					}
				);
			},

			"_fnMouseMove": function ( e )
		{
				var that = this;

				if ( this.dom.drag === null ) {

					if ( Math.pow(
						Math.pow( this._fnCursorPosition( e, 'pageX' ) - this.s.mouse.startX, 2 ) +
						Math.pow( this._fnCursorPosition( e, 'pageY' ) - this.s.mouse.startY, 2 ),
						0.5
					) < 5 ) {
						return;
					}
					this._fnCreateDragNode();
				}

				this.dom.drag.css(
					{
						left: this._fnCursorPosition( e, 'pageX' ) - this.s.mouse.offsetX,
						top: this._fnCursorPosition( e, 'pageY' ) - this.s.mouse.offsetY
						}
				);

				var target;
				var lastToIndex      = this.s.mouse.toIndex;
				var cursorXPosiotion = this._fnCursorPosition( e, 'pageX' );
				var targetsPrev      = function (i) {
					while (i >= 0) {
						i--;

						if (i <= 0) {
							return null;
						}

						if (that.s.aoTargets[i + 1].x !== that.s.aoTargets[i].x) {
							return that.s.aoTargets[i];
						}
					}
				};
				var firstNotHidden   = function () {
					for (var i = 0; i < that.s.aoTargets.length - 1; i++) {
						if (that.s.aoTargets[i].x !== that.s.aoTargets[i + 1].x) {
							return that.s.aoTargets[i];
						}
					}
				};
				var lastNotHidden    = function () {
					for (var i = that.s.aoTargets.length - 1; i > 0; i--) {
						if (that.s.aoTargets[i].x !== that.s.aoTargets[i - 1].x) {
							return that.s.aoTargets[i];
						}
					}
				};

				for (var i = 1; i < this.s.aoTargets.length; i++) {
					var prevTarget = targetsPrev( i );
					if ( ! prevTarget) {
						prevTarget = firstNotHidden();
					}

					var prevTargetMiddle = prevTarget.x + (this.s.aoTargets[i].x - prevTarget.x) / 2;

					if (this._fnIsLtr()) {
						if (cursorXPosiotion < prevTargetMiddle ) {
							target = prevTarget;
							break;
						}
					} else {
						if (cursorXPosiotion > prevTargetMiddle) {
							target = prevTarget;
							break;
						}
					}
				}

				if (target) {
					this.dom.pointer.css( 'left', target.x );
					this.s.mouse.toIndex = target.to;
				} else {

					this.dom.pointer.css( 'left', lastNotHidden().x );
					this.s.mouse.toIndex = lastNotHidden().to;
				}

				if ( this.s.init.bRealtime && lastToIndex !== this.s.mouse.toIndex ) {
					this.s.dt.oInstance.fnColReorder( this.s.mouse.fromIndex, this.s.mouse.toIndex );
					this.s.mouse.fromIndex = this.s.mouse.toIndex;

					if ( this.s.dt.oScroll.sX !== "" || this.s.dt.oScroll.sY !== "" ) {
						this.s.dt.oInstance.fnAdjustColumnSizing( false );
					}

					this._fnRegions();
				}
			},

			"_fnMouseUp": function ( e )
		{
				var that = this;

				$( document ).off( '.ColReorder' );

				if ( this.dom.drag !== null ) {

					this.dom.drag.remove();
					this.dom.pointer.remove();
					this.dom.drag    = null;
					this.dom.pointer = null;

					this.s.dt.oInstance.fnColReorder( this.s.mouse.fromIndex, this.s.mouse.toIndex, true );
					this._fnSetColumnIndexes();

					if ( this.s.dt.oScroll.sX !== "" || this.s.dt.oScroll.sY !== "" ) {
						this.s.dt.oInstance.fnAdjustColumnSizing( false );
					}

					this.s.dt.oInstance.oApi._fnSaveState( this.s.dt );

					if ( this.s.reorderCallback !== null ) {
						this.s.reorderCallback.call( this );
					}
				}
			},

			"_fnRegions": function ()
		{
				var aoColumns = this.s.dt.aoColumns;
				var isLTR     = this._fnIsLtr();
				this.s.aoTargets.splice( 0, this.s.aoTargets.length );
				var lastBound = $( this.s.dt.nTable ).offset().left;

				var aoColumnBounds = [];
				$.each(
					aoColumns,
					function (i, column) {
						if (column.bVisible && column.nTh.style.display !== 'none') {
							var nth   = $( column.nTh );
							var bound = nth.offset().left;

							if (isLTR) {
								bound += nth.outerWidth();
							}

							aoColumnBounds.push(
								{
									index: i,
									bound: bound
								}
							);

							lastBound = bound;
						} else {
							aoColumnBounds.push(
								{
									index: i,
									bound: lastBound
								}
							);
						}
					}
				);

				var firstColumn      = aoColumnBounds[0];
				var firstColumnWidth = $( aoColumns[firstColumn.index].nTh ).outerWidth();

				this.s.aoTargets.push(
					{
						to: 0,
						x: firstColumn.bound - firstColumnWidth
					}
				);

				for (var i = 0; i < aoColumnBounds.length; i++) {
					var columnBound = aoColumnBounds[i];
					var iToPoint    = columnBound.index;

					if (columnBound.index < this.s.mouse.fromIndex) {
						iToPoint++;
					}

					this.s.aoTargets.push(
						{
							to: iToPoint,
							x: columnBound.bound
						}
					);
				}

				if ( this.s.fixedRight !== 0 ) {
					this.s.aoTargets.splice( this.s.aoTargets.length - this.s.fixedRight );
				}

				if ( this.s.fixed !== 0 ) {
					this.s.aoTargets.splice( 0, this.s.fixed );
				}
			},

			"_fnCreateDragNode": function ()
		{
				var scrolling = this.s.dt.oScroll.sX !== "" || this.s.dt.oScroll.sY !== "";

				var origCell  = this.s.dt.aoColumns[ this.s.mouse.targetIndex ].nTh;
				var origTr    = origCell.parentNode;
				var origThead = origTr.parentNode;
				var origTable = origThead.parentNode;
				var cloneCell = $( origCell ).clone();

				this.dom.drag = $( origTable.cloneNode( false ) )
				.addClass( 'DTCR_clonedTable' )
				.append(
					$( origThead.cloneNode( false ) ).append(
						$( origTr.cloneNode( false ) ).append(
							cloneCell[0]
						)
					)
				)
				.css(
					{
						position: 'absolute',
						top: 0,
						left: 0,
						width: $( origCell ).outerWidth(),
						height: $( origCell ).outerHeight()
						}
				)
				.appendTo( 'body' );

				this.dom.pointer = $( '<div></div>' )
				.addClass( 'DTCR_pointer' )
				.css(
					{
						position: 'absolute',
						top: scrolling ?
							$( 'div.dataTables_scroll', this.s.dt.nTableWrapper ).offset().top :
							$( this.s.dt.nTable ).offset().top,
						height : scrolling ?
							$( 'div.dataTables_scroll', this.s.dt.nTableWrapper ).height() :
							$( this.s.dt.nTable ).height()
						}
				)
				.appendTo( 'body' );
			},

			"_fnSetColumnIndexes": function ()
		{
				$.each(
					this.s.dt.aoColumns,
					function (i, column) {
						$( column.nTh ).attr( 'data-column-index', i );
					}
				);
			},

			_fnCursorPosition: function ( e, prop ) {
				if ( e.type.indexOf( 'touch' ) !== -1 ) {
					return e.originalEvent.touches[0][ prop ];
				}
				return e[ prop ];
			},

			_fnIsLtr: function () {
				return $( this.s.dt.nTable ).css( 'direction' ) !== "rtl";
			}
		}
	);

	ColReorder.defaults = {

		aiOrder: null,

		bEnable: true,

		bRealtime: true,

		iFixedColumnsLeft: 0,

		iFixedColumnsRight: 0,

		fnReorderCallback: null
	};

	ColReorder.version = "1.5.2";

	$.fn.dataTable.ColReorder = ColReorder;
	$.fn.DataTable.ColReorder = ColReorder;

	if ( typeof $.fn.dataTable == "function" &&
	 typeof $.fn.dataTableExt.fnVersionCheck == "function" &&
	 $.fn.dataTableExt.fnVersionCheck( '1.10.8' ) ) {
		$.fn.dataTableExt.aoFeatures.push(
			{
				"fnInit": function( settings ) {
					var table = settings.oInstance;

					if ( ! settings._colReorder ) {
						var dtInit = settings.oInit;
						var opts   = dtInit.colReorder || dtInit.oColReorder || {};

						new ColReorder( settings, opts );
					} else {
						table.oApi._fnLog( settings, 1, "ColReorder attempted to initialise twice. Ignoring second" );
					}

					return null;
				},
				"cFeature": "R",
				"sFeature": "ColReorder"
			}
		);
	} else {
		alert( "Warning: ColReorder requires DataTables 1.10.8 or greater - www.datatables.net/download" );
	}

	$( document ).on(
		'preInit.dt.colReorder',
		function (e, settings) {
			if ( e.namespace !== 'dt' ) {
				return;
			}

			var init     = settings.oInit.colReorder;
			var defaults = DataTable.defaults.colReorder;

			if ( init || defaults ) {
				var opts = $.extend( {}, init, defaults );

				if ( init !== false ) {
					new ColReorder( settings, opts );
				}
			}
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.reset()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					ctx._colReorder.fnReset();
				}
			);
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.order()',
		function ( set, original ) {
			if ( set ) {
				return this.iterator(
					'table',
					function ( ctx ) {
						ctx._colReorder.fnOrder( set, original );
					}
				);
			}

			return this.context.length ?
			this.context[0]._colReorder.fnOrder() :
			null;
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.transpose()',
		function ( idx, dir ) {
			return this.context.length && this.context[0]._colReorder ?
			this.context[0]._colReorder.fnTranspose( idx, dir ) :
			idx;
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.move()',
		function( from, to, drop, invalidateRows ) {
			if (this.context.length) {
				this.context[0]._colReorder.s.dt.oInstance.fnColReorder( from, to, drop, invalidateRows );
				this.context[0]._colReorder._fnSetColumnIndexes();
			}
			return this;
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.enable()',
		function( flag ) {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._colReorder ) {
						ctx._colReorder.fnEnable( flag );
					}
				}
			);
		}
	);

	$.fn.dataTable.Api.register(
		'colReorder.disable()',
		function() {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._colReorder ) {
						ctx._colReorder.fnDisable();
					}
				}
			);
		}
	);

	return ColReorder;
}));

(function( factory ){
	if ( typeof define === 'function' && define.amd ) {

		define(
			['jquery', 'datatables.net'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {

		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}

			if ( ! $ || ! $.fn.dataTable ) {
				$ = require( 'datatables.net' )( root, $ ).$;
			}

			return factory( $, root, root.document );
		};
	} else {

		factory( jQuery, window, document );
	}
}(function( $, window, document, undefined ) {
	'use strict';
	var DataTable = $.fn.dataTable;
	var _firefoxScroll;

	var FixedColumns = function ( dt, init ) {
		var that = this;

		if ( ! ( this instanceof FixedColumns ) ) {
			alert( "FixedColumns warning: FixedColumns must be initialised with the 'new' keyword." );
			return;
		}

		if ( init === undefined || init === true ) {
			init = {};
		}

		var camelToHungarian = $.fn.dataTable.camelToHungarian;
		if ( camelToHungarian ) {
			camelToHungarian( FixedColumns.defaults, FixedColumns.defaults, true );
			camelToHungarian( FixedColumns.defaults, init );
		}

		var dtSettings = new $.fn.dataTable.Api( dt ).settings()[0];

		this.s = {

			"dt": dtSettings,

			"iTableColumns": dtSettings.aoColumns.length,

			"aiOuterWidths": [],

			"aiInnerWidths": [],

			rtl: $( dtSettings.nTable ).css( 'direction' ) === 'rtl'
		};

		this.dom = {

			"scroller": null,

			"header": null,

			"body": null,

			"footer": null,

			"grid": {

				"wrapper": null,

				"dt": null,

				"left": {
					"wrapper": null,
					"head": null,
					"body": null,
					"foot": null
				},

				"right": {
					"wrapper": null,
					"head": null,
					"body": null,
					"foot": null
				}
			},

			"clone": {

				"left": {

					"header": null,

					"body": null,

					"footer": null
				},

				"right": {

					"header": null,

					"body": null,

					"footer": null
				}
			}
		};

		if ( dtSettings._oFixedColumns ) {
			throw 'FixedColumns already initialised on this table';
		}

		dtSettings._oFixedColumns = this;

		if ( ! dtSettings._bInitComplete ) {
			dtSettings.oApi._fnCallbackReg(
				dtSettings,
				'aoInitComplete',
				function () {
					that._fnConstruct( init );
				},
				'FixedColumns'
			);
		} else {
			this._fnConstruct( init );
		}
	};

	$.extend(
		FixedColumns.prototype ,
		{

			"fnUpdate": function ()
		{
				this._fnDraw( true );
			},

			"fnRedrawLayout": function ()
		{
				this._fnColCalc();
				this._fnGridLayout();
				this.fnUpdate();
			},

			"fnRecalculateHeight": function ( nTr )
		{
				delete nTr._DTTC_iHeight;
				nTr.style.height = 'auto';
			},

			"fnSetRowHeight": function ( nTarget, iHeight )
		{
				nTarget.style.height = iHeight + "px";
			},

			"fnGetPosition": function ( node )
		{
				var idx;
				var inst = this.s.dt.oInstance;

				if ( ! $( node ).parents( '.DTFC_Cloned' ).length ) {

					return inst.fnGetPosition( node );
				} else {

					if ( node.nodeName.toLowerCase() === 'tr' ) {
						idx = $( node ).index();
						return inst.fnGetPosition( $( 'tr', this.s.dt.nTBody )[ idx ] );
					} else {
						var colIdx = $( node ).index();
						idx        = $( node.parentNode ).index();
						var row    = inst.fnGetPosition( $( 'tr', this.s.dt.nTBody )[ idx ] );

						return [
						row,
						colIdx,
						inst.oApi._fnVisibleToColumnIndex( this.s.dt, colIdx )
						];
					}
				}
			},

			fnToFixedNode: function ( rowIdx, colIdx )
		{
				var found;

				if ( colIdx < this.s.iLeftColumns ) {
					found = $( this.dom.clone.left.body ).find( '[data-dt-row=' + rowIdx + '][data-dt-column=' + colIdx + ']' );
				} else if ( colIdx >= this.s.iRightColumns ) {
					found = $( this.dom.clone.right.body ).find( '[data-dt-row=' + rowIdx + '][data-dt-column=' + colIdx + ']' );
				}

				if ( found && found.length ) {
					return found[0];
				}

				var table = new $.fn.dataTable.Api( this.s.dt );
				return table.cell( rowIdx, colIdx ).node();
			},

			"_fnConstruct": function ( oInit )
		{
				var i, iLen, iWidth,
				that = this;

				if ( typeof this.s.dt.oInstance.fnVersionCheck != 'function' ||
				 this.s.dt.oInstance.fnVersionCheck( '1.8.0' ) !== true ) {
					alert(
						"FixedColumns " + FixedColumns.VERSION + " required DataTables 1.8.0 or later. " +
						"Please upgrade your DataTables installation"
					);
					return;
				}

				if ( this.s.dt.oScroll.sX === "" ) {
					this.s.dt.oInstance.oApi._fnLog(
						this.s.dt,
						1,
						"FixedColumns is not needed (no " +
						"x-scrolling in DataTables enabled), so no action will be taken. Use 'FixedHeader' for " +
						"column fixing when scrolling is not enabled"
					);
					return;
				}

				this.s = $.extend( true, this.s, FixedColumns.defaults, oInit );

				var classes       = this.s.dt.oClasses;
				this.dom.grid.dt  = $( this.s.dt.nTable ).parents( 'div.' + classes.sScrollWrapper )[0];
				this.dom.scroller = $( 'div.' + classes.sScrollBody, this.dom.grid.dt )[0];

				this._fnColCalc();
				this._fnGridSetup();

				var mouseController;
				var mouseDown = false;

				$( this.s.dt.nTableWrapper ).on(
					'mousedown.DTFC',
					function (e) {
						if ( e.button === 0 ) {
							mouseDown = true;

							$( document ).one(
								'mouseup',
								function () {
									mouseDown = false;
								}
							);
						}
					}
				);

				$( this.dom.scroller )
				.on(
					'mouseover.DTFC touchstart.DTFC',
					function () {
						if ( ! mouseDown ) {
							mouseController = 'main';
						}
					}
				)
				.on(
					'scroll.DTFC',
					function (e) {
						if ( ! mouseController && e.originalEvent ) {
							mouseController = 'main';
						}

						if ( mouseController === 'main' || mouseController === 'key' ) {
							if ( that.s.iLeftColumns > 0 ) {
								that.dom.grid.left.liner.scrollTop = that.dom.scroller.scrollTop;
							}
							if ( that.s.iRightColumns > 0 ) {
								that.dom.grid.right.liner.scrollTop = that.dom.scroller.scrollTop;
							}
						}
					}
				);

				var wheelType = 'onwheel' in document.createElement( 'div' ) ?
				'wheel.DTFC' :
				'mousewheel.DTFC';

				if ( that.s.iLeftColumns > 0 ) {

					$( that.dom.grid.left.liner )
					.on(
						'mouseover.DTFC touchstart.DTFC',
						function () {
							if ( ! mouseDown && mouseController !== 'key' ) {
								mouseController = 'left';
							}
						}
					)
						.on(
							'scroll.DTFC',
							function ( e ) {
								if ( ! mouseController && e.originalEvent ) {
									mouseController = 'left';
								}

								if ( mouseController === 'left' ) {
									that.dom.scroller.scrollTop = that.dom.grid.left.liner.scrollTop;
									if ( that.s.iRightColumns > 0 ) {
										that.dom.grid.right.liner.scrollTop = that.dom.grid.left.liner.scrollTop;
									}
								}
							}
						)
						.on(
							wheelType,
							function(e) {
								mouseController = 'left';

								var xDelta                    = e.type === 'wheel' ?
								-e.originalEvent.deltaX :
								e.originalEvent.wheelDeltaX;
								that.dom.scroller.scrollLeft -= xDelta;
							}
						);
				}

				if ( that.s.iRightColumns > 0 ) {

					$( that.dom.grid.right.liner )
					.on(
						'mouseover.DTFC touchstart.DTFC',
						function () {
							if ( ! mouseDown && mouseController !== 'key' ) {
								mouseController = 'right';
							}
						}
					)
						.on(
							'scroll.DTFC',
							function ( e ) {
								if ( ! mouseController && e.originalEvent ) {
									mouseController = 'right';
								}

								if ( mouseController === 'right' ) {
									that.dom.scroller.scrollTop = that.dom.grid.right.liner.scrollTop;
									if ( that.s.iLeftColumns > 0 ) {
										that.dom.grid.left.liner.scrollTop = that.dom.grid.right.liner.scrollTop;
									}
								}
							}
						)
						.on(
							wheelType,
							function(e) {
								mouseController = 'left';

								var xDelta                    = e.type === 'wheel' ?
								-e.originalEvent.deltaX :
								e.originalEvent.wheelDeltaX;
								that.dom.scroller.scrollLeft -= xDelta;
							}
						);
				}

				$( window ).on(
					'resize.DTFC',
					function () {
						that._fnGridLayout.call( that );
					}
				);

				var bFirstDraw = true;
				var jqTable    = $( this.s.dt.nTable );

				jqTable
				.on(
					'draw.dt.DTFC',
					function () {
						that._fnColCalc();
						that._fnDraw.call( that, bFirstDraw );
						bFirstDraw = false;
					}
				)
				.on(
					'key-focus.dt.DTFC',
					function () {

						mouseController = 'key';
					}
				)
				.on(
					'column-sizing.dt.DTFC',
					function () {
						that._fnColCalc();
						that._fnGridLayout( that );
					}
				)
				.on(
					'column-visibility.dt.DTFC',
					function ( e, settings, column, vis, recalc ) {
						if ( recalc === undefined || recalc ) {
							that._fnColCalc();
							that._fnGridLayout( that );
							that._fnDraw( true );
						}
					}
				)
				.on(
					'select.dt.DTFC deselect.dt.DTFC',
					function ( e, dt, type, indexes ) {
						if ( e.namespace === 'dt' ) {
							that._fnDraw( false );
						}
					}
				)
				.on(
					'position.dts.dt.DTFC',
					function (e, tableTop) {

						if (that.dom.grid.left.body) {
							$( that.dom.grid.left.body ).find( 'table' ).eq( 0 ).css( 'top', tableTop );
						}

						if (that.dom.grid.right.body) {
							$( that.dom.grid.right.body ).find( 'table' ).eq( 0 ).css( 'top', tableTop );
						}
					}
				)
				.on(
					'destroy.dt.DTFC',
					function () {
						jqTable.off( '.DTFC' );

						$( that.dom.scroller ).off( '.DTFC' );
						$( window ).off( '.DTFC' );
						$( that.s.dt.nTableWrapper ).off( '.DTFC' );

						$( that.dom.grid.left.liner ).off( '.DTFC ' + wheelType );
						$( that.dom.grid.left.wrapper ).remove();

						$( that.dom.grid.right.liner ).off( '.DTFC ' + wheelType );
						$( that.dom.grid.right.wrapper ).remove();
					}
				);

				this._fnGridLayout();
				this.s.dt.oInstance.fnDraw( false );
			},

			"_fnColCalc": function ()
		{
				var that        = this;
				var iLeftWidth  = 0;
				var iRightWidth = 0;

				this.s.aiInnerWidths = [];
				this.s.aiOuterWidths = [];

				$.each(
					this.s.dt.aoColumns,
					function (i, col) {
						var th = $( col.nTh );
						var border;

						if ( ! th.filter( ':visible' ).length ) {
							that.s.aiInnerWidths.push( 0 );
							that.s.aiOuterWidths.push( 0 );
						} else {

							var iWidth = th.outerWidth();

							if ( that.s.aiOuterWidths.length === 0 ) {
								border  = $( that.s.dt.nTable ).css( 'border-left-width' );
								iWidth += typeof border === 'string' && border.indexOf( 'px' ) === -1 ?
								1 :
								parseInt( border, 10 );
							}

							if ( that.s.aiOuterWidths.length === that.s.dt.aoColumns.length - 1 ) {
								border  = $( that.s.dt.nTable ).css( 'border-right-width' );
								iWidth += typeof border === 'string' && border.indexOf( 'px' ) === -1 ?
								1 :
								parseInt( border, 10 );
							}

							that.s.aiOuterWidths.push( iWidth );
							that.s.aiInnerWidths.push( th.width() );

							if ( i < that.s.iLeftColumns ) {
								iLeftWidth += iWidth;
							}

							if ( that.s.iTableColumns - that.s.iRightColumns <= i ) {
								iRightWidth += iWidth;
							}
						}
					}
				);

				this.s.iLeftWidth  = iLeftWidth;
				this.s.iRightWidth = iRightWidth;
			},

			"_fnGridSetup": function ()
		{
				var that      = this;
				var oOverflow = this._fnDTOverflow();
				var block;

				this.dom.body                                        = this.s.dt.nTable;
				this.dom.header                                      = this.s.dt.nTHead.parentNode;
				this.dom.header.parentNode.parentNode.style.position = "relative";

				var nSWrapper =
				$(
					'<div class="DTFC_ScrollWrapper" style="position:relative; clear:both;">' +
					'<div class="DTFC_LeftWrapper" style="position:absolute; top:0; left:0;" aria-hidden="true">' +
						'<div class="DTFC_LeftHeadWrapper" style="position:relative; top:0; left:0; overflow:hidden;"></div>' +
						'<div class="DTFC_LeftBodyWrapper" style="position:relative; top:0; left:0; height:0; overflow:hidden;">' +
							'<div class="DTFC_LeftBodyLiner" style="position:relative; top:0; left:0; overflow-y:scroll;"></div>' +
						'</div>' +
						'<div class="DTFC_LeftFootWrapper" style="position:relative; top:0; left:0; overflow:hidden;"></div>' +
					'</div>' +
					'<div class="DTFC_RightWrapper" style="position:absolute; top:0; right:0;" aria-hidden="true">' +
						'<div class="DTFC_RightHeadWrapper" style="position:relative; top:0; left:0;">' +
							'<div class="DTFC_RightHeadBlocker DTFC_Blocker" style="position:absolute; top:0; bottom:0;"></div>' +
						'</div>' +
						'<div class="DTFC_RightBodyWrapper" style="position:relative; top:0; left:0; height:0; overflow:hidden;">' +
							'<div class="DTFC_RightBodyLiner" style="position:relative; top:0; left:0; overflow-y:scroll;"></div>' +
						'</div>' +
						'<div class="DTFC_RightFootWrapper" style="position:relative; top:0; left:0;">' +
							'<div class="DTFC_RightFootBlocker DTFC_Blocker" style="position:absolute; top:0; bottom:0;"></div>' +
						'</div>' +
					'</div>' +
					'</div>'
				)[0];
				var nLeft     = nSWrapper.childNodes[0];
				var nRight    = nSWrapper.childNodes[1];

				this.dom.grid.dt.parentNode.insertBefore( nSWrapper, this.dom.grid.dt );
				nSWrapper.appendChild( this.dom.grid.dt );

				this.dom.grid.wrapper = nSWrapper;

				if ( this.s.iLeftColumns > 0 ) {
					this.dom.grid.left.wrapper = nLeft;
					this.dom.grid.left.head    = nLeft.childNodes[0];
					this.dom.grid.left.body    = nLeft.childNodes[1];
					this.dom.grid.left.liner   = $( 'div.DTFC_LeftBodyLiner', nSWrapper )[0];

					nSWrapper.appendChild( nLeft );
				}

				if ( this.s.iRightColumns > 0 ) {
					this.dom.grid.right.wrapper = nRight;
					this.dom.grid.right.head    = nRight.childNodes[0];
					this.dom.grid.right.body    = nRight.childNodes[1];
					this.dom.grid.right.liner   = $( 'div.DTFC_RightBodyLiner', nSWrapper )[0];

					nRight.style.right = oOverflow.bar + "px";

					block                         = $( 'div.DTFC_RightHeadBlocker', nSWrapper )[0];
					block.style.width             = oOverflow.bar + "px";
					block.style.right             = -oOverflow.bar + "px";
					this.dom.grid.right.headBlock = block;

					block                         = $( 'div.DTFC_RightFootBlocker', nSWrapper )[0];
					block.style.width             = oOverflow.bar + "px";
					block.style.right             = -oOverflow.bar + "px";
					this.dom.grid.right.footBlock = block;

					nSWrapper.appendChild( nRight );
				}

				if ( this.s.dt.nTFoot ) {
					this.dom.footer = this.s.dt.nTFoot.parentNode;
					if ( this.s.iLeftColumns > 0 ) {
						this.dom.grid.left.foot = nLeft.childNodes[2];
					}
					if ( this.s.iRightColumns > 0 ) {
						this.dom.grid.right.foot = nRight.childNodes[2];
					}
				}

				if ( this.s.rtl ) {
					$( 'div.DTFC_RightHeadBlocker', nSWrapper ).css(
						{
							left: -oOverflow.bar + 'px',
							right: ''
						}
					);
				}
			},

			"_fnGridLayout": function ()
		{
				var that        = this;
				var oGrid       = this.dom.grid;
				var iWidth      = $( oGrid.wrapper ).width();
				var iBodyHeight = this.s.dt.nTable.parentNode.offsetHeight;
				var iFullHeight = this.s.dt.nTable.parentNode.parentNode.offsetHeight;
				var oOverflow   = this._fnDTOverflow();
				var iLeftWidth  = this.s.iLeftWidth;
				var iRightWidth = this.s.iRightWidth;
				var rtl         = $( this.dom.body ).css( 'direction' ) === 'rtl';
				var wrapper;
				var scrollbarAdjust = function ( node, width ) {
					if ( ! oOverflow.bar ) {

						node.style.width        = (width + 20) + "px";
						node.style.paddingRight = "20px";
						node.style.boxSizing    = "border-box";
					} else if ( that._firefoxScrollError() ) {

						if ( $( node ).height() > 34 ) {
							node.style.width = (width + oOverflow.bar) + "px";
						}
					} else {

						node.style.width = (width + oOverflow.bar) + "px";
					}
				};

				if ( oOverflow.x ) {
					iBodyHeight -= oOverflow.bar;
				}

				oGrid.wrapper.style.height = iFullHeight + "px";

				if ( this.s.iLeftColumns > 0 ) {
					wrapper              = oGrid.left.wrapper;
					wrapper.style.width  = iLeftWidth + 'px';
					wrapper.style.height = '1px';

					if ( rtl ) {
						wrapper.style.left  = '';
						wrapper.style.right = 0;
					} else {
						wrapper.style.left  = 0;
						wrapper.style.right = '';
					}

					oGrid.left.body.style.height = iBodyHeight + "px";
					if ( oGrid.left.foot ) {
						oGrid.left.foot.style.top = (oOverflow.x ? oOverflow.bar : 0) + "px";
					}

					scrollbarAdjust( oGrid.left.liner, iLeftWidth );
					oGrid.left.liner.style.height    = iBodyHeight + "px";
					oGrid.left.liner.style.maxHeight = iBodyHeight + "px";
				}

				if ( this.s.iRightColumns > 0 ) {
					wrapper              = oGrid.right.wrapper;
					wrapper.style.width  = iRightWidth + 'px';
					wrapper.style.height = '1px';

					if ( this.s.rtl ) {
						wrapper.style.left  = oOverflow.y ? oOverflow.bar + 'px' : 0;
						wrapper.style.right = '';
					} else {
						wrapper.style.left  = '';
						wrapper.style.right = oOverflow.y ? oOverflow.bar + 'px' : 0;
					}

					oGrid.right.body.style.height = iBodyHeight + "px";
					if ( oGrid.right.foot ) {
						oGrid.right.foot.style.top = (oOverflow.x ? oOverflow.bar : 0) + "px";
					}

					scrollbarAdjust( oGrid.right.liner, iRightWidth );
					oGrid.right.liner.style.height    = iBodyHeight + "px";
					oGrid.right.liner.style.maxHeight = iBodyHeight + "px";

					oGrid.right.headBlock.style.display = oOverflow.y ? 'block' : 'none';
					oGrid.right.footBlock.style.display = oOverflow.y ? 'block' : 'none';
				}
			},

			"_fnDTOverflow": function ()
		{
				var nTable           = this.s.dt.nTable;
				var nTableScrollBody = nTable.parentNode;
				var out              = {
					"x": false,
					"y": false,
					"bar": this.s.dt.oScroll.iBarWidth
				};

				if ( nTable.offsetWidth > nTableScrollBody.clientWidth ) {
					out.x = true;
				}

				if ( nTable.offsetHeight > nTableScrollBody.clientHeight ) {
					out.y = true;
				}

				return out;
			},

			"_fnDraw": function ( bAll )
		{
				this._fnGridLayout();
				this._fnCloneLeft( bAll );
				this._fnCloneRight( bAll );

				if ( this.s.fnDrawCallback !== null ) {
					this.s.fnDrawCallback.call( this, this.dom.clone.left, this.dom.clone.right );
				}

				$( this ).trigger(
					'draw.dtfc',
					{
						"leftClone": this.dom.clone.left,
						"rightClone": this.dom.clone.right
					}
				);
			},

			"_fnCloneRight": function ( bAll )
		{
				if ( this.s.iRightColumns <= 0 ) {
					return;
				}

				var that  = this,
				i, jq,
				aiColumns = [];

				for ( i = this.s.iTableColumns - this.s.iRightColumns; i < this.s.iTableColumns; i++ ) {
					if ( this.s.dt.aoColumns[i].bVisible ) {
						aiColumns.push( i );
					}
				}

				this._fnClone( this.dom.clone.right, this.dom.grid.right, aiColumns, bAll );
			},

			"_fnCloneLeft": function ( bAll )
		{
				if ( this.s.iLeftColumns <= 0 ) {
					return;
				}

				var that  = this,
				i, jq,
				aiColumns = [];

				for ( i = 0; i < this.s.iLeftColumns; i++ ) {
					if ( this.s.dt.aoColumns[i].bVisible ) {
						aiColumns.push( i );
					}
				}

				this._fnClone( this.dom.clone.left, this.dom.grid.left, aiColumns, bAll );
			},

			"_fnCopyLayout": function ( aoOriginal, aiColumns, events )
		{
				var aReturn = [];
				var aClones = [];
				var aCloned = [];

				for ( var i = 0, iLen = aoOriginal.length; i < iLen; i++ ) {
					var aRow = [];
					aRow.nTr = $( aoOriginal[i].nTr ).clone( events, false )[0];

					for ( var j = 0, jLen = this.s.iTableColumns; j < jLen; j++ ) {
						if ( $.inArray( j, aiColumns ) === -1 ) {
							continue;
						}

						var iCloned = $.inArray( aoOriginal[i][j].cell, aCloned );
						if ( iCloned === -1 ) {
							var nClone = $( aoOriginal[i][j].cell ).clone( events, false )[0];
							aClones.push( nClone );
							aCloned.push( aoOriginal[i][j].cell );

							aRow.push(
								{
									"cell": nClone,
									"unique": aoOriginal[i][j].unique
								}
							);
						} else {
							aRow.push(
								{
									"cell": aClones[ iCloned ],
									"unique": aoOriginal[i][j].unique
								}
							);
						}
					}

					aReturn.push( aRow );
				}

				return aReturn;
			},

			"_fnClone": function ( oClone, oGrid, aiColumns, bAll )
		{
				var that = this,
				i, iLen, j, jLen, jq, nTarget, iColumn, nClone, iIndex, aoCloneLayout,
				jqCloneThead, aoFixedHeader,
				dt       = this.s.dt;

				if ( bAll ) {
					$( oClone.header ).remove();

					oClone.header             = $( this.dom.header ).clone( true, false )[0];
					oClone.header.className  += " DTFC_Cloned";
					oClone.header.style.width = "100%";
					oGrid.head.appendChild( oClone.header );

					aoCloneLayout = this._fnCopyLayout( dt.aoHeader, aiColumns, true );
					jqCloneThead  = $( '>thead', oClone.header );
					jqCloneThead.empty();

					for ( i = 0, iLen = aoCloneLayout.length; i < iLen; i++ ) {
						jqCloneThead[0].appendChild( aoCloneLayout[i].nTr );
					}

					dt.oApi._fnDrawHead( dt, aoCloneLayout, true );
				} else {

					aoCloneLayout = this._fnCopyLayout( dt.aoHeader, aiColumns, false );
					aoFixedHeader = [];

					dt.oApi._fnDetectHeader( aoFixedHeader, $( '>thead', oClone.header )[0] );

					for ( i = 0, iLen = aoCloneLayout.length; i < iLen; i++ ) {
						for ( j = 0, jLen = aoCloneLayout[i].length; j < jLen; j++ ) {
							aoFixedHeader[i][j].cell.className = aoCloneLayout[i][j].cell.className;

							$( 'span.DataTables_sort_icon', aoFixedHeader[i][j].cell ).each(
								function () {
									this.className = $( 'span.DataTables_sort_icon', aoCloneLayout[i][j].cell )[0].className;
								}
							);
						}
					}
				}
				this._fnEqualiseHeights( 'thead', this.dom.header, oClone.header );

				if ( this.s.sHeightMatch == 'auto' ) {

					$( '>tbody>tr', that.dom.body ).css( 'height', 'auto' );
				}

				if ( oClone.body !== null ) {
					$( oClone.body ).remove();
					oClone.body = null;
				}

				oClone.body                     = $( this.dom.body ).clone( true )[0];
				oClone.body.className          += " DTFC_Cloned";
				oClone.body.style.paddingBottom = dt.oScroll.iBarWidth + "px";
				oClone.body.style.marginBottom  = (dt.oScroll.iBarWidth * 2) + "px";
				if ( oClone.body.getAttribute( 'id' ) !== null ) {
					oClone.body.removeAttribute( 'id' );
				}

				$( '>thead>tr', oClone.body ).empty();
				$( '>tfoot', oClone.body ).remove();

				var nBody = $( 'tbody', oClone.body )[0];
				$( nBody ).empty();
				if ( dt.aiDisplay.length > 0 ) {

					var nInnerThead = $( '>thead>tr', oClone.body )[0];
					for ( iIndex = 0; iIndex < aiColumns.length; iIndex++ ) {
						iColumn = aiColumns[iIndex];

						nClone           = $( dt.aoColumns[iColumn].nTh ).clone( true )[0];
						nClone.innerHTML = "";

						var oStyle               = nClone.style;
						oStyle.paddingTop        = "0";
						oStyle.paddingBottom     = "0";
						oStyle.borderTopWidth    = "0";
						oStyle.borderBottomWidth = "0";
						oStyle.height            = 0;
						oStyle.width             = that.s.aiInnerWidths[iColumn] + "px";

						nInnerThead.appendChild( nClone );
					}

					$( '>tbody>tr', that.dom.body ).each(
						function (z) {
							var i    = that.s.dt.oFeatures.bServerSide === false ?
							that.s.dt.aiDisplay[ that.s.dt._iDisplayStart + z ] : z;
							var aTds = that.s.dt.aoData[ i ].anCells || $( this ).children( 'td, th' );

							var n = this.cloneNode( false );
							n.removeAttribute( 'id' );
							n.setAttribute( 'data-dt-row', i );

							for ( iIndex = 0; iIndex < aiColumns.length; iIndex++ ) {
								  iColumn = aiColumns[iIndex];

								if ( aTds.length > 0 ) {
									nClone = $( aTds[iColumn] ).clone( true, true )[0];
									nClone.removeAttribute( 'id' );
									nClone.setAttribute( 'data-dt-row', i );
									nClone.setAttribute( 'data-dt-column', iColumn );
									n.appendChild( nClone );
								}
							}
							nBody.appendChild( n );
						}
					);
				} else {
					$( '>tbody>tr', that.dom.body ).each(
						function (z) {
							nClone            = this.cloneNode( true );
							nClone.className += ' DTFC_NoData';
							$( 'td', nClone ).html( '' );
							nBody.appendChild( nClone );
						}
					);
				}

				oClone.body.style.width   = "100%";
				oClone.body.style.margin  = "0";
				oClone.body.style.padding = "0";

				if ( dt.oScroller !== undefined ) {
					var scrollerForcer = dt.oScroller.dom.force;

					if ( ! oGrid.forcer ) {
						oGrid.forcer = scrollerForcer.cloneNode( true );
						oGrid.liner.appendChild( oGrid.forcer );
					} else {
						oGrid.forcer.style.height = scrollerForcer.style.height;
					}
				}

				oGrid.liner.appendChild( oClone.body );

				this._fnEqualiseHeights( 'tbody', that.dom.body, oClone.body );

				if ( dt.nTFoot !== null ) {
					if ( bAll ) {
						if ( oClone.footer !== null ) {
							oClone.footer.parentNode.removeChild( oClone.footer );
						}
						oClone.footer             = $( this.dom.footer ).clone( true, true )[0];
						oClone.footer.className  += " DTFC_Cloned";
						oClone.footer.style.width = "100%";
						oGrid.foot.appendChild( oClone.footer );

						aoCloneLayout    = this._fnCopyLayout( dt.aoFooter, aiColumns, true );
						var jqCloneTfoot = $( '>tfoot', oClone.footer );
						jqCloneTfoot.empty();

						for ( i = 0, iLen = aoCloneLayout.length; i < iLen; i++ ) {
							jqCloneTfoot[0].appendChild( aoCloneLayout[i].nTr );
						}
						dt.oApi._fnDrawHead( dt, aoCloneLayout, true );
					} else {
						aoCloneLayout    = this._fnCopyLayout( dt.aoFooter, aiColumns, false );
						var aoCurrFooter = [];

						dt.oApi._fnDetectHeader( aoCurrFooter, $( '>tfoot', oClone.footer )[0] );

						for ( i = 0, iLen = aoCloneLayout.length; i < iLen; i++ ) {
							for ( j = 0, jLen = aoCloneLayout[i].length; j < jLen; j++ ) {
								aoCurrFooter[i][j].cell.className = aoCloneLayout[i][j].cell.className;
							}
						}
					}
					this._fnEqualiseHeights( 'tfoot', this.dom.footer, oClone.footer );
				}

				var anUnique = dt.oApi._fnGetUniqueThs( dt, $( '>thead', oClone.header )[0] );
				$( anUnique ).each(
					function (i) {
						iColumn          = aiColumns[i];
						this.style.width = that.s.aiInnerWidths[iColumn] + "px";
					}
				);

				if ( that.s.dt.nTFoot !== null ) {
					anUnique = dt.oApi._fnGetUniqueThs( dt, $( '>tfoot', oClone.footer )[0] );
					$( anUnique ).each(
						function (i) {
							iColumn          = aiColumns[i];
							this.style.width = that.s.aiInnerWidths[iColumn] + "px";
						}
					);
				}
			},

			"_fnGetTrNodes": function ( nIn )
		{
				var aOut = [];
				for ( var i = 0, iLen = nIn.childNodes.length; i < iLen; i++ ) {
					if ( nIn.childNodes[i].nodeName.toUpperCase() == "TR" ) {
						aOut.push( nIn.childNodes[i] );
					}
				}
				return aOut;
			},

			"_fnEqualiseHeights": function ( nodeName, original, clone )
		{
				if ( this.s.sHeightMatch == 'none' && nodeName !== 'thead' && nodeName !== 'tfoot' ) {
					return;
				}

				var that     = this,
				i, iLen, iHeight, iHeight2, iHeightOriginal, iHeightClone,
				rootOriginal = original.getElementsByTagName( nodeName )[0],
				rootClone    = clone.getElementsByTagName( nodeName )[0],
				jqBoxHack    = $( '>' + nodeName + '>tr:eq(0)', original ).children( ':first' ),
				iBoxHack     = jqBoxHack.outerHeight() - jqBoxHack.height(),
				anOriginal   = this._fnGetTrNodes( rootOriginal ),
				anClone      = this._fnGetTrNodes( rootClone ),
				heights      = [];

				for ( i = 0, iLen = anClone.length; i < iLen; i++ ) {
					iHeightOriginal = anOriginal[i].offsetHeight;
					iHeightClone    = anClone[i].offsetHeight;
					iHeight         = iHeightClone > iHeightOriginal ? iHeightClone : iHeightOriginal;

					if ( this.s.sHeightMatch == 'semiauto' ) {
						anOriginal[i]._DTTC_iHeight = iHeight;
					}

					heights.push( iHeight );
				}

				for ( i = 0, iLen = anClone.length; i < iLen; i++ ) {
					anClone[i].style.height    = heights[i] + "px";
					anOriginal[i].style.height = heights[i] + "px";
				}
			},

			_firefoxScrollError: function () {
				if ( _firefoxScroll === undefined ) {
					var test = $( '<div/>' )
					.css(
						{
							position: 'absolute',
							top: 0,
							left: 0,
							height: 10,
							width: 50,
							overflow: 'scroll'
						}
					)
						.appendTo( 'body' );

					_firefoxScroll = (
						test[0].clientWidth === test[0].offsetWidth && this._fnDTOverflow().bar !== 0
					);

					test.remove();
				}

				return _firefoxScroll;
			}
		}
	);

	FixedColumns.defaults = {

		"iLeftColumns": 1,

		"iRightColumns": 0,

		"fnDrawCallback": null,

		"sHeightMatch": "semiauto"
	};

	FixedColumns.version = "3.3.1";

	DataTable.Api.register(
		'fixedColumns()',
		function () {
			return this;
		}
	);

	DataTable.Api.register(
		'fixedColumns().update()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._oFixedColumns ) {
						ctx._oFixedColumns.fnUpdate();
					}
				}
			);
		}
	);

	DataTable.Api.register(
		'fixedColumns().relayout()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._oFixedColumns ) {
						ctx._oFixedColumns.fnRedrawLayout();
					}
				}
			);
		}
	);

	DataTable.Api.register(
		'rows().recalcHeight()',
		function () {
			return this.iterator(
				'row',
				function ( ctx, idx ) {
					if ( ctx._oFixedColumns ) {
						ctx._oFixedColumns.fnRecalculateHeight( this.row( idx ).node() );
					}
				}
			);
		}
	);

	DataTable.Api.register(
		'fixedColumns().rowIndex()',
		function ( row ) {
			row = $( row );

			return row.parents( '.DTFC_Cloned' ).length ?
			this.rows( { page: 'current' } ).indexes()[ row.index() ] :
			this.row( row ).index();
		}
	);

	DataTable.Api.register(
		'fixedColumns().cellIndex()',
		function ( cell ) {
			cell = $( cell );

			if ( cell.parents( '.DTFC_Cloned' ).length ) {
				var rowClonedIdx = cell.parent().index();
				var rowIdx       = this.rows( { page: 'current' } ).indexes()[ rowClonedIdx ];
				var columnIdx;

				if ( cell.parents( '.DTFC_LeftWrapper' ).length ) {
					columnIdx = cell.index();
				} else {
					var columns = this.columns().flatten().length;
					columnIdx   = columns - this.context[0]._oFixedColumns.s.iRightColumns + cell.index();
				}

				return {
					row: rowIdx,
					column: this.column.index( 'toData', columnIdx ),
					columnVisible: columnIdx
				};
			} else {
				return this.cell( cell ).index();
			}
		}
	);

	DataTable.Api.registerPlural(
		'cells().fixedNodes()',
		'cell().fixedNode()',
		function () {
			return this.iterator(
				'cell',
				function ( settings, row, column ) {
					return settings._oFixedColumns
					? settings._oFixedColumns.fnToFixedNode( row, column )
					: this.cell( row, column ).node();
				},
				1
			);
		}
	);

	$( document ).on(
		'init.dt.fixedColumns',
		function (e, settings) {
			if ( e.namespace !== 'dt' ) {
				return;
			}

			var init     = settings.oInit.fixedColumns;
			var defaults = DataTable.defaults.fixedColumns;

			if ( init || defaults ) {
				var opts = $.extend( {}, init, defaults );

				if ( init !== false ) {
					new FixedColumns( settings, opts );
				}
			}
		}
	);

	$.fn.dataTable.FixedColumns = FixedColumns;
	$.fn.DataTable.FixedColumns = FixedColumns;

	return FixedColumns;
}));

(function( factory ){
	if ( typeof define === 'function' && define.amd ) {

		define(
			['jquery', 'datatables.net'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {

		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}

			if ( ! $ || ! $.fn.dataTable ) {
				$ = require( 'datatables.net' )( root, $ ).$;
			}

			return factory( $, root, root.document );
		};
	} else {

		factory( jQuery, window, document );
	}
}(function( $, window, document, undefined ) {
	'use strict';
	var DataTable = $.fn.dataTable;

	var _instCounter = 0;

	var FixedHeader = function ( dt, config ) {

		if ( ! (this instanceof FixedHeader) ) {
			throw "FixedHeader must be initialised with the 'new' keyword.";
		}

		if ( config === true ) {
			config = {};
		}

		dt = new DataTable.Api( dt );

		this.c = $.extend( true, {}, FixedHeader.defaults, config );

		this.s = {
			dt: dt,
			position: {
				theadTop: 0,
				tbodyTop: 0,
				tfootTop: 0,
				tfootBottom: 0,
				width: 0,
				left: 0,
				tfootHeight: 0,
				theadHeight: 0,
				windowHeight: $( window ).height(),
				visible: true
			},
			headerMode: null,
			footerMode: null,
			autoWidth: dt.settings()[0].oFeatures.bAutoWidth,
			namespace: '.dtfc' + (_instCounter++),
			scrollLeft: {
				header: -1,
				footer: -1
			},
			enable: true
		};

		this.dom = {
			floatingHeader: null,
			thead: $( dt.table().header() ),
			tbody: $( dt.table().body() ),
			tfoot: $( dt.table().footer() ),
			header: {
				host: null,
				floating: null,
				placeholder: null
			},
			footer: {
				host: null,
				floating: null,
				placeholder: null
			}
		};

		this.dom.header.host = this.dom.thead.parent();
		this.dom.footer.host = this.dom.tfoot.parent();

		var dtSettings = dt.settings()[0];
		if ( dtSettings._fixedHeader ) {
			throw "FixedHeader already initialised on table " + dtSettings.nTable.id;
		}

		dtSettings._fixedHeader = this;

		this._constructor();
	};

	$.extend(
		FixedHeader.prototype,
		{

			destroy: function () {
				this.s.dt.off( '.dtfc' );
				$( window ).off( this.s.namespace );

				if ( this.c.header ) {
					this._modeChange( 'in-place', 'header', true );
				}

				if ( this.c.footer && this.dom.tfoot.length ) {
					this._modeChange( 'in-place', 'footer', true );
				}
			},

			enable: function ( enable, update )
		{
				this.s.enable = enable;

				if ( update || update === undefined ) {
					this._positions();
					this._scroll( true );
				}
			},

			enabled: function ()
		{
				return this.s.enable;
			},

			headerOffset: function ( offset )
		{
				if ( offset !== undefined ) {
					this.c.headerOffset = offset;
					this.update();
				}

				return this.c.headerOffset;
			},

			footerOffset: function ( offset )
		{
				if ( offset !== undefined ) {
					this.c.footerOffset = offset;
					this.update();
				}

				return this.c.footerOffset;
			},

			update: function ()
		{
				var table = this.s.dt.table().node();

				if ( $( table ).is( ':visible' ) ) {
					this.enable( true, false );
				} else {
					this.enable( false, false );
				}

				this._positions();
				this._scroll( true );
			},

			_constructor: function ()
		{
				var that = this;
				var dt   = this.s.dt;

				$( window )
				.on(
					'scroll' + this.s.namespace,
					function () {
						that._scroll();
					}
				)
				.on(
					'resize' + this.s.namespace,
					DataTable.util.throttle(
						function () {
							that.s.position.windowHeight = $( window ).height();
							that.update();
						},
						50
					)
				);

				var autoHeader = $( '.fh-fixedHeader' );
				if ( ! this.c.headerOffset && autoHeader.length ) {
					this.c.headerOffset = autoHeader.outerHeight();
				}

				var autoFooter = $( '.fh-fixedFooter' );
				if ( ! this.c.footerOffset && autoFooter.length ) {
					this.c.footerOffset = autoFooter.outerHeight();
				}

				dt.on(
					'column-reorder.dt.dtfc column-visibility.dt.dtfc draw.dt.dtfc column-sizing.dt.dtfc responsive-display.dt.dtfc',
					function () {
						that.update();
					}
				);

				dt.on(
					'destroy.dtfc',
					function () {
						that.destroy();
					}
				);

				this._positions();
				this._scroll();
			},

			_clone: function ( item, force )
		{
				var dt          = this.s.dt;
				var itemDom     = this.dom[ item ];
				var itemElement = item === 'header' ?
				this.dom.thead :
				this.dom.tfoot;

				if ( ! force && itemDom.floating ) {

					itemDom.floating.removeClass( 'fixedHeader-floating fixedHeader-locked' );
				} else {
					if ( itemDom.floating ) {
						itemDom.placeholder.remove();
						this._unsize( item );
						itemDom.floating.children().detach();
						itemDom.floating.remove();
					}

					itemDom.floating = $( dt.table().node().cloneNode( false ) )
					.css( 'table-layout', 'fixed' )
					.attr( 'aria-hidden', 'true' )
					.removeAttr( 'id' )
					.append( itemElement )
					.appendTo( 'body' );

					itemDom.placeholder = itemElement.clone( false );
					itemDom.placeholder
					.find( '*[id]' )
					.removeAttr( 'id' );

					itemDom.host.prepend( itemDom.placeholder );

					this._matchWidths( itemDom.placeholder, itemDom.floating );
				}
			},

			_matchWidths: function ( from, to ) {
				var get = function ( name ) {
					return $( name, from )
					.map(
						function () {
							return $( this ).width();
						}
					).toArray();
				};

				var set = function ( name, toWidths ) {
					$( name, to ).each(
						function ( i ) {
							$( this ).css(
								{
									width: toWidths[i],
									minWidth: toWidths[i]
								}
							);
						}
					);
				};

				var thWidths = get( 'th' );
				var tdWidths = get( 'td' );

				set( 'th', thWidths );
				set( 'td', tdWidths );
			},

			_unsize: function ( item ) {
				var el = this.dom[ item ].floating;

				if ( el && (item === 'footer' || (item === 'header' && ! this.s.autoWidth)) ) {
					$( 'th, td', el ).css(
						{
							width: '',
							minWidth: ''
						}
					);
				} else if ( el && item === 'header' ) {
					$( 'th, td', el ).css( 'min-width', '' );
				}
			},

			_horizontal: function ( item, scrollLeft )
		{
				var itemDom        = this.dom[ item ];
				var position       = this.s.position;
				var lastScrollLeft = this.s.scrollLeft;

				if ( itemDom.floating && lastScrollLeft[ item ] !== scrollLeft ) {
					itemDom.floating.css( 'left', position.left - scrollLeft );

					lastScrollLeft[ item ] = scrollLeft;
				}
			},

			_modeChange: function ( mode, item, forceChange )
		{
					var dt       = this.s.dt;
					var itemDom  = this.dom[ item ];
					var position = this.s.position;

					var importantWidth = function (w) {
						itemDom.floating.attr(
							'style',
							function(i,s) {
								return (s || '') + 'width: ' + w + 'px !important;';
							}
						);
					};

					var tablePart = this.dom[ item === 'footer' ? 'tfoot' : 'thead' ];
					var focus     = $.contains( tablePart[0], document.activeElement ) ?
					document.activeElement :
					null;

				if ( focus ) {
					focus.blur();
				}

				if ( mode === 'in-place' ) {

					if ( itemDom.placeholder ) {
						itemDom.placeholder.remove();
						itemDom.placeholder = null;
					}

					this._unsize( item );

					if ( item === 'header' ) {
						itemDom.host.prepend( tablePart );
					} else {
						itemDom.host.append( tablePart );
					}

					if ( itemDom.floating ) {
						itemDom.floating.remove();
						itemDom.floating = null;
					}
				} else if ( mode === 'in' ) {

					this._clone( item, forceChange );

					itemDom.floating
					.addClass( 'fixedHeader-floating' )
					.css( item === 'header' ? 'top' : 'bottom', this.c[item + 'Offset'] )
					.css( 'left', position.left + 'px' );

					importantWidth( position.width );

					if ( item === 'footer' ) {
						itemDom.floating.css( 'top', '' );
					}
				} else if ( mode === 'below' ) {

					this._clone( item, forceChange );

					itemDom.floating
					.addClass( 'fixedHeader-locked' )
					.css( 'top', position.tfootTop - position.theadHeight )
					.css( 'left', position.left + 'px' );

					importantWidth( position.width );
				} else if ( mode === 'above' ) {

					this._clone( item, forceChange );

					itemDom.floating
						.addClass( 'fixedHeader-locked' )
						.css( 'top', position.tbodyTop )
						.css( 'left', position.left + 'px' );

					importantWidth( position.width );
				}

				if ( focus && focus !== document.activeElement ) {
					setTimeout(
						function () {
							focus.focus();
						},
						10
					);
				}

					this.s.scrollLeft.header = -1;
					this.s.scrollLeft.footer = -1;
					this.s[item + 'Mode']    = mode;
			},

			_positions: function ()
		{
				var dt        = this.s.dt;
				var table     = dt.table();
				var position  = this.s.position;
				var dom       = this.dom;
				var tableNode = $( table.node() );

				var thead = tableNode.children( 'thead' );
				var tfoot = tableNode.children( 'tfoot' );
				var tbody = dom.tbody;

				position.visible     = tableNode.is( ':visible' );
				position.width       = tableNode.outerWidth();
				position.left        = tableNode.offset().left;
				position.theadTop    = thead.offset().top;
				position.tbodyTop    = tbody.offset().top;
				position.tbodyHeight = tbody.outerHeight();
				position.theadHeight = position.tbodyTop - position.theadTop;

				if ( tfoot.length ) {
					position.tfootTop    = tfoot.offset().top;
					position.tfootBottom = position.tfootTop + tfoot.outerHeight();
					position.tfootHeight = position.tfootBottom - position.tfootTop;
				} else {
					position.tfootTop    = position.tbodyTop + tbody.outerHeight();
					position.tfootBottom = position.tfootTop;
					position.tfootHeight = position.tfootTop;
				}
			},

			_scroll: function ( forceChange )
		{
				var windowTop  = $( document ).scrollTop();
				var windowLeft = $( document ).scrollLeft();
				var position   = this.s.position;
				var headerMode, footerMode;

				if ( this.c.header ) {
					if ( ! this.s.enable ) {
						headerMode = 'in-place';
					} else if ( ! position.visible || windowTop <= position.theadTop - this.c.headerOffset ) {
						headerMode = 'in-place';
					} else if ( windowTop <= position.tfootTop - position.theadHeight - this.c.headerOffset ) {
						headerMode = 'in';
					} else {
						headerMode = 'below';
					}

					if ( forceChange || headerMode !== this.s.headerMode ) {
						this._modeChange( headerMode, 'header', forceChange );
					}

					this._horizontal( 'header', windowLeft );
				}

				if ( this.c.footer && this.dom.tfoot.length ) {
					if ( ! this.s.enable ) {
						footerMode = 'in-place';
					} else if ( ! position.visible || windowTop + position.windowHeight >= position.tfootBottom + this.c.footerOffset ) {
						footerMode = 'in-place';
					} else if ( position.windowHeight + windowTop > position.tbodyTop + position.tfootHeight + this.c.footerOffset ) {
						footerMode = 'in';
					} else {
						footerMode = 'above';
					}

					if ( forceChange || footerMode !== this.s.footerMode ) {
						this._modeChange( footerMode, 'footer', forceChange );
					}

					this._horizontal( 'footer', windowLeft );
				}
			}
		}
	);

	FixedHeader.version = "3.1.7";

	FixedHeader.defaults = {
		header: true,
		footer: false,
		headerOffset: 0,
		footerOffset: 0
	};

	$.fn.dataTable.FixedHeader = FixedHeader;
	$.fn.DataTable.FixedHeader = FixedHeader;

	$( document ).on(
		'init.dt.dtfh',
		function (e, settings, json) {
			if ( e.namespace !== 'dt' ) {
				return;
			}

			var init     = settings.oInit.fixedHeader;
			var defaults = DataTable.defaults.fixedHeader;

			if ( (init || defaults) && ! settings._fixedHeader ) {
				var opts = $.extend( {}, defaults, init );

				if ( init !== false ) {
					new FixedHeader( settings, opts );
				}
			}
		}
	);

	DataTable.Api.register( 'fixedHeader()', function () {} );

	DataTable.Api.register(
		'fixedHeader.adjust()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					var fh = ctx._fixedHeader;

					if ( fh ) {
						fh.update();
					}
				}
			);
		}
	);

	DataTable.Api.register(
		'fixedHeader.enable()',
		function ( flag ) {
			return this.iterator(
				'table',
				function ( ctx ) {
					var fh = ctx._fixedHeader;

					flag = ( flag !== undefined ? flag : true );
					if ( fh && flag !== fh.enabled() ) {
						fh.enable( flag );
					}
				}
			);
		}
	);

	DataTable.Api.register(
		'fixedHeader.enabled()',
		function () {
			if ( this.context.length ) {
				var fh = this.content[0]._fixedHeader;

				if ( fh ) {
					return fh.enabled();
				}
			}

			return false;
		}
	);

	DataTable.Api.register(
		'fixedHeader.disable()',
		function ( ) {
			return this.iterator(
				'table',
				function ( ctx ) {
					var fh = ctx._fixedHeader;

					if ( fh && fh.enabled() ) {
						fh.enable( false );
					}
				}
			);
		}
	);

	$.each(
		['header', 'footer'],
		function ( i, el ) {
			DataTable.Api.register(
				'fixedHeader.' + el + 'Offset()',
				function ( offset ) {
					var ctx = this.context;

					if ( offset === undefined ) {
						return ctx.length && ctx[0]._fixedHeader ?
						ctx[0]._fixedHeader[el + 'Offset']() :
						undefined;
					}

					return this.iterator(
						'table',
						function ( ctx ) {
							var fh = ctx._fixedHeader;

							if ( fh ) {
								fh[ el + 'Offset' ]( offset );
							}
						}
					);
				}
			);
		}
	);

	return FixedHeader;
}));

(function( factory ){
	if ( typeof define === 'function' && define.amd ) {

		define(
			['jquery', 'datatables.net'],
			function ( $ ) {
				return factory( $, window, document );
			}
		);
	} else if ( typeof exports === 'object' ) {

		module.exports = function (root, $) {
			if ( ! root ) {
				root = window;
			}

			if ( ! $ || ! $.fn.dataTable ) {
				$ = require( 'datatables.net' )( root, $ ).$;
			}

			return factory( $, root, root.document );
		};
	} else {

		factory( jQuery, window, document );
	}
}(function( $, window, document, undefined ) {
	'use strict';
	var DataTable = $.fn.dataTable;

	var Responsive = function ( settings, opts ) {

		if ( ! DataTable.versionCheck || ! DataTable.versionCheck( '1.10.10' ) ) {
			throw 'DataTables Responsive requires DataTables 1.10.10 or newer';
		}

		this.s = {
			dt: new DataTable.Api( settings ),
			columns: [],
			current: []
		};

		if ( this.s.dt.settings()[0].responsive ) {
			return;
		}

		if ( opts && typeof opts.details === 'string' ) {
			opts.details = { type: opts.details };
		} else if ( opts && opts.details === false ) {
			opts.details = { type: false };
		} else if ( opts && opts.details === true ) {
			opts.details = { type: 'inline' };
		}

		this.c              = $.extend( true, {}, Responsive.defaults, DataTable.defaults.responsive, opts );
		settings.responsive = this;
		this._constructor();
	};

	$.extend(
		Responsive.prototype,
		{

			_constructor: function ()
		{
				var that              = this;
				var dt                = this.s.dt;
				var dtPrivateSettings = dt.settings()[0];
				var oldWindowWidth    = $( window ).innerWidth();

				dt.settings()[0]._responsive = this;

				$( window ).on(
					'resize.dtr orientationchange.dtr',
					DataTable.util.throttle(
						function () {

							var width = $( window ).innerWidth();

							if ( width !== oldWindowWidth ) {
								that._resize();
								oldWindowWidth = width;
							}
						}
					)
				);

				dtPrivateSettings.oApi._fnCallbackReg(
					dtPrivateSettings,
					'aoRowCreatedCallback',
					function (tr, data, idx) {
						if ( $.inArray( false, that.s.current ) !== -1 ) {
							$( '>td, >th', tr ).each(
								function ( i ) {
									var idx = dt.column.index( 'toData', i );

									if ( that.s.current[idx] === false ) {
										$( this ).css( 'display', 'none' );
									}
								}
							);
						}
					}
				);

				dt.on(
					'destroy.dtr',
					function () {
						dt.off( '.dtr' );
						$( dt.table().body() ).off( '.dtr' );
						$( window ).off( 'resize.dtr orientationchange.dtr' );
						dt.cells( '.dtr-control' ).nodes().to$().removeClass( 'dtr-control' );

						$.each(
							that.s.current,
							function ( i, val ) {
								if ( val === false ) {
									that._setColumnVis( i, true );
								}
							}
						);
					}
				);

				this.c.breakpoints.sort(
					function (a, b) {
						return a.width < b.width ? 1 :
						a.width > b.width ? -1 : 0;
					}
				);

				this._classLogic();
				this._resizeAuto();

				var details = this.c.details;

				if ( details.type !== false ) {
					that._detailsInit();

					dt.on(
						'column-visibility.dtr',
						function () {

							if ( that._timer ) {
								clearTimeout( that._timer );
							}

							that._timer = setTimeout(
								function () {
									that._timer = null;

									that._classLogic();
									that._resizeAuto();
									that._resize();

									that._redrawChildren();
								},
								100
							);
						}
					);

					dt.on(
						'draw.dtr',
						function () {
							that._redrawChildren();
						}
					);

					$( dt.table().node() ).addClass( 'dtr-' + details.type );
				}

				dt.on(
					'column-reorder.dtr',
					function (e, settings, details) {
						that._classLogic();
						that._resizeAuto();
						that._resize( true );
					}
				);

				dt.on(
					'column-sizing.dtr',
					function () {
						that._resizeAuto();
						that._resize();
					}
				);

				dt.on(
					'preXhr.dtr',
					function () {
						var rowIds = [];
						dt.rows().every(
							function () {
								if ( this.child.isShown() ) {
									rowIds.push( this.id( true ) );
								}
							}
						);

						dt.one(
							'draw.dtr',
							function () {
								that._resizeAuto();
								that._resize();

								dt.rows( rowIds ).every(
									function () {
										that._detailsDisplay( this, false );
									}
								);
							}
						);
					}
				);

				dt
				.on(
					'draw.dtr',
					function () {
						that._controlClass();
					}
				)
				.on(
					'init.dtr',
					function (e, settings, details) {
						if ( e.namespace !== 'dt' ) {
							return;
						}

						that._resizeAuto();
						that._resize();

						if ( $.inArray( false, that.s.current ) ) {
							dt.columns.adjust();
						}
					}
				);

				this._resize();
			},

			_columnsVisiblity: function ( breakpoint )
		{
				var dt      = this.s.dt;
				var columns = this.s.columns;
				var i, ien;

				var order = columns
				.map(
					function ( col, idx ) {
						return {
							columnIdx: idx,
							priority: col.priority
						};
					}
				)
				.sort(
					function ( a, b ) {
						if ( a.priority !== b.priority ) {
							  return a.priority - b.priority;
						}
						return a.columnIdx - b.columnIdx;
					}
				);

				var display = $.map(
					columns,
					function ( col, i ) {
						if ( dt.column( i ).visible() === false ) {
							return 'not-visible';
						}
						return col.auto && col.minWidth === null ?
						false :
						col.auto === true ?
						'-' :
							$.inArray( breakpoint, col.includeIn ) !== -1;
					}
				);

				var requiredWidth = 0;
				for ( i = 0, ien = display.length; i < ien; i++ ) {
					if ( display[i] === true ) {
						requiredWidth += columns[i].minWidth;
					}
				}

				var scrolling      = dt.settings()[0].oScroll;
				var bar            = scrolling.sY || scrolling.sX ? scrolling.iBarWidth : 0;
				var widthAvailable = dt.table().container().offsetWidth - bar;
				var usedWidth      = widthAvailable - requiredWidth;

				for ( i = 0, ien = display.length; i < ien; i++ ) {
					if ( columns[i].control ) {
						usedWidth -= columns[i].minWidth;
					}
				}

				var empty = false;
				for ( i = 0, ien = order.length; i < ien; i++ ) {
					var colIdx = order[i].columnIdx;

					if ( display[colIdx] === '-' && ! columns[colIdx].control && columns[colIdx].minWidth ) {

						if ( empty || usedWidth - columns[colIdx].minWidth < 0 ) {
							empty           = true;
							display[colIdx] = false;
						} else {
							display[colIdx] = true;
						}

						usedWidth -= columns[colIdx].minWidth;
					}
				}

				var showControl = false;

				for ( i = 0, ien = columns.length; i < ien; i++ ) {
					if ( ! columns[i].control && ! columns[i].never && display[i] === false ) {
						showControl = true;
						break;
					}
				}

				for ( i = 0, ien = columns.length; i < ien; i++ ) {
					if ( columns[i].control ) {
						display[i] = showControl;
					}

					if ( display[i] === 'not-visible' ) {
						display[i] = false;
					}
				}

				if ( $.inArray( true, display ) === -1 ) {
					display[0] = true;
				}

				return display;
			},

			_classLogic: function ()
		{
					var that        = this;
					var calc        = {};
					var breakpoints = this.c.breakpoints;
					var dt          = this.s.dt;
					var columns     = dt.columns().eq( 0 ).map(
						function (i) {
							var column       = this.column( i );
							var className    = column.header().className;
							var priority     = dt.settings()[0].aoColumns[i].responsivePriority;
							var dataPriority = column.header().getAttribute( 'data-priority' );

							if ( priority === undefined ) {
								  priority = dataPriority === undefined || dataPriority === null ?
								 10000 :
								 dataPriority * 1;
							}

							return {
								className: className,
								includeIn: [],
								auto:      false,
								control:   false,
								never:     className.match( /\bnever\b/ ) ? true : false,
								priority:  priority
							};
						}
					);

					var add = function ( colIdx, name ) {
						var includeIn = columns[ colIdx ].includeIn;

						if ( $.inArray( name, includeIn ) === -1 ) {
							includeIn.push( name );
						}
					};

					var column = function ( colIdx, name, operator, matched ) {
						var size, i, ien;

						if ( ! operator ) {
							columns[ colIdx ].includeIn.push( name );
						} else if ( operator === 'max-' ) {

							size = that._find( name ).width;

							for ( i = 0, ien = breakpoints.length; i < ien; i++ ) {
								if ( breakpoints[i].width <= size ) {
									add( colIdx, breakpoints[i].name );
								}
							}
						} else if ( operator === 'min-' ) {

							size = that._find( name ).width;

							for ( i = 0, ien = breakpoints.length; i < ien; i++ ) {
								if ( breakpoints[i].width >= size ) {
									add( colIdx, breakpoints[i].name );
								}
							}
						} else if ( operator === 'not-' ) {

							for ( i = 0, ien = breakpoints.length; i < ien; i++ ) {
								if ( breakpoints[i].name.indexOf( matched ) === -1 ) {
									add( colIdx, breakpoints[i].name );
								}
							}
						}
					};

					columns.each(
						function ( col, i ) {
							var classNames = col.className.split( ' ' );
							var hasClass   = false;

							for ( var k = 0, ken = classNames.length; k < ken; k++ ) {
								  var className = $.trim( classNames[k] );

								if ( className === 'all' ) {

									hasClass      = true;
									col.includeIn = $.map(
										breakpoints,
										function (a) {
											return a.name;
										}
									);
									return;
								} else if ( className === 'none' || col.never ) {

									hasClass = true;
									return;
								} else if ( className === 'control' ) {

									hasClass    = true;
									col.control = true;
									return;
								}

								$.each(
									breakpoints,
									function ( j, breakpoint ) {

										var brokenPoint = breakpoint.name.split( '-' );
										var re          = new RegExp( '(min\\-|max\\-|not\\-)?(' + brokenPoint[0] + ')(\\-[_a-zA-Z0-9])?' );
										var match       = className.match( re );

										if ( match ) {
											hasClass = true;

											if ( match[2] === brokenPoint[0] && match[3] === '-' + brokenPoint[1] ) {

												column( i, breakpoint.name, match[1], match[2] + match[3] );
											} else if ( match[2] === brokenPoint[0] && ! match[3] ) {

												column( i, breakpoint.name, match[1], match[2] );
											}
										}
									}
								);
							}

							if ( ! hasClass ) {
								col.auto = true;
							}
						}
					);

					this.s.columns = columns;
			},

			_controlClass: function ()
		{
				if ( this.c.details.type === 'inline' ) {
					var dt           = this.s.dt;
					var columnsVis   = this.s.current;
					var firstVisible = $.inArray( true, columnsVis );

					dt.cells(
						null,
						function(idx) {
							return idx !== firstVisible;
						},
						{page: 'current'}
					)
						.nodes()
						.to$()
						.filter( '.dtr-control' )
						.removeClass( 'dtr-control' );

					dt.cells( null, firstVisible, {page: 'current'} )
						.nodes()
						.to$()
						.addClass( 'dtr-control' );
				}
			},

			_detailsDisplay: function ( row, update )
		{
				var that    = this;
				var dt      = this.s.dt;
				var details = this.c.details;

				if ( details && details.type !== false ) {
					var res = details.display(
						row,
						update,
						function () {
							return details.renderer(
								dt,
								row[0],
								that._detailsObj( row[0] )
							);
						}
					);

					if ( res === true || res === false ) {
						$( dt.table().node() ).triggerHandler( 'responsive-display.dt', [dt, row, res, update] );
					}
				}
			},

			_detailsInit: function ()
		{
				var that    = this;
				var dt      = this.s.dt;
				var details = this.c.details;

				if ( details.type === 'inline' ) {
					details.target = 'td.dtr-control, th.dtr-control';
				}

				dt.on(
					'draw.dtr',
					function () {
						that._tabIndexes();
					}
				);
				that._tabIndexes();

				$( dt.table().body() ).on(
					'keyup.dtr',
					'td, th',
					function (e) {
						if ( e.keyCode === 13 && $( this ).data( 'dtr-keyboard' ) ) {
							$( this ).click();
						}
					}
				);

				var target   = details.target;
				var selector = typeof target === 'string' ? target : 'td, th';

				if ( target !== undefined || target !== null ) {

					$( dt.table().body() )
					.on(
						'click.dtr mousedown.dtr mouseup.dtr',
						selector,
						function (e) {

							if ( ! $( dt.table().node() ).hasClass( 'collapsed' ) ) {
								return;
							}

							if ( $.inArray( $( this ).closest( 'tr' ).get( 0 ), dt.rows().nodes().toArray() ) === -1 ) {
								return;
							}

							if ( typeof target === 'number' ) {
								var targetIdx = target < 0 ?
								dt.columns().eq( 0 ).length + target :
								target;

								if ( dt.cell( this ).index().column !== targetIdx ) {
									return;
								}
							}

							var row = dt.row( $( this ).closest( 'tr' ) );

							if ( e.type === 'click' ) {

								that._detailsDisplay( row, false );
							} else if ( e.type === 'mousedown' ) {

								$( this ).css( 'outline', 'none' );
							} else if ( e.type === 'mouseup' ) {

								$( this ).trigger( 'blur' ).css( 'outline', '' );
							}
						}
					);
				}
			},

			_detailsObj: function ( rowIdx )
		{
				var that = this;
				var dt   = this.s.dt;

				return $.map(
					this.s.columns,
					function( col, i ) {

						if ( col.never || col.control ) {
							return;
						}

						var dtCol = dt.settings()[0].aoColumns[ i ];

						return {
							className:   dtCol.sClass,
							columnIndex: i,
							data:        dt.cell( rowIdx, i ).render( that.c.orthogonal ),
							hidden:      dt.column( i ).visible() && ! that.s.current[ i ],
							rowIndex:    rowIdx,
							title:       dtCol.sTitle !== null ?
							dtCol.sTitle :
							$( dt.column( i ).header() ).text()
						};
					}
				);
			},

			_find: function ( name )
		{
				var breakpoints = this.c.breakpoints;

				for ( var i = 0, ien = breakpoints.length; i < ien; i++ ) {
					if ( breakpoints[i].name === name ) {
						return breakpoints[i];
					}
				}
			},

			_redrawChildren: function ()
		{
				var that = this;
				var dt   = this.s.dt;

				dt.rows( {page: 'current'} ).iterator(
					'row',
					function ( settings, idx ) {
						var row = dt.row( idx );

						that._detailsDisplay( dt.row( idx ), true );
					}
				);
			},

			_resize: function (forceRedraw)
		{
				var that        = this;
				var dt          = this.s.dt;
				var width       = $( window ).innerWidth();
				var breakpoints = this.c.breakpoints;
				var breakpoint  = breakpoints[0].name;
				var columns     = this.s.columns;
				var i, ien;
				var oldVis = this.s.current.slice();

				for ( i = breakpoints.length - 1; i >= 0; i-- ) {
					if ( width <= breakpoints[i].width ) {
						breakpoint = breakpoints[i].name;
						break;
					}
				}

				var columnsVis = this._columnsVisiblity( breakpoint );
				this.s.current = columnsVis;

				var collapsedClass = false;

				for ( i = 0, ien = columns.length; i < ien; i++ ) {
					if ( columnsVis[i] === false && ! columns[i].never && ! columns[i].control && ! dt.column( i ).visible() === false ) {
						collapsedClass = true;
						break;
					}
				}

				$( dt.table().node() ).toggleClass( 'collapsed', collapsedClass );

				var changed = false;
				var visible = 0;

				dt.columns().eq( 0 ).each(
					function ( colIdx, i ) {
						if ( columnsVis[i] === true ) {
							  visible++;
						}

						if ( forceRedraw || columnsVis[i] !== oldVis[i] ) {
							changed = true;
							that._setColumnVis( colIdx, columnsVis[i] );
						}
					}
				);

				if ( changed ) {
					this._redrawChildren();

					$( dt.table().node() ).trigger( 'responsive-resize.dt', [dt, this.s.current] );

					if ( dt.page.info().recordsDisplay === 0 ) {
						$( 'td', dt.table().body() ).eq( 0 ).attr( 'colspan', visible );
					}
				}
			},

			_resizeAuto: function ()
		{
				var dt      = this.s.dt;
				var columns = this.s.columns;

				if ( ! this.c.auto ) {
					return;
				}

				if ( $.inArray( true, $.map( columns, function (c) { return c.auto; } ) ) === -1 ) {
					return;
				}

				if ( ! $.isEmptyObject( _childNodeStore ) ) {
					$.each(
						_childNodeStore,
						function ( key ) {
							var idx = key.split( '-' );

							_childNodesRestore( dt, idx[0] * 1, idx[1] * 1 );
						}
					);
				}

				var tableWidth   = dt.table().node().offsetWidth;
				var columnWidths = dt.columns;
				var clonedTable  = dt.table().node().cloneNode( false );
				var clonedHeader = $( dt.table().header().cloneNode( false ) ).appendTo( clonedTable );
				var clonedBody   = $( dt.table().body() ).clone( false, false ).empty().appendTo( clonedTable );

				clonedTable.style.width = 'auto';

				var headerCells = dt.columns()
				.header()
				.filter(
					function (idx) {
						return dt.column( idx ).visible();
					}
				)
				.to$()
				.clone( false )
				.css( 'display', 'table-cell' )
				.css( 'width', 'auto' )
				.css( 'min-width', 0 );

				$( clonedBody )
				.append( $( dt.rows( { page: 'current' } ).nodes() ).clone( false ) )
				.find( 'th, td' ).css( 'display', '' );

				var footer = dt.table().footer();
				if ( footer ) {
					var clonedFooter = $( footer.cloneNode( false ) ).appendTo( clonedTable );
					var footerCells  = dt.columns()
					.footer()
					.filter(
						function (idx) {
							return dt.column( idx ).visible();
						}
					)
						.to$()
						.clone( false )
						.css( 'display', 'table-cell' );

					$( '<tr/>' )
						.append( footerCells )
						.appendTo( clonedFooter );
				}

				$( '<tr/>' )
				.append( headerCells )
				.appendTo( clonedHeader );

				if ( this.c.details.type === 'inline' ) {
					$( clonedTable ).addClass( 'dtr-inline collapsed' );
				}

				$( clonedTable ).find( '[name]' ).removeAttr( 'name' );

				$( clonedTable ).css( 'position', 'relative' )

				var inserted = $( '<div/>' )
				.css(
					{
						width: 1,
						height: 1,
						overflow: 'hidden',
						clear: 'both'
						}
				)
				.append( clonedTable );

				inserted.insertBefore( dt.table().node() );

				headerCells.each(
					function (i) {
						var idx                 = dt.column.index( 'fromVisible', i );
						columns[ idx ].minWidth = this.offsetWidth || 0;
					}
				);

				inserted.remove();
			},

			_responsiveOnlyHidden: function ()
		{
				var dt = this.s.dt;

				return $.map(
					this.s.current,
					function (v, i) {

						if ( dt.column( i ).visible() === false ) {
							return true;
						}
						return v;
					}
				);
			},

			_setColumnVis: function ( col, showHide )
		{
				var dt      = this.s.dt;
				var display = showHide ? '' : 'none';

				$( dt.column( col ).header() ).css( 'display', display );
				$( dt.column( col ).footer() ).css( 'display', display );
				dt.column( col ).nodes().to$().css( 'display', display );

				if ( ! $.isEmptyObject( _childNodeStore ) ) {
					dt.cells( null, col ).indexes().each(
						function (idx) {
							_childNodesRestore( dt, idx.row, idx.column );
						}
					);
				}
			},

			_tabIndexes: function ()
		{
				var dt     = this.s.dt;
				var cells  = dt.cells( { page: 'current' } ).nodes().to$();
				var ctx    = dt.settings()[0];
				var target = this.c.details.target;

				cells.filter( '[data-dtr-keyboard]' ).removeData( '[data-dtr-keyboard]' );

				if ( typeof target === 'number' ) {
					dt.cells( null, target, { page: 'current' } ).nodes().to$()
					.attr( 'tabIndex', ctx.iTabIndex )
					.data( 'dtr-keyboard', 1 );
				} else {

					if ( target === 'td:first-child, th:first-child' ) {
						target = '>td:first-child, >th:first-child';
					}

					$( target, dt.rows( { page: 'current' } ).nodes() )
					.attr( 'tabIndex', ctx.iTabIndex )
					.data( 'dtr-keyboard', 1 );
				}
			}
		}
	);

	Responsive.breakpoints = [
	{ name: 'desktop',  width: Infinity },
	{ name: 'tablet-l', width: 1024 },
	{ name: 'tablet-p', width: 768 },
	{ name: 'mobile-l', width: 480 },
	{ name: 'mobile-p', width: 320 }
	];

	Responsive.display = {
		childRow: function ( row, update, render ) {
			if ( update ) {
				if ( $( row.node() ).hasClass( 'parent' ) ) {
					row.child( render(), 'child' ).show();

					return true;
				}
			} else {
				if ( ! row.child.isShown()  ) {
					row.child( render(), 'child' ).show();
					$( row.node() ).addClass( 'parent' );

					return true;
				} else {
					row.child( false );
					$( row.node() ).removeClass( 'parent' );

					return false;
				}
			}
		},

		childRowImmediate: function ( row, update, render ) {
			if ( ( ! update && row.child.isShown()) || ! row.responsive.hasHidden() ) {

				row.child( false );
				$( row.node() ).removeClass( 'parent' );

				return false;
			} else {

				row.child( render(), 'child' ).show();
				$( row.node() ).addClass( 'parent' );

				return true;
			}
		},

		modal: function ( options ) {
			return function ( row, update, render ) {
				if ( ! update ) {

					var close = function () {
						modal.remove();
						$( document ).off( 'keypress.dtr' );
					};

					var modal = $( '<div class="dtr-modal"/>' )
					.append(
						$( '<div class="dtr-modal-display"/>' )
						.append(
							$( '<div class="dtr-modal-content"/>' )
							.append( render() )
						)
						.append(
							$( '<div class="dtr-modal-close">&times;</div>' )
							.click(
								function () {
									close();
								}
							)
						)
					)
						.append(
							$( '<div class="dtr-modal-background"/>' )
							.click(
								function () {
									close();
								}
							)
						)
						.appendTo( 'body' );

					$( document ).on(
						'keyup.dtr',
						function (e) {
							if ( e.keyCode === 27 ) {
								e.stopPropagation();

								close();
							}
						}
					);
				} else {
					$( 'div.dtr-modal-content' )
					.empty()
					.append( render() );
				}

				if ( options && options.header ) {
					$( 'div.dtr-modal-content' ).prepend(
						'<h2>' + options.header( row ) + '</h2>'
					);
				}
			};
		}
	};

	var _childNodeStore = {};

	function _childNodes( dt, row, col ) {
		var name = row + '-' + col;

		if ( _childNodeStore[ name ] ) {
			return _childNodeStore[ name ];
		}

		var nodes    = [];
		var children = dt.cell( row, col ).node().childNodes;
		for ( var i = 0, ien = children.length; i < ien; i++ ) {
			nodes.push( children[i] );
		}

		_childNodeStore[ name ] = nodes;

		return nodes;
	}

	function _childNodesRestore( dt, row, col ) {
		var name = row + '-' + col;

		if ( ! _childNodeStore[ name ] ) {
			return;
		}

		var node           = dt.cell( row, col ).node();
		var store          = _childNodeStore[ name ];
		var parent         = store[0].parentNode;
		var parentChildren = parent.childNodes;
		var a              = [];

		for ( var i = 0, ien = parentChildren.length; i < ien; i++ ) {
			a.push( parentChildren[i] );
		}

		for ( var j = 0, jen = a.length; j < jen; j++ ) {
			node.appendChild( a[j] );
		}

		_childNodeStore[ name ] = undefined;
	}

	Responsive.renderer = {
		listHiddenNodes: function () {
			return function ( api, rowIdx, columns ) {
				var ul    = $( '<ul data-dtr-index="' + rowIdx + '" class="dtr-details"/>' );
				var found = false;

				var data = $.each(
					columns,
					function ( i, col ) {
						if ( col.hidden ) {
							var klass = col.className ?
							'class="' + col.className + '"' :
							'';

							$(
								'<li ' + klass + ' data-dtr-index="' + col.columnIndex + '" data-dt-row="' + col.rowIndex + '" data-dt-column="' + col.columnIndex + '">' +
								'<span class="dtr-title">' +
									col.title +
								'</span> ' +
								'</li>'
							)
								.append( $( '<span class="dtr-data"/>' ).append( _childNodes( api, col.rowIndex, col.columnIndex ) ) )
								.appendTo( ul );

							found = true;
						}
					}
				);

				return found ?
				ul :
				false;
			};
		},

		listHidden: function () {
			return function ( api, rowIdx, columns ) {
				var data = $.map(
					columns,
					function ( col ) {
						var klass = col.className ?
						'class="' + col.className + '"' :
						'';

						return col.hidden ?
						'<li ' + klass + ' data-dtr-index="' + col.columnIndex + '" data-dt-row="' + col.rowIndex + '" data-dt-column="' + col.columnIndex + '">' +
							'<span class="dtr-title">' +
								col.title +
							'</span> ' +
							'<span class="dtr-data">' +
								col.data +
							'</span>' +
						'</li>' :
						'';
					}
				).join( '' );

				return data ?
				$( '<ul data-dtr-index="' + rowIdx + '" class="dtr-details"/>' ).append( data ) :
				false;
			}
		},

		tableAll: function ( options ) {
			options = $.extend(
				{
					tableClass: ''
				},
				options
			);

			return function ( api, rowIdx, columns ) {
				var data = $.map(
					columns,
					function ( col ) {
						var klass = col.className ?
						'class="' + col.className + '"' :
						'';

						return '<tr ' + klass + ' data-dt-row="' + col.rowIndex + '" data-dt-column="' + col.columnIndex + '">' +
							'<td>' + col.title + ':' + '</td> ' +
							'<td>' + col.data + '</td>' +
						'</tr>';
					}
				).join( '' );

				return $( '<table class="' + options.tableClass + ' dtr-details" width="100%"/>' ).append( data );
			}
		}
	};

	Responsive.defaults = {

		breakpoints: Responsive.breakpoints,

		auto: true,

		details: {
			display: Responsive.display.childRow,

			renderer: Responsive.renderer.listHidden(),

			target: 0,

			type: 'inline'
		},

		orthogonal: 'display'
	};

	var Api = $.fn.dataTable.Api;

	Api.register(
		'responsive()',
		function () {
			return this;
		}
	);

	Api.register(
		'responsive.index()',
		function ( li ) {
			li = $( li );

			return {
				column: li.data( 'dtr-index' ),
				row:    li.parent().data( 'dtr-index' )
			};
		}
	);

	Api.register(
		'responsive.rebuild()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._responsive ) {
						ctx._responsive._classLogic();
					}
				}
			);
		}
	);

	Api.register(
		'responsive.recalc()',
		function () {
			return this.iterator(
				'table',
				function ( ctx ) {
					if ( ctx._responsive ) {
						ctx._responsive._resizeAuto();
						ctx._responsive._resize();
					}
				}
			);
		}
	);

	Api.register(
		'responsive.hasHidden()',
		function () {
			var ctx = this.context[0];

			return ctx._responsive ?
			$.inArray( false, ctx._responsive._responsiveOnlyHidden() ) !== -1 :
			false;
		}
	);

	Api.registerPlural(
		'columns().responsiveHidden()',
		'column().responsiveHidden()',
		function () {
			return this.iterator(
				'column',
				function ( settings, column ) {
					return settings._responsive ?
					settings._responsive._responsiveOnlyHidden()[ column ] :
					false;
				},
				1
			);
		}
	);

	Responsive.version = '2.2.5';

	$.fn.dataTable.Responsive = Responsive;
	$.fn.DataTable.Responsive = Responsive;

	$( document ).on(
		'preInit.dt.dtr',
		function (e, settings, json) {
			if ( e.namespace !== 'dt' ) {
				return;
			}

			if ( $( settings.nTable ).hasClass( 'responsive' ) ||
			 $( settings.nTable ).hasClass( 'dt-responsive' ) ||
			 settings.oInit.responsive ||
			 DataTable.defaults.responsive
			) {
				var init = settings.oInit.responsive;

				if ( init !== false ) {
					new Responsive( settings, $.isPlainObject( init ) ? init : {} );
				}
			}
		}
	);

	return Responsive;
}));
